@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: actions.c 7910 2009-02-18 18:40:36Z charles $
+ * $Id: actions.c 8778 2009-07-03 20:10:05Z charles $
  */
 
 #include <string.h>
@@ -20,6 +20,8 @@
 #include "tr-prefs.h"
 #include "lock.h"
 #include "logo.h"
+#include "options-icon.h"
+#include "turtles.h"
 
 #define UNUSED G_GNUC_UNUSED
 
@@ -44,20 +46,15 @@ action_cb( GtkAction * a,
 
 static GtkRadioActionEntry sort_radio_entries[] =
 {
-    { "sort-by-activity", NULL,      N_( "Sort by _Activity" ),        NULL,
-      NULL, 0 },
-    { "sort-by-name",     NULL,        N_( "Sort by _Name" ),     NULL,
-      NULL, 1 },
-    { "sort-by-progress", NULL,        N_( "Sort by _Progress" ), NULL,
-      NULL, 2 },
-    { "sort-by-ratio",    NULL,        N_( "Sort by _Ratio" ),    NULL,
-      NULL, 3 },
-    { "sort-by-state",    NULL,        N_( "Sort by _State" ),    NULL,
-      NULL, 4 },
-    { "sort-by-tracker",  NULL,        N_( "Sort by _Tracker" ),  NULL,
-      NULL, 5 },
-    { "sort-by-age",      NULL,        N_( "Sort by A_ge" ),      NULL,
-      NULL, 6 }
+    { "sort-by-activity",  NULL, N_( "Sort by _Activity" ),  NULL, NULL, 0 },
+    { "sort-by-name",      NULL, N_( "Sort by _Name" ),      NULL, NULL, 1 },
+    { "sort-by-progress",  NULL, N_( "Sort by _Progress" ),  NULL, NULL, 2 },
+    { "sort-by-ratio",     NULL, N_( "Sort by Rati_o" ),     NULL, NULL, 3 },
+    { "sort-by-state",     NULL, N_( "Sort by Stat_e" ),     NULL, NULL, 4 },
+    { "sort-by-tracker",   NULL, N_( "Sort by T_racker" ),   NULL, NULL, 5 },
+    { "sort-by-age",       NULL, N_( "Sort by A_ge" ),       NULL, NULL, 6 },
+    { "sort-by-time-left", NULL, N_( "Sort by Time _Left" ), NULL, NULL, 7 },
+    { "sort-by-size",      NULL, N_( "Sort by Si_ze" ),      NULL, NULL, 8 }
 };
 
 static void
@@ -74,10 +71,8 @@ sort_changed_cb( GtkAction            * action UNUSED,
 
 static GtkToggleActionEntry show_toggle_entries[] =
 {
-    { "toggle-main-window", NULL,
-      N_( "_Main Window" ), NULL, NULL, G_CALLBACK( action_cb ), TRUE },
-    { "toggle-message-log", NULL,
-      N_( "Message _Log" ), NULL, NULL, G_CALLBACK( action_cb ), FALSE }
+    { "toggle-main-window", NULL, N_( "_Main Window" ), NULL, NULL, G_CALLBACK( action_cb ), TRUE },
+    { "toggle-message-log", NULL, N_( "Message _Log" ), NULL, NULL, G_CALLBACK( action_cb ), FALSE }
 };
 
 static void
@@ -92,21 +87,15 @@ toggle_pref_cb( GtkToggleAction *  action,
 
 static GtkToggleActionEntry  pref_toggle_entries[] =
 {
-    { "minimal-view",          NULL,
-      N_( "_Minimal View" ), "<alt>M", NULL, G_CALLBACK( toggle_pref_cb ),
-      FALSE },
-    { "sort-reversed",         NULL,
-      N_( "_Reverse Sort Order" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ),
-      FALSE },
-    { "show-filterbar",        NULL,
-      N_( "_Filterbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
-    { "show-statusbar",        NULL,
-      N_( "_Statusbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
-    { "show-toolbar",          NULL,
-      N_( "_Toolbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE }
+    { "alt-speed-enabled", NULL, N_( "Temporary  Speed _Limits" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "minimal-view",      NULL, N_( "_Minimal View" ), "<alt>M", NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "sort-reversed",     NULL, N_( "Re_verse Sort Order" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "show-filterbar",    NULL, N_( "_Filterbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "show-statusbar",    NULL, N_( "_Statusbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "show-toolbar",      NULL, N_( "_Toolbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE }
 };
 
-static GtkActionEntry        entries[] =
+static GtkActionEntry entries[] =
 {
     { "torrent-menu", NULL, N_( "_Torrent" ), NULL, NULL, NULL  },
     { "view-menu", NULL, N_( "_View" ), NULL, NULL, NULL  },
@@ -121,6 +110,7 @@ static GtkActionEntry        entries[] =
     { "pause-torrent", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause" ), "<control>P", N_( "Pause torrent" ), G_CALLBACK( action_cb ) },
     { "pause-all-torrents", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause All" ), NULL, N_( "Pause all torrents" ), G_CALLBACK( action_cb ) },
     { "start-all-torrents", GTK_STOCK_MEDIA_PLAY, N_( "_Start All" ), NULL, N_( "Start all torrents" ), G_CALLBACK( action_cb ) },
+    { "relocate-torrent", NULL, N_("Set _Location" ), NULL, NULL, G_CALLBACK( action_cb ) },
     { "remove-torrent", GTK_STOCK_REMOVE, NULL, "Delete", N_( "Remove torrent" ), G_CALLBACK( action_cb ) },
     { "delete-torrent", GTK_STOCK_DELETE, N_( "_Delete Files and Remove" ), "<shift>Delete", NULL, G_CALLBACK( action_cb ) },
     { "new-torrent", GTK_STOCK_NEW, N_( "_New..." ), NULL, N_( "Create a torrent" ), G_CALLBACK( action_cb ) },
@@ -146,7 +136,10 @@ BuiltinIconInfo;
 static const BuiltinIconInfo my_fallback_icons[] =
 {
     { tr_icon_logo, "transmission"      },
-    { tr_icon_lock, "transmission-lock" }
+    { tr_icon_lock, "transmission-lock" },
+    { options_icon, "options"           },
+    { blue_turtle,  "alt-speed-on"      },
+    { grey_turtle,  "alt-speed-off"     }
 };
 
 static void
