@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: conf.c 7664 2009-01-11 17:46:51Z charles $
+ * $Id: conf.c 8673 2009-06-11 22:36:45Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -39,6 +39,7 @@
 #include <libtransmission/platform.h>
 
 #include "conf.h"
+#include "tr-prefs.h"
 #include "util.h"
 
 #define MY_NAME "transmission"
@@ -143,6 +144,67 @@ cf_lock( tr_lockfile_state_t *tr_state,
 ****
 ***/
 
+static void cf_check_older_configs( void );
+
+/**
+ * This is where we initialize the preferences file with the default values.
+ * If you add a new preferences key, you /must/ add a default value here.
+ */
+static void
+tr_prefs_init_defaults( tr_benc * d )
+{
+    const char * str;
+
+    cf_check_older_configs( );
+
+#ifdef HAVE_GIO
+    str = NULL;
+    if( !str ) str = g_get_user_special_dir( G_USER_DIRECTORY_DOWNLOAD );
+    if( !str ) str = g_get_user_special_dir( G_USER_DIRECTORY_DESKTOP );
+    if( !str ) str = tr_getDefaultDownloadDir( );
+    tr_bencDictAddStr( d, PREF_KEY_DIR_WATCH, str );
+    tr_bencDictAddBool( d, PREF_KEY_DIR_WATCH_ENABLED, FALSE );
+#endif
+
+    tr_bencDictAddBool( d, PREF_KEY_INHIBIT_HIBERNATION, FALSE );
+    tr_bencDictAddBool( d, PREF_KEY_BLOCKLIST_UPDATES_ENABLED, TRUE );
+
+    tr_bencDictAddStr( d, PREF_KEY_OPEN_DIALOG_FOLDER, g_get_home_dir( ) );
+
+    tr_bencDictAddBool( d, PREF_KEY_TOOLBAR, TRUE );
+    tr_bencDictAddBool( d, PREF_KEY_FILTERBAR, TRUE );
+    tr_bencDictAddBool( d, PREF_KEY_STATUSBAR, TRUE );
+    tr_bencDictAddBool( d, PREF_KEY_SHOW_TRAY_ICON, FALSE );
+    tr_bencDictAddBool( d, PREF_KEY_SHOW_DESKTOP_NOTIFICATION, TRUE );
+    tr_bencDictAddStr( d, PREF_KEY_STATUSBAR_STATS, "total-ratio" );
+
+    tr_bencDictAddBool( d, PREF_KEY_OPTIONS_PROMPT, TRUE );
+
+    tr_bencDictAddBool( d, PREF_KEY_MAIN_WINDOW_IS_MAXIMIZED, FALSE );
+    tr_bencDictAddInt( d, PREF_KEY_MAIN_WINDOW_HEIGHT, 500 );
+    tr_bencDictAddInt( d, PREF_KEY_MAIN_WINDOW_WIDTH, 300 );
+    tr_bencDictAddInt( d, PREF_KEY_MAIN_WINDOW_X, 50 );
+    tr_bencDictAddInt( d, PREF_KEY_MAIN_WINDOW_Y, 50 );
+    tr_bencDictAddStr( d, PREF_KEY_MAIN_WINDOW_LAYOUT_ORDER, "menu,toolbar,filter,list,statusbar" );
+
+    str = NULL;
+#if GLIB_CHECK_VERSION( 2, 14, 0 )
+    if( !str ) str = g_get_user_special_dir( G_USER_DIRECTORY_DOWNLOAD );
+#endif
+    if( !str ) str = tr_getDefaultDownloadDir( );
+    tr_bencDictAddStr( d, TR_PREFS_KEY_DOWNLOAD_DIR, str );
+
+    tr_bencDictAddBool( d, PREF_KEY_ASKQUIT, TRUE );
+
+    tr_bencDictAddStr( d, PREF_KEY_FILTER_MODE, "show-all" );
+    tr_bencDictAddStr( d, PREF_KEY_SORT_MODE, "sort-by-name" );
+    tr_bencDictAddBool( d, PREF_KEY_SORT_REVERSED, FALSE );
+    tr_bencDictAddBool( d, PREF_KEY_MINIMAL_VIEW, FALSE );
+
+    tr_bencDictAddBool( d, PREF_KEY_START, TRUE );
+    tr_bencDictAddBool( d, PREF_KEY_TRASH_ORIGINAL, FALSE );
+}
+
 static char*
 getPrefsFilename( void )
 {
@@ -159,6 +221,7 @@ getPrefs( void )
     if( !loaded )
     {
         tr_bencInitDict( &settings, 0 );
+        tr_prefs_init_defaults( &settings );
         tr_sessionLoadSettings( &settings, gl_confdir, MY_NAME );
         loaded = TRUE;
     }
@@ -192,12 +255,19 @@ pref_int_set( const char * key,
     tr_bencDictAddInt( getPrefs( ), key, value );
 }
 
-void
-pref_int_set_default( const char * key,
-                      int64_t      value )
+double
+pref_double_get( const char * key )
 {
-    if( !tr_bencDictFind( getPrefs( ), key ) )
-        pref_int_set( key, value );
+    double d = 0.0;
+    tr_bencDictFindReal( getPrefs( ), key, &d );
+    return d;
+}
+
+void
+pref_double_set( const char * key,
+                 double       value )
+{
+    tr_bencDictAddReal( getPrefs( ), key, value );
 }
 
 /***
@@ -207,10 +277,9 @@ pref_int_set_default( const char * key,
 gboolean
 pref_flag_get( const char * key )
 {
-    int64_t i;
-
-    tr_bencDictFindInt( getPrefs( ), key, &i );
-    return i != 0;
+    tr_bool boolVal;
+    tr_bencDictFindBool( getPrefs( ), key, &boolVal );
+    return boolVal != 0;
 }
 
 gboolean
@@ -234,14 +303,7 @@ void
 pref_flag_set( const char * key,
                gboolean     value )
 {
-    pref_int_set( key, value != 0 );
-}
-
-void
-pref_flag_set_default( const char * key,
-                       gboolean     value )
-{
-    pref_int_set_default( key, value != 0 );
+    tr_bencDictAddBool( getPrefs( ), key, value );
 }
 
 /***
@@ -252,26 +314,16 @@ const char*
 pref_string_get( const char * key )
 {
     const char * str = NULL;
-
     tr_bencDictFindStr( getPrefs( ), key, &str );
     return str;
 }
 
 void
-pref_string_set( const char * key,
-                 const char * value )
+pref_string_set( const char * key, const char * value )
 {
     tr_bencDictAddStr( getPrefs( ), key, value );
 }
 
-void
-pref_string_set_default( const char * key,
-                         const char * value )
-{
-    if( !tr_bencDictFind( getPrefs( ), key ) )
-        pref_string_set( key, value );
-}
-
 /***
 ****
 ***/
@@ -305,14 +357,6 @@ tr_file_set_contents( const char *   filename,
  #define g_file_set_contents tr_file_set_contents
 #endif
 
-static char*
-getCompat080PrefsFilename( void )
-{
-    assert( gl_confdir != NULL );
-
-    return g_build_filename( g_get_home_dir( ), ".transmission", "gtk", "prefs", NULL );
-}
-
 static char*
 getCompat090PrefsFilename( void )
 {
@@ -327,63 +371,6 @@ getCompat121PrefsFilename( void )
     return g_build_filename( g_get_user_config_dir( ), "transmission", "gtk", "prefs.ini", NULL );
 }
 
-static void
-translate_08_to_09( const char* oldfile,
-                    const char* newfile )
-{
-    static struct pref_entry {
-        const char*   oldkey;
-        const char*   newkey;
-    } pref_table[] = {
-        { "add-behavior-ipc",       "add-behavior-ipc"               },
-        { "add-behavior-standard",  "add-behavior-standard"          },
-        { "download-directory",     "default-download-directory"     },
-        { "download-limit",         "download-limit"                 },
-        { "use-download-limit",     "download-limit-enabled"         },
-        { "listening-port",         "listening-port"                 },
-        { "use-nat-traversal",      "nat-traversal-enabled"          },
-        { "use-peer-exchange",      "pex-enabled"                    },
-        { "ask-quit",               "prompt-before-exit"             },
-        { "ask-download-directory", "prompt-for-download-directory"  },
-        { "use-tray-icon",          "system-tray-icon-enabled"       },
-        { "upload-limit",           "upload-limit"                   },
-        { "use-upload-limit",       "upload-limit-enabled"           }
-    };
-
-    GString * out = g_string_new( NULL );
-    gchar *   contents = NULL;
-    gsize     contents_len = 0;
-    tr_benc   top;
-
-    memset( &top, 0, sizeof( tr_benc ) );
-
-    if( g_file_get_contents( oldfile, &contents, &contents_len, NULL )
-      && !tr_bencLoad( contents, contents_len, &top, NULL )
-      && top.type == TYPE_DICT )
-    {
-        unsigned int i;
-        g_string_append( out, "\n[general]\n" );
-        for( i = 0; i < G_N_ELEMENTS( pref_table ); ++i )
-        {
-            const tr_benc * val = tr_bencDictFind( &top,
-                                                   pref_table[i].oldkey );
-            if( val != NULL )
-            {
-                const char * valstr = val->val.s.s;
-                if( !strcmp( valstr, "yes" ) ) valstr = "true";
-                if( !strcmp( valstr, "no" ) ) valstr = "false";
-                g_string_append_printf( out, "%s=%s\n",
-                                        pref_table[i].newkey,
-                                        valstr );
-            }
-        }
-    }
-
-    g_file_set_contents( newfile, out->str, out->len, NULL );
-    g_string_free( out, TRUE );
-    g_free( contents );
-}
-
 static void
 translate_keyfile_to_json( const char * old_file,
                            const char * new_file )
@@ -442,11 +429,11 @@ translate_keyfile_to_json( const char * old_file,
     }
 
     g_key_file_free( keyfile );
-    tr_bencSaveJSONFile( new_file, &dict );
+    tr_bencToFile( &dict, TR_FMT_JSON, new_file );
     tr_bencFree( &dict );
 }
 
-void
+static void
 cf_check_older_configs( void )
 {
     char * filename = getPrefsFilename( );
@@ -455,33 +442,21 @@ cf_check_older_configs( void )
     {
         char * key1 = getCompat121PrefsFilename( );
         char * key2 = getCompat090PrefsFilename( );
-        char * benc = getCompat080PrefsFilename( );
 
         if( g_file_test( key1, G_FILE_TEST_IS_REGULAR ) )
         {
+            g_message( _( "Importing \"%s\"" ), key1 );
             translate_keyfile_to_json( key1, filename );
         }
         else if( g_file_test( key2, G_FILE_TEST_IS_REGULAR ) )
         {
+            g_message( _( "Importing \"%s\"" ), key2 );
             translate_keyfile_to_json( key2, filename );
         }
-        else if( g_file_test( benc, G_FILE_TEST_IS_REGULAR ) )
-        {
-            char * tmpfile;
-            int    fd =
-                g_file_open_tmp( "transmission-prefs-XXXXXX", &tmpfile,
-                                 NULL );
-            if( fd != -1 ) close( fd );
-            translate_08_to_09( benc, tmpfile );
-            translate_keyfile_to_json( tmpfile, filename );
-            unlink( tmpfile );
-        }
 
-        g_free( benc );
         g_free( key2 );
         g_free( key1 );
     }
 
     g_free( filename );
 }
-
