@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: main.c 7910 2009-02-18 18:40:36Z charles $
+ * $Id: main.c 8766 2009-07-01 04:10:04Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -35,12 +35,6 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 
-#include <gdk/gdk.h>
-#ifdef GDK_WINDOWING_X11
- #include <X11/Xatom.h>
- #include <gdk/gdkx.h>
-#endif
-
 #include <libtransmission/transmission.h>
 #include <libtransmission/rpcimpl.h>
 #include <libtransmission/utils.h>
@@ -48,7 +42,6 @@
 
 #include "actions.h"
 #include "add-dialog.h"
-#include "blocklist.h"
 #include "conf.h"
 #include "details.h"
 #include "dialogs.h"
@@ -56,6 +49,7 @@
 #include "makemeta-ui.h"
 #include "msgwin.h"
 #include "notify.h"
+#include "relocate.h"
 #include "stats.h"
 #include "tr-core.h"
 #include "tr-icon.h"
@@ -65,65 +59,33 @@
 #include "util.h"
 #include "ui.h"
 
-#include <libtransmission/transmission.h>
-#include <libtransmission/version.h>
-
 #define MY_NAME "transmission"
 
-/* interval in milliseconds to update the torrent list display */
-#define UPDATE_INTERVAL         1666
-
-/* interval in milliseconds to check for stopped torrents and update display */
-#define EXIT_CHECK_INTERVAL     500
+#define REFRESH_INTERVAL_SECONDS 2
 
 #if GTK_CHECK_VERSION( 2, 8, 0 )
  #define SHOW_LICENSE
 static const char * LICENSE =
-    "The Transmission binaries and most of its source code is distributed "
-    "license. "
-    "\n\n"
-    "Some files are copyrighted by Charles Kerr and are covered by "
-    "the GPL version 2.  Works owned by the Transmission project "
-    "are granted a special exemption to clause 2(b) so that the bulk "
-    "of its code can remain under the MIT license.  This exemption does "
-    "not extend to original or derived works not owned by the "
-    "Transmission project. "
-    "\n\n"
-    "Permission is hereby granted, free of charge, to any person obtaining "
-    "a copy of this software and associated documentation files (the "
-    "'Software'), to deal in the Software without restriction, including "
-    "without limitation the rights to use, copy, modify, merge, publish, "
-    "distribute, sublicense, and/or sell copies of the Software, and to "
-    "permit persons to whom the Software is furnished to do so, subject to "
-    "the following conditions: "
-    "\n\n"
-    "The above copyright notice and this permission notice shall be included "
-    "in all copies or substantial portions of the Software. "
-    "\n\n"
-    "THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, "
-    "EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF "
-    "MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. "
-    "IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY "
-    "CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, "
-    "TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE "
-    "SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.";
+"The OS X client, CLI client, and parts of libtransmission are licensed under the terms of the MIT license.\n\n"
+"The Transmission daemon, GTK+ client, Qt client, Web client, and most of libtransmission are licensed under the terms of the GNU GPL version 2, with two special exceptions:\n\n"
+"1. The MIT-licensed portions of Transmission listed above are exempt from GPLv2 clause 2(b) and may retain their MIT license.\n\n"
+"2. Permission is granted to link the code in this release with the OpenSSL project's 'OpenSSL' library and to distribute the linked executables.  Works derived from Transmission may, at their authors' discretion, keep or delete this exception.";
 #endif
 
 struct cbdata
 {
-    unsigned int  isIconified : 1;
-    unsigned int  isClosing   : 1;
-    guint         timer;
-    gpointer      icon;
-    GtkWindow *   wind;
-    TrCore *      core;
-    GtkWidget *   msgwin;
-    GtkWidget *   prefs;
-    GSList *      errqueue;
-    GSList *      dupqueue;
-    GHashTable *  tor2details;
-    GHashTable *  details2tor;
-    GtkTreeSelection * sel;
+    gboolean            isIconified;
+    gboolean            isClosing;
+    guint               timer;
+    gpointer            icon;
+    GtkWindow         * wind;
+    TrCore            * core;
+    GtkWidget         * msgwin;
+    GtkWidget         * prefs;
+    GSList            * errqueue;
+    GSList            * dupqueue;
+    GSList            * details;
+    GtkTreeSelection  * sel;
 };
 
 #define CBDATA_PTR "callback-data-pointer"
@@ -153,10 +115,7 @@ static void           gotdrag( GtkWidget *       widget,
                                guint             time,
                                gpointer          gdata );
 
-static void           coreerr( TrCore *         core,
-                               enum tr_core_err code,
-                               const char *     msg,
-                               gpointer         gdata );
+static void coreerr( TrCore *, guint, const char *, struct cbdata * );
 
 static void           onAddTorrent( TrCore *,
                                     tr_ctor *,
@@ -188,10 +147,10 @@ accumulateStatusForeach( GtkTreeModel *      model,
 
     gtk_tree_model_get( model, iter, MC_ACTIVITY, &activity, -1 );
 
-    if( TR_STATUS_IS_ACTIVE( activity ) )
-        ++counts->activeCount;
-    else
+    if( activity == TR_STATUS_STOPPED )
         ++counts->inactiveCount;
+    else
+        ++counts->activeCount;
 }
 
 static void
@@ -206,7 +165,7 @@ accumulateCanUpdateForeach( GtkTreeModel *      model,
 }
 
 static void
-refreshTorrentActions( struct cbdata * data )
+refreshActions( struct cbdata * data )
 {
     int canUpdate;
     struct counts_data counts;
@@ -222,7 +181,7 @@ refreshTorrentActions( struct cbdata * data )
     action_sensitize( "delete-torrent", counts.totalCount != 0 );
     action_sensitize( "verify-torrent", counts.totalCount != 0 );
     action_sensitize( "open-torrent-folder", counts.totalCount == 1 );
-    action_sensitize( "show-torrent-properties", counts.totalCount == 1 );
+    action_sensitize( "relocate-torrent", counts.totalCount == 1 );
 
     canUpdate = 0;
     gtk_tree_selection_selected_foreach( s, accumulateCanUpdateForeach, &canUpdate );
@@ -245,10 +204,38 @@ refreshTorrentActions( struct cbdata * data )
     }
 }
 
+static void
+refreshDetailsDialog( struct cbdata * data, GtkWidget * details )
+{
+    GtkTreeSelection * s = tr_window_get_selection( data->wind );
+    GtkTreeModel * model;
+    GSList * ids = NULL;
+    GList * selrows = NULL;
+    GList * l;
+
+    /* build a list of the selected torrents' ids */
+    s = tr_window_get_selection( data->wind );
+    for( selrows=l=gtk_tree_selection_get_selected_rows(s,&model); l; l=l->next ) {
+        GtkTreeIter iter;
+        if( gtk_tree_model_get_iter( model, &iter, l->data ) ) {
+            tr_torrent * tor;
+            gtk_tree_model_get( model, &iter, MC_TORRENT_RAW, &tor, -1 );
+            ids = g_slist_append( ids, GINT_TO_POINTER( tr_torrentId( tor ) ) );
+        }
+    }
+
+    torrent_inspector_set_torrents( details, ids );
+
+    /* cleanup */
+    g_slist_free( ids );
+    g_list_foreach( selrows, (GFunc)gtk_tree_path_free, NULL );
+    g_list_free( selrows );
+}
+
 static void
 selectionChangedCB( GtkTreeSelection * s UNUSED, gpointer data )
 {
-    refreshTorrentActions( data );
+    refreshActions( data );
 }
 
 static void
@@ -260,6 +247,8 @@ onMainWindowSizeAllocated( GtkWidget *            window,
                             && ( gdk_window_get_state( window->window )
                                  & GDK_WINDOW_STATE_MAXIMIZED );
 
+    pref_int_set( PREF_KEY_MAIN_WINDOW_IS_MAXIMIZED, isMaximized );
+
     if( !isMaximized )
     {
         int x, y, w, h;
@@ -313,8 +302,7 @@ onRPCChanged( tr_session            * session UNUSED,
     switch( type )
     {
         case TR_RPC_TORRENT_ADDED:
-            tr_core_add_torrent( cbdata->core,
-                                 tr_torrent_new_preexisting( tor ) );
+            tr_core_add_torrent( cbdata->core, tr_torrent_new_preexisting( tor ), TRUE );
             break;
 
         case TR_RPC_TORRENT_STARTED:
@@ -344,7 +332,6 @@ main( int     argc,
       char ** argv )
 {
     char *              err = NULL;
-    struct cbdata *     cbdata;
     GSList *            argfiles;
     GError *            gerr;
     gboolean            didinit = FALSE;
@@ -352,7 +339,7 @@ main( int     argc,
     gboolean            showversion = FALSE;
     gboolean            startpaused = FALSE;
     gboolean            startminimized = FALSE;
-    char *              domain = MY_NAME;
+    const char *        domain = MY_NAME;
     char *              configDir = NULL;
     tr_lockfile_state_t tr_state;
 
@@ -371,10 +358,6 @@ main( int     argc,
         { NULL, 0,   0, 0, NULL, NULL, NULL }
     };
 
-    cbdata = g_new0( struct cbdata, 1 );
-    cbdata->tor2details = g_hash_table_new( g_str_hash, g_str_equal );
-    cbdata->details2tor = g_hash_table_new( g_direct_hash, g_direct_equal );
-
     /* bind the gettext domain */
     setlocale( LC_ALL, "" );
     bindtextdomain( domain, TRANSMISSIONLOCALEDIR );
@@ -387,8 +370,8 @@ main( int     argc,
         g_thread_init( NULL );
 
     gerr = NULL;
-    if( !gtk_init_with_args( &argc, &argv, _( "[torrent files]" ), entries,
-                             domain, &gerr ) )
+    if( !gtk_init_with_args( &argc, &argv, (char*)_( "[torrent files]" ), entries,
+                             (char*)domain, &gerr ) )
     {
         fprintf( stderr, "%s\n", gerr->message );
         g_clear_error( &gerr );
@@ -406,52 +389,83 @@ main( int     argc,
 
     tr_notify_init( );
     didinit = cf_init( configDir, NULL ); /* must come before actions_init */
-    tr_prefs_init_global( );
-
-    myUIManager = gtk_ui_manager_new ( );
-    actions_init ( myUIManager, cbdata );
-    gtk_ui_manager_add_ui_from_string ( myUIManager, fallback_ui_file, -1, NULL );
-    gtk_ui_manager_ensure_update ( myUIManager );
-    gtk_window_set_default_icon_name ( MY_NAME );
 
     setupsighandlers( ); /* set up handlers for fatal signals */
 
-    /* either get a lockfile s.t. this is the one instance of
-     * transmission that's running, OR if there are files to
-     * be added, delegate that to the running instance via dbus */
     didlock = cf_lock( &tr_state, &err );
     argfiles = checkfilenames( argc - 1, argv + 1 );
+
     if( !didlock && argfiles )
     {
+        /* We have torrents to add but there's another copy of Transmsision
+         * running... chances are we've been invoked from a browser, etc.
+         * So send the files over to the "real" copy of Transmission, and
+         * if that goes well, then our work is done. */
         GSList * l;
         gboolean delegated = FALSE;
+
         for( l = argfiles; l; l = l->next )
             delegated |= gtr_dbus_add_torrent( l->data );
-        if( delegated )
-            err = NULL;
+
+        if( delegated ) {
+            g_slist_foreach( argfiles, (GFunc)g_free, NULL );
+            g_slist_free( argfiles );
+            argfiles = NULL;
+
+            if( err ) {
+                g_free( err );
+                err = NULL;
+            }
+        }
     }
     else if( ( !didlock ) && ( tr_state == TR_LOCKFILE_ELOCK ) )
     {
+        /* There's already another copy of Transmission running,
+         * so tell it to present its window to the user */
         gtr_dbus_present_window( );
         err = NULL;
     }
 
     if( didlock && ( didinit || cf_init( configDir, &err ) ) )
     {
+        /* No other copy of Transmission running...
+         * so we're going to be the primary. */
+
+        const char * str;
         GtkWindow * win;
         tr_session * session;
+        struct cbdata * cbdata = g_new0( struct cbdata, 1 );
+
+        /* ensure the directories are created */
+       if(( str = pref_string_get( PREF_KEY_DIR_WATCH )))
+           mkdir_p( str, 0777 );
+       if(( str = pref_string_get( TR_PREFS_KEY_DOWNLOAD_DIR )))
+           mkdir_p( str, 0777 );
 
         /* initialize the libtransmission session */
         session = tr_sessionInit( "gtk", configDir, TRUE, pref_get_all( ) );
+        pref_flag_set( TR_PREFS_KEY_ALT_SPEED_ENABLED, tr_sessionUsesAltSpeed( session ) );
+        pref_int_set( TR_PREFS_KEY_PEER_PORT, tr_sessionGetPeerPort( session ) );
         cbdata->core = tr_core_new( session );
 
+        /* init the ui manager */
+        myUIManager = gtk_ui_manager_new ( );
+        actions_init ( myUIManager, cbdata );
+        gtk_ui_manager_add_ui_from_string ( myUIManager, fallback_ui_file, -1, NULL );
+        gtk_ui_manager_ensure_update ( myUIManager );
+        gtk_window_set_default_icon_name ( MY_NAME );
+
         /* create main window now to be a parent to any error dialogs */
         win = GTK_WINDOW( tr_window_new( myUIManager, cbdata->core ) );
         g_signal_connect( win, "size-allocate", G_CALLBACK( onMainWindowSizeAllocated ), cbdata );
 
         appsetup( win, argfiles, cbdata, startpaused, startminimized );
         tr_sessionSetRPCCallback( session, onRPCChanged, cbdata );
-        gtr_blocklist_maybe_autoupdate( cbdata->core );
+
+        /* on startup, check & see if it's time to update the blocklist */
+        if( pref_flag_get( PREF_KEY_BLOCKLIST_UPDATES_ENABLED )
+            && ( time( NULL ) - pref_int_get( "blocklist-date" ) > ( 60 * 60 * 24 * 7 ) ) )
+                tr_core_blocklist_update( cbdata->core );
 
         gtk_main( );
     }
@@ -466,96 +480,6 @@ main( int     argc,
     return 0;
 }
 
-static gboolean
-updateScheduledLimits( gpointer data )
-{
-    tr_session *    tr = data;
-    static gboolean last_state = FALSE;
-    gboolean        in_sched_state = FALSE;
-
-    if( !pref_flag_get( PREF_KEY_SCHED_LIMIT_ENABLED ) )
-    {
-        in_sched_state = FALSE;
-    }
-    else
-    {
-        const int  begin_time = pref_int_get( PREF_KEY_SCHED_BEGIN );
-        const int  end_time = pref_int_get( PREF_KEY_SCHED_END );
-        time_t     t;
-        struct tm *tm;
-        int        cur_time;
-
-        time( &t );
-        tm = localtime ( &t );
-        cur_time = ( tm->tm_hour * 60 ) + tm->tm_min;
-
-        if( end_time >= begin_time )
-        {
-            if( ( cur_time >= begin_time ) && ( cur_time <= end_time ) )
-                in_sched_state = TRUE;
-        }
-        else
-        {
-            if( ( cur_time >= begin_time ) || ( cur_time <= end_time ) )
-                in_sched_state = TRUE;
-        }
-    }
-
-    if( last_state != in_sched_state )
-    {
-        if( in_sched_state )
-        {
-            int limit;
-
-            tr_inf ( _( "Beginning to use scheduled bandwidth limits" ) );
-
-            tr_sessionSetSpeedLimitEnabled( tr, TR_DOWN, TRUE );
-            limit = pref_int_get( PREF_KEY_SCHED_DL_LIMIT );
-            tr_sessionSetSpeedLimit( tr, TR_DOWN, limit );
-            tr_sessionSetSpeedLimitEnabled( tr, TR_UP, TRUE );
-            limit = pref_int_get( PREF_KEY_SCHED_UL_LIMIT );
-            tr_sessionSetSpeedLimit( tr, TR_UP, limit );
-        }
-        else
-        {
-            gboolean b;
-            int      limit;
-
-            tr_inf ( _( "Ending use of scheduled bandwidth limits" ) );
-
-            b = pref_flag_get( TR_PREFS_KEY_DSPEED_ENABLED );
-            tr_sessionSetSpeedLimitEnabled( tr, TR_DOWN, b );
-            limit = pref_int_get( TR_PREFS_KEY_DSPEED );
-            tr_sessionSetSpeedLimit( tr, TR_DOWN, limit );
-            b = pref_flag_get( TR_PREFS_KEY_USPEED_ENABLED );
-            tr_sessionSetSpeedLimitEnabled( tr, TR_UP, b );
-            limit = pref_int_get( TR_PREFS_KEY_USPEED );
-            tr_sessionSetSpeedLimit( tr, TR_UP, limit );
-        }
-
-        last_state = in_sched_state;
-    }
-    else if( in_sched_state )
-    {
-        static int old_dl_limit = 0, old_ul_limit = 0;
-        int        dl_limit = pref_int_get( PREF_KEY_SCHED_DL_LIMIT );
-        int        ul_limit = pref_int_get( PREF_KEY_SCHED_UL_LIMIT );
-
-        if( ( dl_limit != old_dl_limit ) || ( ul_limit != old_ul_limit ) )
-        {
-            tr_sessionSetSpeedLimitEnabled( tr, TR_DOWN, TRUE );
-            tr_sessionSetSpeedLimit( tr, TR_DOWN, dl_limit );
-            tr_sessionSetSpeedLimitEnabled( tr, TR_UP, TRUE );
-            tr_sessionSetSpeedLimit( tr, TR_UP, ul_limit );
-
-            old_dl_limit = dl_limit;
-            old_ul_limit = ul_limit;
-        }
-    }
-
-    return TRUE;
-}
-
 static void
 appsetup( TrWindow *      wind,
           GSList *        torrentFiles,
@@ -594,7 +518,7 @@ appsetup( TrWindow *      wind,
 
     /* add torrents from command-line and saved state */
     tr_core_load( cbdata->core, forcepause );
-    tr_core_add_list( cbdata->core, torrentFiles, start, prompt );
+    tr_core_add_list( cbdata->core, torrentFiles, start, prompt, TRUE );
     torrentFiles = NULL;
     tr_core_torrents_added( cbdata->core );
 
@@ -605,13 +529,9 @@ appsetup( TrWindow *      wind,
     prefschanged( cbdata->core, PREF_KEY_SHOW_TRAY_ICON, cbdata );
 
     /* start model update timer */
-    cbdata->timer = g_timeout_add( UPDATE_INTERVAL, updatemodel, cbdata );
+    cbdata->timer = gtr_timeout_add_seconds( REFRESH_INTERVAL_SECONDS, updatemodel, cbdata );
     updatemodel( cbdata );
 
-    /* start scheduled rate timer */
-    updateScheduledLimits ( tr_core_session( cbdata->core ) );
-    gtr_timeout_add_seconds( 60, updateScheduledLimits, tr_core_session( cbdata->core ) );
-
     /* either show the window or iconify it */
     if( !isIconified )
         gtk_widget_show( GTK_WIDGET( wind ) );
@@ -682,7 +602,7 @@ rowChangedCB( GtkTreeModel  * model UNUSED,
 {
     struct cbdata * data = gdata;
     if( gtk_tree_selection_path_is_selected ( data->sel, path ) )
-        refreshTorrentActions( gdata );
+        refreshActions( gdata );
 }
 
 static void
@@ -701,7 +621,7 @@ winsetup( struct cbdata * cbdata,
     model = tr_core_model( cbdata->core );
     g_signal_connect( model, "row-changed", G_CALLBACK( rowChangedCB ), cbdata );
     g_signal_connect( wind, "delete-event", G_CALLBACK( winclose ), cbdata );
-    refreshTorrentActions( cbdata );
+    refreshActions( cbdata );
 
     setupdrag( GTK_WIDGET( wind ), cbdata );
 }
@@ -710,10 +630,15 @@ static gpointer
 quitThreadFunc( gpointer gdata )
 {
     struct cbdata * cbdata = gdata;
+    gdk_threads_enter( );
 
     tr_core_close( cbdata->core );
 
     /* shutdown the gui */
+    if( cbdata->details ) {
+        g_slist_foreach( cbdata->details, (GFunc)gtk_widget_destroy, NULL );
+        g_slist_free( cbdata->details );
+    }
     if( cbdata->prefs )
         gtk_widget_destroy( GTK_WIDGET( cbdata->prefs ) );
     if( cbdata->wind )
@@ -721,23 +646,19 @@ quitThreadFunc( gpointer gdata )
     g_object_unref( cbdata->core );
     if( cbdata->icon )
         g_object_unref( cbdata->icon );
-    if( cbdata->errqueue )
-    {
+    if( cbdata->errqueue ) {
         g_slist_foreach( cbdata->errqueue, (GFunc)g_free, NULL );
         g_slist_free( cbdata->errqueue );
     }
-    if( cbdata->dupqueue )
-    {
+    if( cbdata->dupqueue ) {
         g_slist_foreach( cbdata->dupqueue, (GFunc)g_free, NULL );
         g_slist_free( cbdata->dupqueue );
     }
-
-    g_hash_table_destroy( cbdata->details2tor );
-    g_hash_table_destroy( cbdata->tor2details );
     g_free( cbdata );
 
-    /* exit the gtk main loop */
     gtk_main_quit( );
+    gdk_threads_leave( );
+
     return NULL;
 }
 
@@ -751,8 +672,8 @@ do_exit_cb( GtkWidget *w  UNUSED,
 static void
 wannaquit( void * vdata )
 {
-    GtkWidget *     r, * p, * b, * w, *c;
-    struct cbdata * cbdata = vdata;
+    GtkWidget *r, *p, *b, *w, *c;
+    struct cbdata *cbdata = vdata;
 
     /* stop the update timer */
     if( cbdata->timer )
@@ -787,14 +708,22 @@ wannaquit( void * vdata )
     w = gtr_button_new_from_stock( GTK_STOCK_QUIT, _( "_Quit Now" ) );
     g_signal_connect( w, "clicked", G_CALLBACK( do_exit_cb ), NULL );
     gtk_container_add( GTK_CONTAINER( b ), w );
-    gtk_table_attach( GTK_TABLE(
-                          p ), b, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 0, 10 );
+    gtk_table_attach( GTK_TABLE( p ), b, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 0, 10 );
 
     gtk_widget_show_all( r );
+    gtk_widget_grab_focus( w );
 
     /* clear the UI */
     gtk_list_store_clear( GTK_LIST_STORE( tr_core_model( cbdata->core ) ) );
 
+    /* ensure the window is in its previous position & size.
+     * this seems to be necessary because changing the main window's
+     * child seems to unset the size */
+    gtk_window_resize( cbdata->wind, pref_int_get( PREF_KEY_MAIN_WINDOW_WIDTH ),
+                                     pref_int_get( PREF_KEY_MAIN_WINDOW_HEIGHT ) );
+    gtk_window_move( cbdata->wind, pref_int_get( PREF_KEY_MAIN_WINDOW_X ),
+                                   pref_int_get( PREF_KEY_MAIN_WINDOW_Y ) );
+
     /* shut down libT */
     g_thread_create( quitThreadFunc, vdata, TRUE, NULL );
 }
@@ -876,7 +805,7 @@ gotdrag( GtkWidget         * widget UNUSED,
         if( paths )
         {
             paths = g_slist_reverse( paths );
-            tr_core_add_list_defaults( data->core, paths );
+            tr_core_add_list_defaults( data->core, paths, TRUE );
             tr_core_torrents_added( data->core );
         }
 
@@ -893,9 +822,9 @@ setupdrag( GtkWidget *    widget,
            struct cbdata *data )
 {
     GtkTargetEntry targets[] = {
-        { "STRING",          0, 0 },
-        { "text/plain",      0, 0 },
-        { "text/uri-list",   0, 0 },
+        { (char*)"STRING",          0, 0 },
+        { (char*)"text/plain",      0, 0 },
+        { (char*)"text/uri-list",   0, 0 },
     };
 
     g_signal_connect( widget, "drag_data_received", G_CALLBACK(
@@ -957,13 +886,8 @@ showTorrentErrors( struct cbdata * cbdata )
 }
 
 static void
-coreerr( TrCore * core    UNUSED,
-         enum tr_core_err code,
-         const char *     msg,
-         gpointer         gdata )
+coreerr( TrCore * core UNUSED, guint code, const char * msg, struct cbdata * c )
 {
-    struct cbdata * c = gdata;
-
     switch( code )
     {
         case TR_EINVALID:
@@ -997,7 +921,8 @@ on_main_window_focus_in( GtkWidget      * widget UNUSED,
 {
     struct cbdata * cbdata = gdata;
 
-    gtk_window_set_urgency_hint( GTK_WINDOW( cbdata->wind ), FALSE );
+    if( cbdata->wind )
+        gtk_window_set_urgency_hint( GTK_WINDOW( cbdata->wind ), FALSE );
 }
 
 #endif
@@ -1013,7 +938,8 @@ onAddTorrent( TrCore *  core,
 #if GTK_CHECK_VERSION( 2, 8, 0 )
     g_signal_connect( w, "focus-in-event",
                       G_CALLBACK( on_main_window_focus_in ),  cbdata );
-    gtk_window_set_urgency_hint( cbdata->wind, TRUE );
+    if( cbdata->wind )
+        gtk_window_set_urgency_hint( cbdata->wind, TRUE );
 #endif
 }
 
@@ -1027,9 +953,7 @@ prefschanged( TrCore * core UNUSED,
 
     if( !strcmp( key, TR_PREFS_KEY_ENCRYPTION ) )
     {
-        const int encryption = pref_int_get( key );
-        g_message( "setting encryption to %d", encryption );
-        tr_sessionSetEncryption( tr, encryption );
+        tr_sessionSetEncryption( tr, pref_int_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_DOWNLOAD_DIR ) )
     {
@@ -1039,54 +963,47 @@ prefschanged( TrCore * core UNUSED,
     {
         tr_setMessageLevel( pref_int_get( key ) );
     }
-    else if( !strcmp( key, TR_PREFS_KEY_PEER_PORT_RANDOM_ENABLED ) )
-    {
-        /* FIXME */
-    }
     else if( !strcmp( key, TR_PREFS_KEY_PEER_PORT ) )
     {
-        const int port = pref_int_get( key );
-        tr_sessionSetPeerPort( tr, port );
+        tr_sessionSetPeerPort( tr, pref_int_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_BLOCKLIST_ENABLED ) )
     {
-        const gboolean flag = pref_flag_get( key );
-        tr_blocklistSetEnabled( tr, flag );
+        tr_blocklistSetEnabled( tr, pref_flag_get( key ) );
     }
     else if( !strcmp( key, PREF_KEY_SHOW_TRAY_ICON ) )
     {
         const int show = pref_flag_get( key );
         if( show && !cbdata->icon )
             cbdata->icon = tr_icon_new( cbdata->core );
-        else if( !show && cbdata->icon )
-        {
+        else if( !show && cbdata->icon ) {
             g_object_unref( cbdata->icon );
             cbdata->icon = NULL;
         }
     }
     else if( !strcmp( key, TR_PREFS_KEY_DSPEED_ENABLED ) )
     {
-        const gboolean b = pref_flag_get( key );
-        tr_sessionSetSpeedLimitEnabled( tr, TR_DOWN, b );
+        tr_sessionLimitSpeed( tr, TR_DOWN, pref_flag_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_DSPEED ) )
     {
-        const int limit = pref_int_get( key );
-        tr_sessionSetSpeedLimit( tr, TR_DOWN, limit );
+        tr_sessionSetSpeedLimit( tr, TR_DOWN, pref_int_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_USPEED_ENABLED ) )
     {
-        const gboolean b = pref_flag_get( key );
-        tr_sessionSetSpeedLimitEnabled( tr, TR_UP, b );
+        tr_sessionLimitSpeed( tr, TR_UP, pref_flag_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_USPEED ) )
     {
-        const int limit = pref_int_get( key );
-        tr_sessionSetSpeedLimit( tr, TR_UP, limit );
+        tr_sessionSetSpeedLimit( tr, TR_UP, pref_int_get( key ) );
     }
-    else if( !strncmp( key, "sched-", 6 ) )
+    else if( !strcmp( key, TR_PREFS_KEY_RATIO_ENABLED ) )
     {
-        updateScheduledLimits( tr );
+        tr_sessionSetRatioLimited( tr, pref_flag_get( key ) );
+    }
+    else if( !strcmp( key, TR_PREFS_KEY_RATIO ) )
+    {
+        tr_sessionSetRatioLimit( tr, pref_double_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PORT_FORWARDING ) )
     {
@@ -1096,6 +1013,10 @@ prefschanged( TrCore * core UNUSED,
     {
         tr_sessionSetPexEnabled( tr, pref_flag_get( key ) );
     }
+    else if( !strcmp( key, TR_PREFS_KEY_DHT_ENABLED ) )
+    {
+        tr_sessionSetDHTEnabled( tr, pref_flag_get( key ) );
+    }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_PORT ) )
     {
         tr_sessionSetRPCPort( tr, pref_int_get( key ) );
@@ -1106,8 +1027,7 @@ prefschanged( TrCore * core UNUSED,
     }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_WHITELIST ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetRPCWhitelist( tr, s );
+        tr_sessionSetRPCWhitelist( tr, pref_string_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_WHITELIST_ENABLED ) )
     {
@@ -1115,57 +1035,77 @@ prefschanged( TrCore * core UNUSED,
     }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_USERNAME ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetRPCUsername( tr, s );
+        tr_sessionSetRPCUsername( tr, pref_string_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_PASSWORD ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetRPCPassword( tr, s );
+        tr_sessionSetRPCPassword( tr, pref_string_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_AUTH_REQUIRED ) )
     {
-        const gboolean enabled = pref_flag_get( key );
-        tr_sessionSetRPCPasswordEnabled( tr, enabled );
+        tr_sessionSetRPCPasswordEnabled( tr, pref_flag_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetProxy( tr, s );
+        tr_sessionSetProxy( tr, pref_string_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY_TYPE ) )
     {
-        const int i = pref_int_get( key );
-        tr_sessionSetProxyType( tr, i );
+        tr_sessionSetProxyType( tr, pref_int_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY_ENABLED ) )
     {
-        const gboolean enabled = pref_flag_get( key );
-        tr_sessionSetProxyEnabled( tr, enabled );
+        tr_sessionSetProxyEnabled( tr, pref_flag_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY_AUTH_ENABLED ) )
     {
-        const gboolean enabled = pref_flag_get( key );
-        tr_sessionSetProxyAuthEnabled( tr, enabled );
+        tr_sessionSetProxyAuthEnabled( tr, pref_flag_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY_USERNAME ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetProxyUsername( tr, s );
+        tr_sessionSetProxyUsername( tr, pref_string_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY_PASSWORD ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetProxyPassword( tr, s );
+        tr_sessionSetProxyPassword( tr, pref_string_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY_PORT ) )
     {
         tr_sessionSetProxyPort( tr, pref_int_get( key ) );
     }
-    else if( !strcmp( key, TR_PREFS_KEY_RPC_PASSWORD ) )
+    else if( !strcmp( key, TR_PREFS_KEY_ALT_SPEED_UP ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetProxyPassword( tr, s );
+        tr_sessionSetAltSpeed( tr, TR_UP, pref_int_get( key ) );
+    }
+    else if( !strcmp( key, TR_PREFS_KEY_ALT_SPEED_DOWN ) )
+    {
+        tr_sessionSetAltSpeed( tr, TR_DOWN, pref_int_get( key ) );
+    }
+    else if( !strcmp( key, TR_PREFS_KEY_ALT_SPEED_ENABLED ) )
+    {
+        const gboolean b = pref_flag_get( key );
+        tr_sessionUseAltSpeed( tr, b );
+        action_toggle( key, b );
+    }
+    else if( !strcmp( key, TR_PREFS_KEY_ALT_SPEED_TIME_BEGIN ) )
+    {
+        tr_sessionSetAltSpeedBegin( tr, pref_int_get( key ) );
+    }
+    else if( !strcmp( key, TR_PREFS_KEY_ALT_SPEED_TIME_END ) )
+    {
+        tr_sessionSetAltSpeedEnd( tr, pref_int_get( key ) );
+    }
+    else if( !strcmp( key, TR_PREFS_KEY_ALT_SPEED_TIME_ENABLED ) )
+    {
+        tr_sessionUseAltSpeedTime( tr, pref_flag_get( key ) );
+    }
+    else if( !strcmp( key, TR_PREFS_KEY_ALT_SPEED_TIME_DAY ) )
+    {
+        tr_sessionSetAltSpeedDay( tr, pref_int_get( key ) );
+    }
+    else if( !strcmp( key, TR_PREFS_KEY_PEER_PORT_RANDOM_ON_START ) )
+    {
+        tr_sessionSetPeerPortRandomOnStart( tr, pref_flag_get( key ) );
     }
 }
 
@@ -1185,7 +1125,7 @@ updatemodel( gpointer gdata )
             tr_window_update( data->wind );
 
         /* update the actions */
-        refreshTorrentActions( data );
+        refreshActions( data );
     }
 
     return !done;
@@ -1206,9 +1146,7 @@ about( GtkWindow * parent )
     {
         "Charles Kerr (Backend; GTK+)",
         "Mitchell Livingston (Backend; OS X)",
-        "Eric Petit (Backend; OS X)",
-        "Josh Elsasser (Daemon; Backend; GTK+)",
-        "Bryan Varner (BeOS)",
+        "Kevin Glowacz (Web client)",
         NULL
     };
 
@@ -1275,18 +1213,6 @@ updateTrackerForeach( GtkTreeModel *      model,
     tr_torrentManualUpdate( tor );
 }
 
-static void
-detailsClosed( gpointer  user_data,
-               GObject * details )
-{
-    struct cbdata * data = user_data;
-    gpointer        hashString = g_hash_table_lookup( data->details2tor,
-                                                      details );
-
-    g_hash_table_remove( data->details2tor, details );
-    g_hash_table_remove( data->tor2details, hashString );
-}
-
 static void
 openFolderForeach( GtkTreeModel *           model,
                    GtkTreePath  * path      UNUSED,
@@ -1300,34 +1226,6 @@ openFolderForeach( GtkTreeModel *           model,
     g_object_unref( G_OBJECT( gtor ) );
 }
 
-static void
-showInfoForeach( GtkTreeModel *      model,
-                 GtkTreePath  * path UNUSED,
-                 GtkTreeIter *       iter,
-                 gpointer            user_data )
-{
-    const char *    hashString;
-    struct cbdata * data = user_data;
-    TrTorrent *     tor = NULL;
-    GtkWidget *     w;
-
-    gtk_tree_model_get( model, iter, MC_TORRENT, &tor, -1 );
-    hashString = tr_torrent_info( tor )->hashString;
-    w = g_hash_table_lookup( data->tor2details, hashString );
-    if( w != NULL )
-        gtk_window_present( GTK_WINDOW( w ) );
-    else
-    {
-        w = torrent_inspector_new( GTK_WINDOW( data->wind ), tor );
-        gtk_widget_show( w );
-        g_hash_table_insert( data->tor2details, (gpointer)hashString, w );
-        g_hash_table_insert( data->details2tor, w, (gpointer)hashString );
-        g_object_weak_ref( G_OBJECT( w ), detailsClosed, data );
-    }
-
-    g_object_unref( G_OBJECT( tor ) );
-}
-
 static void
 recheckTorrentForeach( GtkTreeModel *      model,
                        GtkTreePath  * path UNUSED,
@@ -1393,6 +1291,31 @@ pauseAllTorrents( struct cbdata * data )
     tr_rpc_request_exec_json( session, cmd, strlen( cmd ), NULL, NULL );
 }
 
+static tr_torrent*
+getFirstSelectedTorrent( struct cbdata * data )
+{
+    tr_torrent * tor = NULL;
+    GtkTreeSelection * s = tr_window_get_selection( data->wind );
+    GtkTreeModel * m;
+    GList * l = gtk_tree_selection_get_selected_rows( s, &m );
+    if( l != NULL ) {
+        GtkTreePath * p = l->data;
+        GtkTreeIter i;
+        if( gtk_tree_model_get_iter( m, &i, p ) )
+            gtk_tree_model_get( m, &i, MC_TORRENT_RAW, &tor, -1 );
+    }
+    g_list_foreach( l, (GFunc)gtk_tree_path_free, NULL );
+    g_list_free( l );
+    return tor;
+}
+
+static void
+detailsClosed( gpointer gdata, GObject * dead )
+{
+    struct cbdata * data = gdata;
+    data->details = g_slist_remove( data->details, dead );
+}
+
 void
 doAction( const char * action_name, gpointer user_data )
 {
@@ -1423,6 +1346,16 @@ doAction( const char * action_name, gpointer user_data )
     {
         startAllTorrents( data );
     }
+    else if( !strcmp( action_name, "relocate-torrent" ) )
+    {
+        tr_torrent * tor = getFirstSelectedTorrent( data );
+        if( tor )
+        {
+            GtkWindow * parent = GTK_WINDOW( data->wind );
+            GtkWidget * w = gtr_relocate_dialog_new( parent, tor );
+            gtk_widget_show( w );
+        }
+    }
     else if( !strcmp( action_name, "pause-torrent" ) )
     {
         GtkTreeSelection * s = tr_window_get_selection( data->wind );
@@ -1442,8 +1375,11 @@ doAction( const char * action_name, gpointer user_data )
     }
     else if( !strcmp( action_name, "show-torrent-properties" ) )
     {
-        GtkTreeSelection * s = tr_window_get_selection( data->wind );
-        gtk_tree_selection_selected_foreach( s, showInfoForeach, data );
+        GtkWidget * w = torrent_inspector_new( GTK_WINDOW( data->wind ), data->core );
+        data->details = g_slist_prepend( data->details, w );
+        g_object_weak_ref( G_OBJECT( w ), detailsClosed, data );
+        refreshDetailsDialog( data, w );
+        gtk_widget_show( w );
     }
     else if( !strcmp( action_name, "update-tracker" ) )
     {
@@ -1529,4 +1465,3 @@ doAction( const char * action_name, gpointer user_data )
     if( changed )
         updatemodel( data );
 }
-
