@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: makemeta-ui.c 7664 2009-01-11 17:46:51Z charles $
+ * $Id: makemeta-ui.c 8757 2009-06-29 17:22:35Z charles $
  */
 
 #include <string.h>
@@ -24,8 +24,6 @@
 #include "tracker-list.h"
 #include "util.h"
 
-#define UPDATE_INTERVAL_MSEC 200
-
 #define UI_KEY "ui"
 #define ANNOUNCE_KEY "recent-announce-url"
 
@@ -230,7 +228,7 @@ response_cb( GtkDialog* d,
                     gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( ui->
                                                                      private_check ) ) );
 
-    tag = g_timeout_add ( UPDATE_INTERVAL_MSEC, refresh_cb, ui );
+    tag = gtr_timeout_add_seconds( 1, refresh_cb, ui );
     g_object_set_data_full ( G_OBJECT( d ), "tag", GUINT_TO_POINTER(
                                  tag ), remove_tag );
 
@@ -303,7 +301,7 @@ onSourceActivated( GtkEditable * editable,
 
     if( ui->builder )
         tr_metaInfoBuilderFree( ui->builder );
-    ui->builder = tr_metaInfoBuilderCreate( ui->session, filename );
+    ui->builder = tr_metaInfoBuilderCreate( filename );
     refreshFromBuilder( ui );
 }
 
@@ -426,7 +424,7 @@ make_meta_ui( GtkWindow  * parent,
     hig_workarea_add_section_divider( t, &row );
     hig_workarea_add_section_title( t, &row, _( "Trackers" ) );
 
-    w = tracker_list_new( NULL );
+    w = tracker_list_new( session, -1, TRUE );
 
     n = pref_int_get( ANNOUNCE_KEY"-count" );
     if( n > 0 )
