@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: msgwin.c 7664 2009-01-11 17:46:51Z charles $
+ * $Id: msgwin.c 8724 2009-06-20 23:55:36Z charles $
  */
 
 #include <errno.h>
@@ -459,8 +459,7 @@ msgwin_new( TrCore * core )
     **/
 
     toolbar = gtk_toolbar_new( );
-    gtk_toolbar_set_orientation( GTK_TOOLBAR(
-                                     toolbar ), GTK_ORIENTATION_HORIZONTAL );
+    gtr_toolbar_set_orientation( GTK_TOOLBAR( toolbar ), GTK_ORIENTATION_HORIZONTAL );
     gtk_toolbar_set_style( GTK_TOOLBAR( toolbar ), GTK_TOOLBAR_BOTH_HORIZ );
 
     item = gtk_tool_button_new_from_stock( GTK_STOCK_SAVE );
