@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tr-window.c 7664 2009-01-11 17:46:51Z charles $
+ * $Id: tr-window.c 8763 2009-06-30 18:08:50Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -27,12 +27,18 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
+#if !GTK_CHECK_VERSION( 2,16,0 )
+ /* FIXME: when 2.16 has been out long enough, it would be really nice to
+  * get rid of this libsexy usage because of its makefile strangeness */
+ #define USE_SEXY
+ #include "sexy-icon-entry.h"
+#endif
+
 #include <libtransmission/transmission.h>
 
 #include "actions.h"
 #include "conf.h"
 #include "hig.h"
-#include "sexy-icon-entry.h"
 #include "torrent-cell-renderer.h"
 #include "tr-prefs.h"
 #include "tr-torrent.h"
@@ -73,16 +79,25 @@ filter_mode_t;
 
 typedef struct
 {
+    GtkWidget *           speedlimit_on_item[2];
+    GtkWidget *           speedlimit_off_item[2];
+    GtkWidget *           ratio_on_item;
+    GtkWidget *           ratio_off_item;
     GtkWidget *           scroll;
     GtkWidget *           view;
     GtkWidget *           toolbar;
     GtkWidget *           filter;
     GtkWidget *           status;
     GtkWidget *           status_menu;
+    GtkWidget *           ul_hbox;
+    GtkWidget *           dl_hbox;
     GtkWidget *           ul_lb;
     GtkWidget *           dl_lb;
     GtkWidget *           stats_lb;
     GtkWidget *           gutter_lb;
+    GtkWidget *           alt_speed_image[2]; /* 0==off, 1==on */
+    GtkWidget *           alt_speed_button;
+    GtkWidget *           options_menu;
     GtkTreeSelection *    selection;
     GtkCellRenderer *     renderer;
     GtkTreeViewColumn *   column;
@@ -92,14 +107,35 @@ typedef struct
     filter_mode_t         filter_mode;
     filter_text_mode_t    filter_text_mode;
     char *                filter_text;
+    GtkToggleButton     * filter_toggles[FILTER_MODE_QTY];
 }
 PrivateData;
 
-#define PRIVATE_DATA_KEY "private-data"
+static const char*
+getFilterName( int mode )
+{
+    switch( mode )
+    {
+        case FILTER_MODE_ACTIVE:      return "show-active";
+        case FILTER_MODE_DOWNLOADING: return "show-downloading";
+        case FILTER_MODE_SEEDING:     return "show-seeding";
+        case FILTER_MODE_PAUSED:      return "show-paused";
+        default:                      return "show-all"; /* the fallback */
+    }
+}
+static int
+getFilterModeFromName( const char * name )
+{
+    if( !strcmp( name, "show-active"      ) ) return FILTER_MODE_ACTIVE;
+    if( !strcmp( name, "show-downloading" ) ) return FILTER_MODE_DOWNLOADING;
+    if( !strcmp( name, "show-seeding"     ) ) return FILTER_MODE_SEEDING;
+    if( !strcmp( name, "show-paused"      ) ) return FILTER_MODE_PAUSED;
+    return FILTER_MODE_ALL; /* the fallback */
+}
 
+#define PRIVATE_DATA_KEY "private-data"
 #define FILTER_MODE_KEY "tr-filter-mode"
 #define FILTER_TEXT_MODE_KEY "tr-filter-text-mode"
-#define FILTER_TOGGLES_KEY "tr-filter-toggles"
 
 static PrivateData*
 get_private_data( TrWindow * w )
@@ -147,15 +183,20 @@ makeview( PrivateData * p,
 
     view = gtk_tree_view_new( );
     gtk_tree_view_set_headers_visible( GTK_TREE_VIEW( view ), FALSE );
+    gtk_tree_view_set_fixed_height_mode( GTK_TREE_VIEW( view ), TRUE );
 
     p->selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
 
+    p->column = col = GTK_TREE_VIEW_COLUMN (g_object_new (GTK_TYPE_TREE_VIEW_COLUMN,		      
+        "title", _("Torrent"),
+        "resizable", TRUE,
+        "sizing", GTK_TREE_VIEW_COLUMN_FIXED,
+        NULL));
+
     p->renderer = r = torrent_cell_renderer_new( );
-    p->column = col = gtk_tree_view_column_new_with_attributes(
-                    _( "Torrent" ), r, "torrent", MC_TORRENT_RAW, NULL );
-    g_object_set( G_OBJECT( col ), "resizable", TRUE,
-                  "sizing", GTK_TREE_VIEW_COLUMN_FIXED,
-                  NULL );
+    gtk_tree_view_column_pack_start( col, r, FALSE );
+    gtk_tree_view_column_add_attribute( col, r, "torrent", MC_TORRENT_RAW );
+    
     gtk_tree_view_append_column( GTK_TREE_VIEW( view ), col );
     g_object_set( r, "xpad", GUI_PAD_SMALL, "ypad", GUI_PAD_SMALL, NULL );
 
@@ -188,6 +229,9 @@ makeview( PrivateData * p,
     return view;
 }
 
+static void syncAltSpeedButton( PrivateData * p );
+static void setFilter( PrivateData * p, int mode );
+
 static void
 prefsChanged( TrCore * core UNUSED,
               const char *  key,
@@ -198,8 +242,14 @@ prefsChanged( TrCore * core UNUSED,
     if( !strcmp( key, PREF_KEY_MINIMAL_VIEW ) )
     {
         g_object_set( p->renderer, "minimal", pref_flag_get( key ), NULL );
-        gtk_tree_view_column_queue_resize( p->column );
-        gtk_widget_queue_draw( p->view );
+        /* since the cell size has changed, we need gtktreeview to revalidate
+         * its fixed-height mode values.  Unfortunately there's not an API call
+         * for that, but it *does* revalidate when it thinks the style's been tweaked */
+        g_signal_emit_by_name( p->view, "style-set", NULL, NULL );
+    }
+    else if( !strcmp( key, PREF_KEY_FILTER_MODE ) )
+    {
+        setFilter( p, getFilterModeFromName( pref_string_get( key ) ) );
     }
     else if( !strcmp( key, PREF_KEY_STATUSBAR ) )
     {
@@ -220,6 +270,12 @@ prefsChanged( TrCore * core UNUSED,
     {
         tr_window_update( (TrWindow*)wind );
     }
+    else if( !strcmp( key, TR_PREFS_KEY_ALT_SPEED_ENABLED ) ||
+             !strcmp( key, TR_PREFS_KEY_ALT_SPEED_UP ) ||
+             !strcmp( key, TR_PREFS_KEY_ALT_SPEED_DOWN ) )
+    {
+        syncAltSpeedButton( p );
+    }
 }
 
 static void
@@ -228,20 +284,20 @@ privateFree( gpointer vprivate )
     PrivateData * p = vprivate;
 
     g_signal_handler_disconnect( p->core, p->pref_handler_id );
+    g_object_unref( G_OBJECT( p->alt_speed_image[1] ) );
+    g_object_unref( G_OBJECT( p->alt_speed_image[0] ) );
     g_free( p->filter_text );
     g_free( p );
 }
 
 static void
-onYinYangReleased( GtkWidget * w           UNUSED,
-                   GdkEventButton * button UNUSED,
-                   gpointer                vprivate )
+onYinYangReleased( GtkWidget * w UNUSED, gpointer vprivate )
 {
     PrivateData * p = vprivate;
 
-    gtk_menu_popup( GTK_MENU(
-                       p->status_menu ), NULL, NULL, NULL, NULL, 0,
-                   gtk_get_current_event_time( ) );
+    gtk_menu_popup( GTK_MENU( p->status_menu ),
+                    NULL, NULL, NULL, NULL, 0,
+                    gtk_get_current_event_time( ) );
 }
 
 #define STATS_MODE "stats-mode"
@@ -269,6 +325,36 @@ status_menu_toggled_cb( GtkCheckMenuItem * menu_item,
     }
 }
 
+static void
+syncAltSpeedButton( PrivateData * p )
+{
+    char u[32];
+    char d[32];
+    char buf[128];
+    const char * fmt;
+    const gboolean b = pref_flag_get( TR_PREFS_KEY_ALT_SPEED_ENABLED );
+    GtkWidget * w = p->alt_speed_button;
+
+    tr_strlspeed( u, pref_int_get( TR_PREFS_KEY_ALT_SPEED_UP ), sizeof( u ) );
+    tr_strlspeed( d, pref_int_get( TR_PREFS_KEY_ALT_SPEED_DOWN ), sizeof( d ) );
+    fmt = b ? _( "Click to disable Temporary Speed Limits\n(%1$s down, %2$s up)" )
+            : _( "Click to enable Temporary Speed Limits\n(%1$s down, %2$s up)" );
+    g_snprintf( buf, sizeof( buf ), fmt, d, u );
+
+    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), b );
+    gtk_button_set_image( GTK_BUTTON( w ), p->alt_speed_image[b?1:0] );
+    gtk_button_set_alignment( GTK_BUTTON( w ), 0.5, 0.5 );
+    gtr_widget_set_tooltip_text( w, buf );
+}
+
+static void
+alt_speed_toggled_cb( GtkToggleButton * button, gpointer vprivate )
+{
+    PrivateData * p = vprivate;
+    const gboolean b = gtk_toggle_button_get_active( button );
+    tr_core_set_pref_bool( p->core, TR_PREFS_KEY_ALT_SPEED_ENABLED,  b );
+}
+    
 /***
 ****  FILTER
 ***/
@@ -332,7 +418,9 @@ checkFilterMode( filter_mode_t filter_mode,
         case FILTER_MODE_ACTIVE:
         {
             const tr_stat * s = tr_torrentStatCached( tor );
-            ret = s->peersSendingToUs > 0 || s->peersGettingFromUs > 0;
+            ret = s->peersSendingToUs > 0
+               || s->peersGettingFromUs > 0
+               || tr_torrentGetActivity( tor ) == TR_STATUS_CHECK;
             break;
         }
 
@@ -382,37 +470,35 @@ filter_text_toggled_cb( GtkCheckMenuItem * menu_item,
 }
 
 static void
-filter_toggled_cb( GtkToggleButton * toggle,
-                   gpointer          vprivate )
+setFilter( PrivateData * p, int mode )
 {
-    PrivateData *       p = vprivate;
-    GSList *            l;
-    GSList *            toggles = g_object_get_data( G_OBJECT(
-                                                         toggle ),
-                                                     FILTER_TOGGLES_KEY );
-    const gboolean      isActive = gtk_toggle_button_get_active( toggle );
-    const filter_mode_t mode =
-        GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( toggle ),
-                                             FILTER_MODE_KEY ) );
-
-    /* update the filter */
-    if( isActive )
+    if( mode != (int)p->filter_mode )
     {
+        int i;
+
+        /* refilter */
         p->filter_mode = mode;
         refilter( p );
-    }
 
-    /* deactivate the other toggles */
-    for( l = toggles; l != NULL; l = l->next )
+        /* update the prefs */
+        tr_core_set_pref( p->core, PREF_KEY_FILTER_MODE, getFilterName( mode ) );
+
+        /* update the togglebuttons */
+        for( i=0; i<FILTER_MODE_QTY; ++i )
+            gtk_toggle_button_set_active( p->filter_toggles[i], i==mode );
+    }
+}
+  
+
+static void
+filter_toggled_cb( GtkToggleButton * toggle, gpointer vprivate )
+{
+    if( gtk_toggle_button_get_active( toggle ) )
     {
-        GtkToggleButton * walk = GTK_TOGGLE_BUTTON( l->data );
-        if( isActive && ( toggle != walk ) )
-            gtk_toggle_button_set_active( walk, FALSE );
+        PrivateData * p = vprivate;
+        const int mode = GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( toggle ), FILTER_MODE_KEY ) );
+        setFilter( p, mode );
     }
-
-    /* at least one button must always be set */
-    if( !isActive && ( p->filter_mode == mode ) )
-        gtk_toggle_button_set_active( toggle, TRUE );
 }
 
 static void
@@ -429,17 +515,33 @@ filter_entry_changed( GtkEditable * e,
     g_free( pch );
 }
 
+
+#ifdef USE_SEXY
 static void
-entry_icon_released( SexyIconEntry         * entry  UNUSED,
-                     SexyIconEntryPosition          icon_pos,
-                     int                     button UNUSED,
-                     gpointer                       menu )
+entry_icon_released( SexyIconEntry           * entry  UNUSED,
+                     SexyIconEntryPosition     icon_pos,
+                     int                       button UNUSED,
+                     gpointer                  menu )
 {
     if( icon_pos == SEXY_ICON_ENTRY_PRIMARY )
-        gtk_menu_popup ( GTK_MENU(
-                            menu ), NULL, NULL, NULL, NULL, 0,
+        gtk_menu_popup( GTK_MENU( menu ), NULL, NULL, NULL, NULL, 0,
                         gtk_get_current_event_time( ) );
 }
+#else
+static void
+entry_icon_release( GtkEntry              * entry  UNUSED,
+                    GtkEntryIconPosition    icon_pos,
+                    GdkEventButton        * event  UNUSED,
+                    gpointer                menu )
+{
+    if( icon_pos == GTK_ENTRY_ICON_SECONDARY )
+        gtk_entry_set_text( entry, "" );
+
+    if( icon_pos == GTK_ENTRY_ICON_PRIMARY )
+        gtk_menu_popup( GTK_MENU( menu ), NULL, NULL, NULL, NULL, 0,
+                        gtk_get_current_event_time( ) );
+}
+#endif
 
 #if GTK_CHECK_VERSION( 2, 12, 0 )
 
@@ -493,6 +595,228 @@ onAskTrackerQueryTooltip( GtkWidget *            widget UNUSED,
 
 #endif
 
+static gboolean
+onAltSpeedToggledIdle( gpointer vp )
+{
+    PrivateData * p = vp;
+    gboolean b = tr_sessionUsesAltSpeed( tr_core_session( p->core ) );
+    tr_core_set_pref_bool( p->core, TR_PREFS_KEY_ALT_SPEED_ENABLED, b );
+
+    return FALSE;
+}
+
+static void
+onAltSpeedToggled( tr_session * s UNUSED, tr_bool isEnabled UNUSED, tr_bool byUser UNUSED, void * p )
+{
+    gtr_idle_add( onAltSpeedToggledIdle, p );
+}
+
+/***
+****  Speed limit menu
+***/
+
+#define DIRECTION_KEY "direction-key"
+#define ENABLED_KEY "enabled-key"
+#define SPEED_KEY "speed-key"
+
+static void
+onSpeedToggled( GtkCheckMenuItem * check, gpointer vp ) 
+{
+    PrivateData * p = vp;
+    GObject * o = G_OBJECT( check );
+    gboolean isEnabled = g_object_get_data( o, ENABLED_KEY ) != 0;
+    tr_direction dir = GPOINTER_TO_INT( g_object_get_data( o, DIRECTION_KEY ) );
+    const char * key = dir == TR_UP ? TR_PREFS_KEY_USPEED_ENABLED
+                                    : TR_PREFS_KEY_DSPEED_ENABLED;
+
+    if( gtk_check_menu_item_get_active( check ) )
+        tr_core_set_pref_bool( p->core, key, isEnabled );
+}
+
+static void
+onSpeedSet( GtkCheckMenuItem * check, gpointer vp ) 
+{
+    const char * key;
+    PrivateData * p = vp;
+    GObject * o = G_OBJECT( check );
+    const int speed = GPOINTER_TO_INT( g_object_get_data( o, SPEED_KEY ) );
+    tr_direction dir = GPOINTER_TO_INT( g_object_get_data( o, DIRECTION_KEY ) );
+
+    key = dir==TR_UP ? TR_PREFS_KEY_USPEED : TR_PREFS_KEY_DSPEED;
+    tr_core_set_pref_int( p->core, key, speed );
+
+    key = dir==TR_UP ? TR_PREFS_KEY_USPEED_ENABLED : TR_PREFS_KEY_DSPEED_ENABLED;
+    tr_core_set_pref_bool( p->core, key, TRUE );
+}
+
+static GtkWidget*
+createSpeedMenu( PrivateData * p, tr_direction dir )
+{
+    int i, n;
+    GtkWidget *w, *m;
+    const int speeds[] = { 5, 10, 20, 30, 40, 50, 75, 100, 150, 200, 250, 500, 750 };
+
+    m = gtk_menu_new( );
+
+    w = gtk_radio_menu_item_new_with_label( NULL, _( "Unlimited" ) );
+    p->speedlimit_off_item[dir] = w;
+    g_object_set_data( G_OBJECT( w ), DIRECTION_KEY, GINT_TO_POINTER( dir ) );
+    g_object_set_data( G_OBJECT( w ), ENABLED_KEY, GINT_TO_POINTER( FALSE ) );
+    g_signal_connect( w, "toggled", G_CALLBACK(onSpeedToggled), p );
+    gtk_menu_shell_append( GTK_MENU_SHELL( m ), w );
+
+    w = gtk_radio_menu_item_new_with_label_from_widget( GTK_RADIO_MENU_ITEM( w ), "" );
+    p->speedlimit_on_item[dir] = w;
+    g_object_set_data( G_OBJECT( w ), DIRECTION_KEY, GINT_TO_POINTER( dir ) );
+    g_object_set_data( G_OBJECT( w ), ENABLED_KEY, GINT_TO_POINTER( TRUE ) );
+    g_signal_connect( w, "toggled", G_CALLBACK(onSpeedToggled), p );
+    gtk_menu_shell_append( GTK_MENU_SHELL( m ), w );
+
+    w = gtk_separator_menu_item_new( );
+    gtk_menu_shell_append( GTK_MENU_SHELL( m ), w );
+
+    for( i=0, n=G_N_ELEMENTS(speeds); i<n; ++i )
+    {
+        char buf[128];
+        tr_strlspeed( buf, speeds[i], sizeof( buf ) );
+        w = gtk_menu_item_new_with_label( buf );
+        g_object_set_data( G_OBJECT( w ), DIRECTION_KEY, GINT_TO_POINTER( dir ) );
+        g_object_set_data( G_OBJECT( w ), SPEED_KEY, GINT_TO_POINTER( speeds[i] ) );
+        g_signal_connect( w, "activate", G_CALLBACK(onSpeedSet), p );
+        gtk_menu_shell_append( GTK_MENU_SHELL( m ), w );
+    }
+
+    return m;
+}
+
+/***
+****  Speed limit menu
+***/
+
+#define RATIO_KEY "stock-ratio-index"
+
+static const double stockRatios[] = { 0.25, 0.5, 0.75, 1, 1.5, 2, 3 };
+
+static void
+onRatioToggled( GtkCheckMenuItem * check, gpointer vp ) 
+{
+    PrivateData * p = vp;
+    if( gtk_check_menu_item_get_active( check ) )
+    {
+        gboolean f = g_object_get_data( G_OBJECT( check ), ENABLED_KEY ) != 0;
+        tr_core_set_pref_bool( p->core, TR_PREFS_KEY_RATIO_ENABLED, f );
+    }
+}
+static void
+onRatioSet( GtkCheckMenuItem * check, gpointer vp ) 
+{
+    PrivateData * p = vp;
+    int i = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( check ), RATIO_KEY ) );
+    const double ratio = stockRatios[i];
+    tr_core_set_pref_double( p->core, TR_PREFS_KEY_RATIO, ratio );
+    tr_core_set_pref_bool  ( p->core, TR_PREFS_KEY_RATIO_ENABLED, TRUE );
+}
+
+static GtkWidget*
+createRatioMenu( PrivateData * p )
+{
+    int i, n;
+    GtkWidget *m, *w;
+
+    m = gtk_menu_new( );
+
+    w = gtk_radio_menu_item_new_with_label( NULL, _( "Seed Forever" ) );
+    p->ratio_off_item = w;
+    g_object_set_data( G_OBJECT( w ), ENABLED_KEY, GINT_TO_POINTER( FALSE ) );
+    g_signal_connect( w, "toggled", G_CALLBACK(onRatioToggled), p );
+    gtk_menu_shell_append( GTK_MENU_SHELL( m ), w );
+
+    w = gtk_radio_menu_item_new_with_label_from_widget( GTK_RADIO_MENU_ITEM( w ), "" );
+    p->ratio_on_item = w;
+    g_object_set_data( G_OBJECT( w ), ENABLED_KEY, GINT_TO_POINTER( TRUE ) );
+    g_signal_connect( w, "toggled", G_CALLBACK(onRatioToggled), p );
+    gtk_menu_shell_append( GTK_MENU_SHELL( m ), w );
+
+    w = gtk_separator_menu_item_new( );
+    gtk_menu_shell_append( GTK_MENU_SHELL( m ), w );
+
+    for( i=0, n=G_N_ELEMENTS(stockRatios); i<n; ++i )
+    {
+        char buf[128];
+        tr_strlratio( buf, stockRatios[i], sizeof( buf ) );
+        w = gtk_menu_item_new_with_label( buf );
+        g_object_set_data( G_OBJECT( w ), RATIO_KEY, GINT_TO_POINTER( i ) );
+        g_signal_connect( w, "activate", G_CALLBACK(onRatioSet), p );
+        gtk_menu_shell_append( GTK_MENU_SHELL( m ), w );
+    }
+
+    return m;
+}
+
+/***
+****  Option menu
+***/
+
+static GtkWidget*
+createOptionsMenu( PrivateData * p )
+{
+    GtkWidget * m;
+    GtkWidget * top = gtk_menu_new( );
+
+    m = gtk_menu_item_new_with_label( _( "Limit Download Speed" ) );
+    gtk_menu_item_set_submenu( GTK_MENU_ITEM( m ), createSpeedMenu( p, TR_DOWN ) );
+    gtk_menu_shell_append( GTK_MENU_SHELL( top ), m );
+
+    m = gtk_menu_item_new_with_label( _( "Limit Upload Speed" ) );
+    gtk_menu_item_set_submenu( GTK_MENU_ITEM( m ), createSpeedMenu( p, TR_UP ) );
+    gtk_menu_shell_append( GTK_MENU_SHELL( top ), m );
+
+    m = gtk_separator_menu_item_new( );
+    gtk_menu_shell_append( GTK_MENU_SHELL( top ), m );
+
+    m = gtk_menu_item_new_with_label( _( "Stop Seeding at Ratio" ) );
+    gtk_menu_item_set_submenu( GTK_MENU_ITEM( m ), createRatioMenu( p ) );
+    gtk_menu_shell_append( GTK_MENU_SHELL( top ), m );
+
+    gtk_widget_show_all( top );
+    return top;
+}
+
+static void
+onOptionsClicked( GtkButton * button UNUSED, gpointer vp )
+{
+    char buf1[512];
+    char buf2[512];
+    gboolean b;
+    GtkWidget * w;
+    PrivateData * p = vp;
+
+    w = p->speedlimit_on_item[TR_DOWN];
+    tr_strlspeed( buf1, pref_int_get( TR_PREFS_KEY_DSPEED ), sizeof( buf1 ) );
+    gtk_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( w ) ) ), buf1 );
+
+    b = pref_flag_get( TR_PREFS_KEY_DSPEED_ENABLED );
+    w = b ? p->speedlimit_on_item[TR_DOWN] : p->speedlimit_off_item[TR_DOWN];
+    gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( w ), TRUE );
+
+    w = p->speedlimit_on_item[TR_UP];
+    tr_strlspeed( buf1, pref_int_get( TR_PREFS_KEY_USPEED ), sizeof( buf1 ) );
+    gtk_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( w ) ) ), buf1 );
+
+    b = pref_flag_get( TR_PREFS_KEY_USPEED_ENABLED );
+    w = b ? p->speedlimit_on_item[TR_UP] : p->speedlimit_off_item[TR_UP];
+    gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( w ), TRUE );
+
+    tr_strlratio( buf1, pref_double_get( TR_PREFS_KEY_RATIO ), sizeof( buf1 ) );
+    g_snprintf( buf2, sizeof( buf2 ), _( "Stop at Ratio (%s)" ), buf1 );
+    gtk_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( p->ratio_on_item ) ) ), buf2 );
+
+    b = pref_flag_get( TR_PREFS_KEY_RATIO_ENABLED );
+    gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( b ? p->ratio_on_item : p->ratio_off_item ), TRUE );
+
+    gtk_menu_popup ( GTK_MENU( p->options_menu ), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time( ) );
+}
+
 /***
 ****  PUBLIC
 ***/
@@ -501,13 +825,12 @@ GtkWidget *
 tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
 {
     int           i, n;
-    const char *  pch;
+    const char  * pch;
     PrivateData * p;
-    GtkWidget   *mainmenu, *toolbar, *filter, *list, *status;
-    GtkWidget *   vbox, *w, *self, *h, *c, *s, *image, *menu;
-    GtkWindow *   win;
-    GSList *      l;
-    GSList *      toggles;
+    GtkWidget   * mainmenu, *toolbar, *filter, *list, *status;
+    GtkWidget   * vbox, *w, *self, *h, *s, *hbox, *menu;
+    GtkWindow   * win;
+    GSList      * l;
 
     const char *  filter_names[FILTER_MODE_QTY] = {
         /* show all torrents */
@@ -526,7 +849,6 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
     };
 
     p = g_new0( PrivateData, 1 );
-    p->filter_mode = FILTER_MODE_ALL;
     p->filter_text_mode = FILTER_TEXT_MODE_NAME;
     p->filter_text = NULL;
 
@@ -541,7 +863,9 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
                                  pref_int_get( PREF_KEY_MAIN_WINDOW_WIDTH ),
                                  pref_int_get( PREF_KEY_MAIN_WINDOW_HEIGHT ) );
     gtk_window_move( win, pref_int_get( PREF_KEY_MAIN_WINDOW_X ),
-                     pref_int_get( PREF_KEY_MAIN_WINDOW_Y ) );
+                          pref_int_get( PREF_KEY_MAIN_WINDOW_Y ) );
+    if( pref_flag_get( PREF_KEY_MAIN_WINDOW_IS_MAXIMIZED ) )
+        gtk_window_maximize( win );
     gtk_window_add_accel_group( win, gtk_ui_manager_get_accel_group( ui_mgr ) );
 
     /* window's main container */
@@ -560,33 +884,38 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
     w = toolbar = p->toolbar = action_get_widget( "/main-window-toolbar" );
 
     /* filter */
-    toggles = NULL;
     h = filter = p->filter = gtk_hbox_new( FALSE, 0 );
     gtk_container_set_border_width( GTK_CONTAINER( h ), GUI_PAD_SMALL );
     for( i = 0; i < FILTER_MODE_QTY; ++i )
     {
         const char * mnemonic = _( filter_names[i] );
         w = gtk_toggle_button_new_with_mnemonic( mnemonic );
-        g_object_set_data( G_OBJECT( w ), FILTER_MODE_KEY,
-                          GINT_TO_POINTER( i ) );
+        g_object_set_data( G_OBJECT( w ), FILTER_MODE_KEY, GINT_TO_POINTER( i ) );
         gtk_button_set_relief( GTK_BUTTON( w ), GTK_RELIEF_NONE );
-        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(
-                                          w ), i == FILTER_MODE_ALL );
-        toggles = g_slist_prepend( toggles, w );
+        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), i == FILTER_MODE_ALL );
+        p->filter_toggles[i] = GTK_TOGGLE_BUTTON( w );
         g_signal_connect( w, "toggled", G_CALLBACK( filter_toggled_cb ), p );
         gtk_box_pack_start( GTK_BOX( h ), w, FALSE, FALSE, 0 );
     }
-    for( l = toggles; l != NULL; l = l->next )
-        g_object_set_data( G_OBJECT( l->data ), FILTER_TOGGLES_KEY, toggles );
+
+#ifdef USE_SEXY
     s = sexy_icon_entry_new( );
     sexy_icon_entry_add_clear_button( SEXY_ICON_ENTRY( s ) );
-    image = gtk_image_new_from_stock( GTK_STOCK_FIND, GTK_ICON_SIZE_MENU );
-    sexy_icon_entry_set_icon( SEXY_ICON_ENTRY(
-                                 s ), SEXY_ICON_ENTRY_PRIMARY,
-                             GTK_IMAGE( image ) );
-    sexy_icon_entry_set_icon_highlight( SEXY_ICON_ENTRY(
-                                            s ), SEXY_ICON_ENTRY_PRIMARY,
-                                        TRUE );
+    w = gtk_image_new_from_stock( GTK_STOCK_FIND, GTK_ICON_SIZE_MENU );
+    sexy_icon_entry_set_icon( SEXY_ICON_ENTRY( s ),
+                              SEXY_ICON_ENTRY_PRIMARY,
+                              GTK_IMAGE( w ) );
+    sexy_icon_entry_set_icon_highlight( SEXY_ICON_ENTRY( s ),
+                                        SEXY_ICON_ENTRY_PRIMARY, TRUE );
+#else
+    s = gtk_entry_new( );
+    gtk_entry_set_icon_from_stock( GTK_ENTRY( s ),
+                                   GTK_ENTRY_ICON_PRIMARY,
+                                   GTK_STOCK_FIND);
+   gtk_entry_set_icon_from_stock( GTK_ENTRY( s ),
+                                  GTK_ENTRY_ICON_SECONDARY,
+                                  GTK_STOCK_CLEAR );
+#endif
     gtk_box_pack_end( GTK_BOX( h ), s, FALSE, FALSE, 0 );
     g_signal_connect( s, "changed", G_CALLBACK( filter_entry_changed ), p );
 
@@ -612,36 +941,64 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
 
     /* status */
     h = status = p->status = gtk_hbox_new( FALSE, GUI_PAD );
-    gtk_container_set_border_width( GTK_CONTAINER( h ), GUI_PAD );
-    w = p->gutter_lb = gtk_label_new( "N Torrents" );
-    gtk_box_pack_start( GTK_BOX( h ), w, 0, 0, 0 );
-    w = p->ul_lb = gtk_label_new( NULL );
-    gtk_box_pack_end( GTK_BOX( h ), w, FALSE, FALSE, 0 );
-    w = gtk_image_new_from_stock( GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU );
-    gtk_box_pack_end( GTK_BOX( h ), w, FALSE, FALSE, 0 );
-    w = gtk_alignment_new( 0.0f, 0.0f, 0.0f, 0.0f );
-    gtk_widget_set_size_request( w, GUI_PAD, 0u );
-    gtk_box_pack_end( GTK_BOX( h ), w, FALSE, FALSE, 0 );
-    w = p->dl_lb = gtk_label_new( NULL );
-    gtk_box_pack_end( GTK_BOX( h ), w, FALSE, FALSE, 0 );
-    w = gtk_image_new_from_stock( GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_MENU );
-    gtk_box_pack_end( GTK_BOX( h ), w, FALSE, FALSE, 0 );
-    w = gtk_alignment_new( 0.0f, 0.0f, 0.0f, 0.0f );
-    gtk_widget_set_size_request( w, GUI_PAD, 0u );
-    gtk_box_pack_end( GTK_BOX( h ), w, FALSE, FALSE, 0 );
-    w = p->stats_lb = gtk_label_new( NULL );
-    gtk_box_pack_end( GTK_BOX( h ), w, FALSE, FALSE, 0 );
-    w = gtk_image_new_from_stock( GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU );
-    c = gtk_event_box_new( );
-    gtk_container_add( GTK_CONTAINER( c ), w );
-    w = c;
-    gtk_box_pack_end( GTK_BOX( h ), w, FALSE, FALSE, 0 );
-    g_signal_connect( w, "button-release-event",
-                      G_CALLBACK( onYinYangReleased ), p );
+    gtk_container_set_border_width( GTK_CONTAINER( h ), GUI_PAD_SMALL );
+
+        w = gtk_button_new( );
+        gtk_container_add( GTK_CONTAINER( w ), gtk_image_new_from_stock( "options", GTK_ICON_SIZE_SMALL_TOOLBAR ) );
+        gtk_box_pack_start( GTK_BOX( h ), w, 0, 0, 0 );
+        gtk_button_set_relief( GTK_BUTTON( w ), GTK_RELIEF_NONE );
+        p->options_menu = createOptionsMenu( p );
+        g_signal_connect( w, "clicked", G_CALLBACK(onOptionsClicked), p );
+
+        p->alt_speed_image[0] = gtk_image_new_from_stock( "alt-speed-off", -1 );
+        p->alt_speed_image[1]  = gtk_image_new_from_stock( "alt-speed-on", -1 );
+        w = p->alt_speed_button = gtk_toggle_button_new( );
+        gtk_button_set_relief( GTK_BUTTON( w ), GTK_RELIEF_NONE );
+        g_object_ref( G_OBJECT( p->alt_speed_image[0] ) );
+        g_object_ref( G_OBJECT( p->alt_speed_image[1] ) );
+        g_signal_connect( w, "toggled", G_CALLBACK(alt_speed_toggled_cb ), p );
+        gtk_box_pack_start( GTK_BOX( h ), w, 0, 0, 0 );
+
+        w = p->gutter_lb = gtk_label_new( "N Torrents" );
+        gtk_box_pack_start( GTK_BOX( h ), w, 1, 1, GUI_PAD_BIG );
+
+        hbox = p->ul_hbox = gtk_hbox_new( FALSE, GUI_PAD_SMALL );
+            w = gtk_alignment_new( 0.0f, 0.0f, 0.0f, 0.0f );
+            gtk_widget_set_size_request( w, GUI_PAD, 0u );
+            gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
+            w = gtk_image_new_from_stock( GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU );
+            gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
+            w = p->ul_lb = gtk_label_new( NULL );
+            gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 ); 
+        gtk_box_pack_end( GTK_BOX( h ), hbox, FALSE, FALSE, 0 );
+
+        hbox = p->dl_hbox = gtk_hbox_new( FALSE, GUI_PAD_SMALL );
+            w = gtk_alignment_new( 0.0f, 0.0f, 0.0f, 0.0f );
+            gtk_widget_set_size_request( w, GUI_PAD, 0u );
+            gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
+            w = gtk_image_new_from_stock( GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_MENU );
+            gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
+            w = p->dl_lb = gtk_label_new( NULL );
+            gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 ); 
+        gtk_box_pack_end( GTK_BOX( h ), hbox, FALSE, FALSE, 0 );
+
+        hbox = gtk_hbox_new( FALSE, GUI_PAD_SMALL );
+            w = gtk_alignment_new( 0.0f, 0.0f, 0.0f, 0.0f );
+            gtk_widget_set_size_request( w, GUI_PAD, 0u );
+            gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
+            w = gtk_button_new( );
+            gtk_container_add( GTK_CONTAINER( w ), gtk_image_new_from_stock( GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU ) );
+            gtk_button_set_relief( GTK_BUTTON( w ), GTK_RELIEF_NONE );
+            g_signal_connect( w, "clicked", G_CALLBACK( onYinYangReleased ), p );
+            gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
+            w = p->stats_lb = gtk_label_new( NULL );
+            gtk_box_pack_end( GTK_BOX( h ), w, FALSE, FALSE, 0 );
+        gtk_box_pack_end( GTK_BOX( h ), hbox, FALSE, FALSE, 0 );
+
 
     menu = gtk_menu_new( );
     l = NULL;
-    for( i = 0; i < FILTER_TEXT_MODE_QTY; ++i )
+    for( i=0; i<FILTER_TEXT_MODE_QTY; ++i )
     {
         const char * name = _( filter_text_names[i] );
         GtkWidget *  w = gtk_radio_menu_item_new_with_label ( l, name );
@@ -653,8 +1010,13 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
         gtk_menu_shell_append( GTK_MENU_SHELL( menu ), w );
         gtk_widget_show( w );
     }
-    g_signal_connect( s, "icon-released",
-                      G_CALLBACK( entry_icon_released ), menu );
+
+#ifdef USE_SEXY
+    g_signal_connect( s, "icon-released", G_CALLBACK( entry_icon_released ), menu );
+#else
+    g_signal_connect( s, "icon-release", G_CALLBACK( entry_icon_release ), menu );
+
+#endif
 
     /* workarea */
     p->view = makeview( p, core );
@@ -697,9 +1059,13 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
     prefsChanged( core, PREF_KEY_STATUSBAR, self );
     prefsChanged( core, PREF_KEY_STATUSBAR_STATS, self );
     prefsChanged( core, PREF_KEY_TOOLBAR, self );
+    prefsChanged( core, PREF_KEY_FILTER_MODE, self );
+    prefsChanged( core, TR_PREFS_KEY_ALT_SPEED_ENABLED, self );
     p->pref_handler_id = g_signal_connect( core, "prefs-changed",
                                            G_CALLBACK( prefsChanged ), self );
 
+    tr_sessionSetAltSpeedFunc( tr_core_session( core ), onAltSpeedToggled, p );
+
     filter_entry_changed( GTK_EDITABLE( s ), p );
     return self;
 }
@@ -715,16 +1081,17 @@ updateTorrentCount( PrivateData * p )
         const int visibleCount = gtk_tree_model_iter_n_children(
             p->filter_model, NULL );
 
-        if( torrentCount != visibleCount )
+        if( !torrentCount )
+            *buf = '\0';
+        else if( torrentCount != visibleCount )
             g_snprintf( buf, sizeof( buf ),
                         ngettext( "%1$'d of %2$'d Torrent",
                                   "%1$'d of %2$'d Torrents",
                                   torrentCount ),
                         visibleCount, torrentCount );
         else
-            g_snprintf( buf, sizeof( buf ), ngettext( "%'d Torrent",
-                                                      "%'d Torrents",
-                                                      torrentCount ),
+            g_snprintf( buf, sizeof( buf ),
+                        ngettext( "%'d Torrent", "%'d Torrents", torrentCount ),
                         torrentCount );
         gtk_label_set_text( GTK_LABEL( p->gutter_lb ), buf );
     }
@@ -790,10 +1157,12 @@ updateSpeeds( PrivateData * p )
         d = tr_sessionGetPieceSpeed( session, TR_DOWN );
         tr_strlspeed( buf, d, sizeof( buf ) );
         gtk_label_set_text( GTK_LABEL( p->dl_lb ), buf );
+        g_object_set( p->dl_hbox, "visible", d>=0.01, NULL );
 
         d = tr_sessionGetPieceSpeed( session, TR_UP );
         tr_strlspeed( buf, d, sizeof( buf ) );
         gtk_label_set_text( GTK_LABEL( p->ul_lb ), buf );
+        g_object_set( p->ul_hbox, "visible", d>=0.01, NULL );
     }
 }
 
