@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: util.c 7776 2009-01-22 22:04:37Z charles $
+ * $Id: util.c 8758 2009-06-29 17:52:24Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -38,10 +38,11 @@
  #include <dbus/dbus-glib.h>
 #endif
 
-#include <libevent/evhttp.h>
+#include <evhttp.h>
 
 #include <libtransmission/transmission.h> /* TR_RATIO_NA, TR_RATIO_INF */
 #include <libtransmission/utils.h> /* tr_inf */
+#include <libtransmission/version.h> /* tr_inf */
 
 #include "conf.h"
 #include "hig.h"
@@ -49,21 +50,9 @@
 #include "util.h"
 
 char*
-tr_strlratio( char * buf,
-              double ratio,
-              size_t buflen )
+tr_strlratio( char * buf, double ratio, size_t buflen )
 {
-    if( (int)ratio == TR_RATIO_NA )
-        g_strlcpy( buf, _( "None" ), buflen );
-    else if( (int)ratio == TR_RATIO_INF )
-        g_strlcpy( buf, "\xE2\x88\x9E", buflen );
-    else if( ratio < 10.0 )
-        g_snprintf( buf, buflen, "%'.2f", ratio );
-    else if( ratio < 100.0 )
-        g_snprintf( buf, buflen, "%'.1f", ratio );
-    else
-        g_snprintf( buf, buflen, "%'.0f", ratio );
-    return buf;
+    return tr_strratio( buf, buflen, ratio, "\xE2\x88\x9E" );
 }
 
 #define KILOBYTE_FACTOR 1024.0
@@ -122,11 +111,11 @@ tr_strlspeed( char * buf,
     if( speed < 1000.0 )  /* 0.0 KB to 999.9 KB */
         g_snprintf( buf, buflen, _( "%'.1f KB/s" ), speed );
     else if( speed < 102400.0 ) /* 0.98 MB to 99.99 MB */
-        g_snprintf( buf, buflen, _( "%'.2f MB/s" ), ( speed / 1024 ) );
+        g_snprintf( buf, buflen, _( "%'.2f MB/s" ), ( speed / KILOBYTE_FACTOR ) );
     else if( speed < 1024000.0 ) /* 100.0 MB to 999.9 MB */
-        g_snprintf( buf, buflen, _( "%'.1f MB/s" ), ( speed / 1024 ) );
+        g_snprintf( buf, buflen, _( "%'.1f MB/s" ), ( speed / MEGABYTE_FACTOR ) );
     else /* insane speeds */
-        g_snprintf( buf, buflen, _( "%'.2f GB/s" ), ( speed / 1048576 ) );
+        g_snprintf( buf, buflen, _( "%'.2f GB/s" ), ( speed / GIGABYTE_FACTOR ) );
 
     return buf;
 }
@@ -210,6 +199,15 @@ gtr_localtime( time_t time )
     return g_locale_to_utf8( buf, -1, NULL, NULL, NULL );
 }
 
+char *
+gtr_localtime2( char * buf, time_t time, size_t buflen )
+{
+    char * tmp = gtr_localtime( time );
+    g_strlcpy( buf, tmp, buflen );
+    g_free( tmp );
+    return buf;
+}
+
 int
 mkdir_p( const char * path,
          mode_t       mode )
@@ -395,7 +393,7 @@ addTorrentErrorDialog( GtkWidget *  child,
                                GTK_DIALOG_DESTROY_WITH_PARENT,
                                GTK_MESSAGE_ERROR,
                                GTK_BUTTONS_CLOSE,
-                               _( "Error opening torrent" ) );
+                               "%s", _( "Error opening torrent" ) );
     gtk_message_dialog_format_secondary_text( GTK_MESSAGE_DIALOG( w ),
                                               "%s", secondary );
     g_signal_connect_swapped( w, "response",
@@ -569,7 +567,7 @@ gtr_open_file( const char * path )
 #endif
         if( !opened )
         {
-            char * argv[] = { "xdg-open", (char*)path, NULL };
+            char * argv[] = { (char*)"xdg-open", (char*)path, NULL };
             g_spawn_async( NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
                            NULL, NULL, NULL, NULL );
         }
@@ -583,29 +581,43 @@ gtr_open_file( const char * path )
 gboolean
 gtr_dbus_add_torrent( const char * filename )
 {
-    static gboolean   success = FALSE;
-
+    /* FIXME: why is this static? */
+    static gboolean success = FALSE;
 #ifdef HAVE_DBUS_GLIB
-    DBusGProxy *      proxy = NULL;
-    GError *          err = NULL;
+    DBusGProxy * proxy = NULL;
+    GError * err = NULL;
     DBusGConnection * conn;
-    if( ( conn = dbus_g_bus_get( DBUS_BUS_SESSION, &err ) ) )
-        proxy = dbus_g_proxy_new_for_name ( conn, VALUE_SERVICE_NAME,
-                                            VALUE_SERVICE_OBJECT_PATH,
-                                            VALUE_SERVICE_INTERFACE );
-    else if( err )
-        g_message( "err: %s", err->message );
-    if( proxy )
-        dbus_g_proxy_call( proxy, "AddFile", &err,
-                           G_TYPE_STRING, filename,
-                           G_TYPE_INVALID,
-                           G_TYPE_BOOLEAN, &success,
-                           G_TYPE_INVALID );
-    if( err )
-        g_message( "err: %s", err->message );
+    char * file_contents;
+    gsize file_length;
+
+    if( g_file_get_contents( filename, &file_contents, &file_length, NULL ) )
+    {
+        char * b64 = tr_base64_encode( file_contents, file_length, NULL );
+        if(( conn = dbus_g_bus_get( DBUS_BUS_SESSION, &err )))
+            proxy = dbus_g_proxy_new_for_name (conn, VALUE_SERVICE_NAME,
+                                                     VALUE_SERVICE_OBJECT_PATH,
+                                                     VALUE_SERVICE_INTERFACE );
+        else if( err )
+           g_message( "err: %s", err->message );
+        if( proxy )
+            dbus_g_proxy_call( proxy, "AddMetainfo", &err,
+                               G_TYPE_STRING, b64,
+                               G_TYPE_INVALID,
+                               G_TYPE_BOOLEAN, &success,
+                               G_TYPE_INVALID );
+        if( err )
+           g_message( "err: %s", err->message );
+
+        if( proxy )
+            g_object_unref( proxy );
+        if( conn )
+            dbus_g_connection_unref( conn );
+
+        tr_free( b64 );
+        g_free( file_contents );
+    }
+    else g_message( "couldn't read %s", filename );
 
-    g_object_unref( proxy );
-    dbus_g_connection_unref( conn );
 #endif
     return success;
 }
@@ -655,14 +667,82 @@ gtr_button_new_from_stock( const char * stock,
 ****
 ***/
 
-guint
-gtr_timeout_add_seconds( guint        interval,
-                         GSourceFunc  function,
-                         gpointer     data )
+void
+gtr_widget_set_tooltip_text( GtkWidget * w, const char * tip )
 {
-#if GLIB_CHECK_VERSION( 2,14,0 )
-    return g_timeout_add_seconds( interval, function, data );
+#if GTK_CHECK_VERSION( 2,12,0 )
+    gtk_widget_set_tooltip_text( w, tip );
 #else
-    return g_timeout_add( interval*1000, function, data );
+    static GtkTooltips * tips = NULL;
+    if( tips == NULL )
+        tips = gtk_tooltips_new( );
+    gtk_tooltips_set_tip( tips, w, tip, NULL );
+#endif
+}
+
+void
+gtr_toolbar_set_orientation( GtkToolbar      * toolbar,
+                             GtkOrientation    orientation )
+{
+#if GTK_CHECK_VERSION( 2,16,0 )
+    gtk_orientable_set_orientation( GTK_ORIENTABLE( toolbar ), orientation );
+#else
+    gtk_toolbar_set_orientation( toolbar, orientation );
+#endif
+}
+
+/***
+****
+***/
+
+#if !GTK_CHECK_VERSION( 2,12,0 )
+struct gtr_func_data
+{
+    GSourceFunc function;
+    gpointer data;
+};
+
+static gboolean
+gtr_thread_func( gpointer data )
+{
+    struct gtr_func_data * idle_data = data;
+    gboolean more;
+
+    gdk_threads_enter( );
+    more = idle_data->function( idle_data->data );
+    gdk_threads_leave( );
+
+    if( !more )
+        g_free( data );
+
+    return more;
+}
+#endif
+
+void
+gtr_idle_add( GSourceFunc function, gpointer data )
+{
+#if GTK_CHECK_VERSION( 2,12,0 )
+    gdk_threads_add_idle( function, data );
+#else
+    struct gtr_func_data * d = g_new( struct gtr_func_data, 1 );
+    d->function = function;
+    d->data = data;
+    g_idle_add( gtr_thread_func, d );
+#endif
+}
+
+guint
+gtr_timeout_add_seconds( guint seconds, GSourceFunc function, gpointer data )
+{
+#if GTK_CHECK_VERSION( 2,14,0 )
+    return gdk_threads_add_timeout_seconds( seconds, function, data );
+#elif GTK_CHECK_VERSION( 2,12,0 )
+    return gdk_threads_add_timeout( seconds*1000, function, data );
+#else
+    struct gtr_func_data * d = g_new( struct gtr_func_data, 1 );
+    d->function = function;
+    d->data = data;
+    return g_timeout_add( seconds*1000, gtr_thread_func, d );
 #endif
 }
