@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: util.h 7776 2009-01-22 22:04:37Z charles $
+ * $Id: util.h 8757 2009-06-29 17:22:35Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -61,6 +61,8 @@ char*       tr_strltime( char * buf,
 
 char*       gtr_localtime( time_t time );
 
+char*       gtr_localtime2( char * buf, time_t time, size_t buflen );
+
 /* create a directory and any missing parent directories */
 int         mkdir_p( const char *name,
                      mode_t      mode );
@@ -92,13 +94,21 @@ char*       gtr_get_help_url( void );
 
 #ifdef GTK_MAJOR_VERSION
 
-GtkWidget * gtr_button_new_from_stock( const char * stock,
-                                       const char * mnemonic );
-
-guint       gtr_timeout_add_seconds( guint       interval,
+guint       gtr_timeout_add_seconds( guint       seconds,
                                      GSourceFunc function,
                                      gpointer    data );
 
+void        gtr_idle_add( GSourceFunc  func, 
+                          gpointer     data );
+
+void        gtr_toolbar_set_orientation( GtkToolbar      * toolbar,
+                                         GtkOrientation    orientation );
+
+void        gtr_widget_set_tooltip_text( GtkWidget * w, const char * tip );
+
+GtkWidget * gtr_button_new_from_stock( const char * stock,
+                                       const char * mnemonic );
+
 void        addTorrentErrorDialog( GtkWidget *  window_or_child,
                                    int          err,
                                    const char * filename );
