@@ -7,17 +7,19 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: bencode.c 7908 2009-02-18 16:47:28Z charles $
+ * $Id: bencode.c 8825 2009-07-17 18:00:29Z charles $
  */
 
 #include <assert.h>
 #include <ctype.h> /* isdigit, isprint, isspace */
 #include <errno.h>
+#include <math.h> /* fabs */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include <locale.h>
+#include <unistd.h> /* close() */
 
 #include <event.h> /* evbuffer */
 
@@ -47,7 +49,10 @@ isContainer( const tr_benc * val )
 static tr_bool
 isSomething( const tr_benc * val )
 {
-    return isContainer( val ) || tr_bencIsInt( val ) || tr_bencIsString( val );
+    return isContainer( val ) || tr_bencIsInt( val )
+                              || tr_bencIsString( val )
+                              || tr_bencIsReal( val )
+                              || tr_bencIsBool( val );
 }
 
 static void
@@ -151,13 +156,13 @@ tr_bencParseStr( const uint8_t *  buf,
 }
 
 /* set to 1 to help expose bugs with tr_bencListAdd and tr_bencDictAdd */
-#define LIST_SIZE 8 /* number of items to increment list/dict buffer by */
+#define LIST_SIZE 4 /* number of items to increment list/dict buffer by */
 
 static int
 makeroom( tr_benc * val,
           size_t    count )
 {
-    assert( TYPE_LIST == val->type || TYPE_DICT == val->type );
+    assert( TR_TYPE_LIST == val->type || TR_TYPE_DICT == val->type );
 
     if( val->val.l.count + count > val->val.l.alloc )
     {
@@ -193,8 +198,8 @@ getNode( tr_benc *     top,
     assert( parent );
 
     /* dictionary keys must be strings */
-    if( ( parent->type == TYPE_DICT )
-      && ( type != TYPE_STR )
+    if( ( parent->type == TR_TYPE_DICT )
+      && ( type != TR_TYPE_STR )
       && ( !( parent->val.l.count % 2 ) ) )
         return NULL;
 
@@ -234,7 +239,7 @@ tr_bencParseImpl( const void *     buf_in,
             if( ( err = tr_bencParseInt( buf, bufend, &end, &val ) ) )
                 return err;
 
-            node = getNode( top, parentStack, TYPE_INT );
+            node = getNode( top, parentStack, TR_TYPE_INT );
             if( !node )
                 return EILSEQ;
 
@@ -246,19 +251,19 @@ tr_bencParseImpl( const void *     buf_in,
         }
         else if( *buf == 'l' ) /* list */
         {
-            tr_benc * node = getNode( top, parentStack, TYPE_LIST );
+            tr_benc * node = getNode( top, parentStack, TR_TYPE_LIST );
             if( !node )
                 return EILSEQ;
-            tr_bencInit( node, TYPE_LIST );
+            tr_bencInit( node, TR_TYPE_LIST );
             tr_ptrArrayAppend( parentStack, node );
             ++buf;
         }
         else if( *buf == 'd' ) /* dict */
         {
-            tr_benc * node = getNode( top, parentStack, TYPE_DICT );
+            tr_benc * node = getNode( top, parentStack, TR_TYPE_DICT );
             if( !node )
                 return EILSEQ;
-            tr_bencInit( node, TYPE_DICT );
+            tr_bencInit( node, TR_TYPE_DICT );
             tr_ptrArrayAppend( parentStack, node );
             ++buf;
         }
@@ -291,7 +296,7 @@ tr_bencParseImpl( const void *     buf_in,
             if( ( err = tr_bencParseStr( buf, bufend, &end, &str, &str_len ) ) )
                 return err;
 
-            node = getNode( top, parentStack, TYPE_STR );
+            node = getNode( top, parentStack, TR_TYPE_STR );
             if( !node )
                 return EILSEQ;
 
@@ -352,9 +357,30 @@ tr_bencLoad( const void * buf_in,
 ****
 ***/
 
+/* returns true if the given string length would fit in our string buffer */
+static TR_INLINE int
+stringFitsInBuffer( const tr_benc * val, int len )
+{
+    return len < (int)sizeof( val->val.s.str.buf );
+}
+
+/* returns true if the benc's string was malloced.
+ * this occurs when the string is too long for our string buffer */
+static TR_INLINE int
+stringIsAlloced( const tr_benc * val )
+{
+    return !stringFitsInBuffer( val, val->val.s.len );
+}
+
+/* returns a const pointer to the benc's string */
+static TR_INLINE const char*
+getStr( const tr_benc* val )
+{
+    return stringIsAlloced(val) ? val->val.s.str.ptr : val->val.s.str.buf;
+}
+
 static int
-dictIndexOf( const tr_benc * val,
-             const char *    key )
+dictIndexOf( const tr_benc * val, const char * key )
 {
     if( tr_bencIsDict( val ) )
     {
@@ -364,10 +390,9 @@ dictIndexOf( const tr_benc * val,
         for( i = 0; ( i + 1 ) < val->val.l.count; i += 2 )
         {
             const tr_benc * child = val->val.l.vals + i;
-
-            if( ( child->type == TYPE_STR )
-              && ( child->val.s.i == len )
-              && !memcmp( child->val.s.s, key, len ) )
+            if( ( child->type == TR_TYPE_STR )
+              && ( child->val.s.len == len )
+              && !memcmp( getStr(child), key, len ) )
                 return i;
         }
     }
@@ -383,14 +408,10 @@ tr_bencDictFind( tr_benc * val, const char * key )
     return i < 0 ? NULL : &val->val.l.vals[i + 1];
 }
 
-static tr_benc*
-tr_bencDictFindType( tr_benc *    val,
-                     const char * key,
-                     int          type )
+static tr_bool
+tr_bencDictFindType( tr_benc * dict, const char * key, int type, tr_benc ** setme )
 {
-    tr_benc * ret = tr_bencDictFind( val, key );
-
-    return ( ret && ( ret->type == type ) ) ? ret : NULL;
+    return tr_bencIsType( *setme = tr_bencDictFind( dict, key ), type );
 }
 
 size_t
@@ -410,14 +431,27 @@ tr_bencListChild( tr_benc * val,
     return ret;
 }
 
+static void
+tr_benc_warning( const char * err )
+{
+    fprintf( stderr, "warning: %s\n", err );
+}
+
 tr_bool
 tr_bencGetInt( const tr_benc * val,
                int64_t *       setme )
 {
-    const int success = tr_bencIsInt( val );
+    tr_bool success = FALSE;
 
-    if( success && setme )
-        *setme = val->val.i;
+    if( !success && (( success = tr_bencIsInt( val ))))
+        if( setme )
+            *setme = val->val.i;
+
+    if( !success && (( success = tr_bencIsBool( val )))) {
+        tr_benc_warning( "reading bool as an int" );
+        if( setme )
+            *setme = val->val.b ? 1 : 0;
+    }
 
     return success;
 }
@@ -429,80 +463,96 @@ tr_bencGetStr( const tr_benc * val,
     const int success = tr_bencIsString( val );
 
     if( success )
-        *setme = val->val.s.s;
+        *setme = getStr( val );
+
+    return success;
+}
+
+tr_bool
+tr_bencGetBool( const tr_benc * val, tr_bool * setme )
+{
+    const char * str;
+    tr_bool success = FALSE;
+
+    if(( success = tr_bencIsBool( val )))
+        *setme = val->val.b;
+
+    if( !success && tr_bencIsInt( val ) )
+        if(( success = ( val->val.i==0 || val->val.i==1 ) ))
+            *setme = val->val.i!=0;
+
+    if( !success && tr_bencGetStr( val, &str ) )
+        if(( success = ( !strcmp(str,"true") || !strcmp(str,"false"))))
+            *setme = !strcmp(str,"true");
+
+    return success;
+}
+
+tr_bool
+tr_bencGetReal( const tr_benc * val, double * setme )
+{
+    tr_bool success = FALSE;
+
+    if( !success && (( success = tr_bencIsReal( val ))))
+        *setme = val->val.d;
+
+    if( !success && (( success = tr_bencIsInt( val ))))
+        *setme = val->val.i;
+
+    if( !success && tr_bencIsString(val) )
+    {
+        char * endptr;
+        char locale[128];
+        double d;
+
+        /* the json spec requires a '.' decimal point regardless of locale */
+        tr_strlcpy( locale, setlocale( LC_NUMERIC, NULL ), sizeof( locale ) );
+        setlocale( LC_NUMERIC, "POSIX" );
+        d  = strtod( getStr(val), &endptr );
+        setlocale( LC_NUMERIC, locale );
+
+        if(( success = ( getStr(val) != endptr ) && !*endptr ))
+            *setme = d;
+    }
+
+
     return success;
 }
 
 tr_bool
 tr_bencDictFindInt( tr_benc * dict, const char * key, int64_t * setme )
 {
-    tr_bool found = FALSE;
-    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_INT );
-
-    if( child )
-        found = tr_bencGetInt( child, setme );
-
-    return found;
+    return tr_bencGetInt( tr_bencDictFind( dict, key ), setme );
 }
 
 tr_bool
-tr_bencDictFindDouble( tr_benc * dict, const char * key, double * setme )
+tr_bencDictFindBool( tr_benc * dict, const char * key, tr_bool * setme )
 {
-    const char * str;
-    const tr_bool success = tr_bencDictFindStr( dict, key, &str );
-
-    if( success && setme )
-        *setme = strtod( str, NULL );
-
-    return success;
+    return tr_bencGetBool( tr_bencDictFind( dict, key ), setme );
 }
 
 tr_bool
-tr_bencDictFindList( tr_benc * dict, const char * key, tr_benc ** setme )
+tr_bencDictFindReal( tr_benc * dict, const char * key, double * setme )
 {
-    tr_bool found = FALSE;
-    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_LIST );
-
-    if( child )
-    {
-        if( setme != NULL )
-            *setme = child;
-        found = TRUE;
-    }
-
-    return found;
-}
-
-tr_bool
-tr_bencDictFindDict( tr_benc * dict, const char * key, tr_benc ** setme )
-{
-    tr_bool found = FALSE;
-    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_DICT );
-
-    if( child )
-    {
-        if( setme != NULL )
-            *setme = child;
-        found = TRUE;
-    }
-
-    return found;
+    return tr_bencGetReal( tr_bencDictFind( dict, key ), setme );
 }
 
 tr_bool
 tr_bencDictFindStr( tr_benc *  dict, const char *  key, const char ** setme )
 {
-    tr_bool found = FALSE;
-    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_STR );
+    return tr_bencGetStr( tr_bencDictFind( dict, key ), setme );
+}
 
-    if( child )
-    {
-        if( setme )
-            *setme = child->val.s.s;
-        found = TRUE;
-    }
+tr_bool
+tr_bencDictFindList( tr_benc * dict, const char * key, tr_benc ** setme )
+{
+    return tr_bencDictFindType( dict, key, TR_TYPE_LIST, setme );
+}
 
-    return found;
+tr_bool
+tr_bencDictFindDict( tr_benc * dict, const char * key, tr_benc ** setme )
+{
+    return tr_bencDictFindType( dict, key, TR_TYPE_DICT, setme );
 }
 
 tr_bool
@@ -511,14 +561,12 @@ tr_bencDictFindRaw( tr_benc         * dict,
                     const uint8_t  ** setme_raw,
                     size_t          * setme_len )
 {
-    tr_bool found = FALSE;
-    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_STR );
+    tr_benc * child;
+    const tr_bool found = tr_bencDictFindType( dict, key, TR_TYPE_STR, &child );
 
-    if( child )
-    {
-        *setme_raw = (uint8_t*) child->val.s.s;
-        *setme_len = child->val.s.i;
-        found = TRUE;
+    if( found ) {
+        *setme_raw = (uint8_t*) getStr(child);
+        *setme_len = child->val.s.len;
     }
 
     return found;
@@ -529,70 +577,80 @@ tr_bencDictFindRaw( tr_benc         * dict,
 ***/
 
 void
-tr_bencInitRaw( tr_benc *    val,
-                const void * src,
-                size_t       byteCount )
+tr_bencInitRaw( tr_benc * val, const void * src, size_t byteCount )
 {
-    tr_bencInit( val, TYPE_STR );
-    val->val.s.i = byteCount;
-    val->val.s.s = tr_memdup( src, byteCount );
-}
+    tr_bencInit( val, TR_TYPE_STR );
 
-void
-tr_bencInitStr( tr_benc *    val,
-                const void * str,
-                int          len )
-{
-    tr_bencInit( val, TYPE_STR );
-
-    val->val.s.s = tr_strndup( str, len );
-
-    if( val->val.s.s == NULL )
-        val->val.s.i = 0;
-    else if( len < 0 )
-        val->val.s.i = strlen( val->val.s.s );
+    if( stringFitsInBuffer( val, val->val.s.len = byteCount ))
+        memcpy( val->val.s.str.buf, src, byteCount );
     else
-        val->val.s.i = len;
+        val->val.s.str.ptr = tr_memdup( src, byteCount );
 }
 
 void
-tr_bencInitInt( tr_benc * val,
-                int64_t   num )
+tr_bencInitStr( tr_benc * val, const void * str, int len )
 {
-    tr_bencInit( val, TYPE_INT );
-    val->val.i = num;
+    tr_bencInit( val, TR_TYPE_STR );
+
+    if( str == NULL )
+        len = 0;
+    else if( len < 0 )
+        len = strlen( str );
+
+    if( stringFitsInBuffer( val, val->val.s.len = len )) {
+        memcpy( val->val.s.str.buf, str, len );
+        val->val.s.str.buf[len] = '\0';
+    } else
+        val->val.s.str.ptr = tr_strndup( str, len );
+}
+
+void
+tr_bencInitBool( tr_benc * b, int value )
+{
+    tr_bencInit( b, TR_TYPE_BOOL );
+    b->val.b = value != 0;
+}
+
+void
+tr_bencInitReal( tr_benc * b, double value )
+{
+    tr_bencInit( b, TR_TYPE_REAL );
+    b->val.d = value;
+}
+
+void
+tr_bencInitInt( tr_benc * b, int64_t value )
+{
+    tr_bencInit( b, TR_TYPE_INT );
+    b->val.i = value;
 }
 
 int
-tr_bencInitList( tr_benc * val,
-                 size_t    reserveCount )
+tr_bencInitList( tr_benc * b, size_t reserveCount )
 {
-    tr_bencInit( val, TYPE_LIST );
-    return tr_bencListReserve( val, reserveCount );
+    tr_bencInit( b, TR_TYPE_LIST );
+    return tr_bencListReserve( b, reserveCount );
 }
 
 int
-tr_bencListReserve( tr_benc * val,
-                    size_t    count )
+tr_bencListReserve( tr_benc * b, size_t count )
 {
-    assert( tr_bencIsList( val ) );
-    return makeroom( val, count );
+    assert( tr_bencIsList( b ) );
+    return makeroom( b, count );
 }
 
 int
-tr_bencInitDict( tr_benc * val,
-                 size_t    reserveCount )
+tr_bencInitDict( tr_benc * b, size_t reserveCount )
 {
-    tr_bencInit( val, TYPE_DICT );
-    return tr_bencDictReserve( val, reserveCount );
+    tr_bencInit( b, TR_TYPE_DICT );
+    return tr_bencDictReserve( b, reserveCount );
 }
 
 int
-tr_bencDictReserve( tr_benc * val,
-                    size_t    reserveCount )
+tr_bencDictReserve( tr_benc * b, size_t reserveCount )
 {
-    assert( tr_bencIsDict( val ) );
-    return makeroom( val, reserveCount * 2 );
+    assert( tr_bencIsDict( b ) );
+    return makeroom( b, reserveCount * 2 );
 }
 
 tr_benc *
@@ -609,7 +667,7 @@ tr_bencListAdd( tr_benc * list )
 
     item = &list->val.l.vals[list->val.l.count];
     list->val.l.count++;
-    tr_bencInit( item, TYPE_INT );
+    tr_bencInit( item, TR_TYPE_INT );
 
     return item;
 }
@@ -669,21 +727,19 @@ tr_bencDictAdd( tr_benc *    dict,
     tr_bencInitStr( keyval, key, -1 );
 
     itemval = dict->val.l.vals + dict->val.l.count++;
-    tr_bencInit( itemval, TYPE_INT );
+    tr_bencInit( itemval, TR_TYPE_INT );
 
     return itemval;
 }
 
-tr_benc*
-tr_bencDictAddInt( tr_benc *    dict,
-                   const char * key,
-                   int64_t      val )
+static tr_benc*
+dictFindOrAdd( tr_benc * dict, const char * key, int type )
 {
     tr_benc * child;
 
     /* see if it already exists, and if so, try to reuse it */
     if(( child = tr_bencDictFind( dict, key ))) {
-        if( !tr_bencIsInt( child ) ) {
+        if( !tr_bencIsType( child, type ) ) {
             tr_bencDictRemove( dict, key );
             child = NULL;
         }
@@ -693,9 +749,32 @@ tr_bencDictAddInt( tr_benc *    dict,
     if( child == NULL )
         child = tr_bencDictAdd( dict, key );
 
-    /* set it */
-    tr_bencInitInt( child, val );
+    return child;
+}
 
+tr_benc*
+tr_bencDictAddInt( tr_benc *    dict,
+                   const char * key,
+                   int64_t      val )
+{
+    tr_benc * child = dictFindOrAdd( dict, key, TR_TYPE_INT );
+    tr_bencInitInt( child, val );
+    return child;
+}
+
+tr_benc*
+tr_bencDictAddBool( tr_benc * dict, const char * key, tr_bool val )
+{
+    tr_benc * child = dictFindOrAdd( dict, key, TR_TYPE_BOOL );
+    tr_bencInitBool( child, val );
+    return child;
+}
+
+tr_benc*
+tr_bencDictAddReal( tr_benc * dict, const char * key, double val )
+{
+    tr_benc * child = dictFindOrAdd( dict, key, TR_TYPE_REAL );
+    tr_bencInitReal( child, val );
     return child;
 }
 
@@ -706,9 +785,10 @@ tr_bencDictAddStr( tr_benc * dict, const char * key, const char * val )
 
     /* see if it already exists, and if so, try to reuse it */
     if(( child = tr_bencDictFind( dict, key ))) {
-        if( tr_bencIsString( child ) )
-            tr_free( child->val.s.s );
-        else {
+        if( tr_bencIsString( child ) ) {
+            if( stringIsAlloced( child ) )
+                tr_free( child->val.s.str.ptr );
+        } else {
             tr_bencDictRemove( dict, key );
             child = NULL;
         }
@@ -724,24 +804,6 @@ tr_bencDictAddStr( tr_benc * dict, const char * key, const char * val )
     return child;
 }
 
-tr_benc*
-tr_bencDictAddDouble( tr_benc *    dict,
-                      const char * key,
-                      double       d )
-{
-    char buf[128];
-    char * locale;
-
-    /* the json spec requires a '.' decimal point regardless of locale */
-    locale = tr_strdup( setlocale ( LC_NUMERIC, NULL ) );
-    setlocale( LC_NUMERIC, "POSIX" );
-    tr_snprintf( buf, sizeof( buf ), "%f", d );
-    setlocale( LC_NUMERIC, locale );
-    tr_free( locale );
-
-    return tr_bencDictAddStr( dict, key, buf );
-}
-
 tr_benc*
 tr_bencDictAddList( tr_benc *    dict,
                     const char * key,
@@ -845,7 +907,7 @@ nodeNewDict( const tr_benc * val )
     indices = tr_new( struct KeyIndex, nKeys );
     for( i = j = 0; i < ( nKeys * 2 ); i += 2, ++j )
     {
-        indices[j].key = val->val.l.vals[i].val.s.s;
+        indices[j].key = getStr(&val->val.l.vals[i]);
         indices[j].index = i;
     }
     qsort( indices, j, sizeof( struct KeyIndex ), compareKeyIndex );
@@ -912,6 +974,8 @@ typedef void ( *BencWalkFunc )( const tr_benc * val, void * user_data );
 struct WalkFuncs
 {
     BencWalkFunc    intFunc;
+    BencWalkFunc    boolFunc;
+    BencWalkFunc    realFunc;
     BencWalkFunc    stringFunc;
     BencWalkFunc    dictBeginFunc;
     BencWalkFunc    listBeginFunc;
@@ -924,9 +988,9 @@ struct WalkFuncs
  * attack via maliciously-crafted bencoded data. (#667)
  */
 static void
-bencWalk( const tr_benc *    top,
-          struct WalkFuncs * walkFuncs,
-          void *             user_data )
+bencWalk( const tr_benc          * top,
+          const struct WalkFuncs * walkFuncs,
+          void                   * user_data )
 {
     tr_ptrArray stack = TR_PTR_ARRAY_INIT;
 
@@ -959,22 +1023,30 @@ bencWalk( const tr_benc *    top,
 
         if( val ) switch( val->type )
             {
-                case TYPE_INT:
+                case TR_TYPE_INT:
                     walkFuncs->intFunc( val, user_data );
                     break;
 
-                case TYPE_STR:
+                case TR_TYPE_BOOL:
+                    walkFuncs->boolFunc( val, user_data );
+                    break;
+
+                case TR_TYPE_REAL:
+                    walkFuncs->realFunc( val, user_data );
+                    break;
+
+                case TR_TYPE_STR:
                     walkFuncs->stringFunc( val, user_data );
                     break;
 
-                case TYPE_LIST:
+                case TR_TYPE_LIST:
                     if( val != node->val )
                         tr_ptrArrayAppend( &stack, nodeNew( val ) );
                     else
                         walkFuncs->listBeginFunc( val, user_data );
                     break;
 
-                case TYPE_DICT:
+                case TR_TYPE_DICT:
                     if( val != node->val )
                         tr_ptrArrayAppend( &stack, nodeNew( val ) );
                     else
@@ -983,7 +1055,7 @@ bencWalk( const tr_benc *    top,
 
                 default:
                     /* did caller give us an uninitialized val? */
-                    tr_err( _( "Invalid metadata" ) );
+                    tr_err( "%s", _( "Invalid metadata" ) );
                     break;
             }
     }
@@ -996,66 +1068,71 @@ bencWalk( const tr_benc *    top,
 ****/
 
 static void
-saveIntFunc( const tr_benc * val,
-             void *          evbuf )
+saveIntFunc( const tr_benc * val, void * evbuf )
 {
     evbuffer_add_printf( evbuf, "i%" PRId64 "e", val->val.i );
 }
 
 static void
-saveStringFunc( const tr_benc * val,
-                void *          vevbuf )
+saveBoolFunc( const tr_benc * val, void * evbuf )
 {
-    struct evbuffer * evbuf = vevbuf;
-
-    evbuffer_add_printf( evbuf, "%lu:", (unsigned long)val->val.s.i );
-    evbuffer_add( evbuf, val->val.s.s, val->val.s.i );
+    if( val->val.b )
+        evbuffer_add( evbuf, "i1e", 3 );
+    else
+        evbuffer_add( evbuf, "i0e", 3 );
 }
 
 static void
-saveDictBeginFunc( const tr_benc * val UNUSED,
-                   void *              evbuf )
+saveRealFunc( const tr_benc * val, void * evbuf )
 {
-    evbuffer_add_printf( evbuf, "d" );
+    char buf[128];
+    char locale[128];
+    size_t len;
+
+    /* always use a '.' decimal point s.t. locale-hopping doesn't bite us */
+    tr_strlcpy( locale, setlocale( LC_NUMERIC, NULL ), sizeof( locale ) );
+    setlocale( LC_NUMERIC, "POSIX" );
+    tr_snprintf( buf, sizeof( buf ), "%f", val->val.d );
+    setlocale( LC_NUMERIC, locale );
+
+    len = strlen( buf );
+    evbuffer_add_printf( evbuf, "%lu:", (unsigned long)len );
+    evbuffer_add( evbuf, buf, len );
 }
 
 static void
-saveListBeginFunc( const tr_benc * val UNUSED,
-                   void *              evbuf )
+saveStringFunc( const tr_benc * val, void * evbuf )
 {
-    evbuffer_add_printf( evbuf, "l" );
+    evbuffer_add_printf( evbuf, "%lu:", (unsigned long)val->val.s.len );
+    evbuffer_add( evbuf, getStr(val), val->val.s.len );
 }
 
 static void
-saveContainerEndFunc( const tr_benc * val UNUSED,
-                      void *              evbuf )
+saveDictBeginFunc( const tr_benc * val UNUSED, void * evbuf )
 {
-    evbuffer_add_printf( evbuf, "e" );
+    evbuffer_add( evbuf, "d", 1 );
 }
 
-char*
-tr_bencSave( const tr_benc * top,
-             int *           len )
+static void
+saveListBeginFunc( const tr_benc * val UNUSED, void * evbuf )
 {
-    char *            ret;
-    struct WalkFuncs  walkFuncs;
-    struct evbuffer * out = tr_getBuffer( );
-
-    walkFuncs.intFunc = saveIntFunc;
-    walkFuncs.stringFunc = saveStringFunc;
-    walkFuncs.dictBeginFunc = saveDictBeginFunc;
-    walkFuncs.listBeginFunc = saveListBeginFunc;
-    walkFuncs.containerEndFunc = saveContainerEndFunc;
-    bencWalk( top, &walkFuncs, out );
-
-    if( len )
-        *len = EVBUFFER_LENGTH( out );
-    ret = tr_strndup( EVBUFFER_DATA( out ), EVBUFFER_LENGTH( out ) );
-
-    tr_releaseBuffer( out );
-    return ret;
+    evbuffer_add( evbuf, "l", 1 );
 }
 
+static void
+saveContainerEndFunc( const tr_benc * val UNUSED, void * evbuf )
+{
+    evbuffer_add( evbuf, "e", 1 );
+}
+
+static const struct WalkFuncs saveFuncs = { saveIntFunc,
+                                            saveBoolFunc,
+                                            saveRealFunc,
+                                            saveStringFunc,
+                                            saveDictBeginFunc,
+                                            saveListBeginFunc,
+                                            saveContainerEndFunc };
+
 /***
 ****
 ***/
@@ -1069,7 +1146,8 @@ static void
 freeStringFunc( const tr_benc * val,
                 void *          freeme )
 {
-    tr_ptrArrayAppend( freeme, val->val.s.s );
+    if( stringIsAlloced( val ) )
+        tr_ptrArrayAppend( freeme, val->val.s.str.ptr );
 }
 
 static void
@@ -1079,21 +1157,21 @@ freeContainerBeginFunc( const tr_benc * val,
     tr_ptrArrayAppend( freeme, val->val.l.vals );
 }
 
+static const struct WalkFuncs freeWalkFuncs = { freeDummyFunc,
+                                                freeDummyFunc,
+                                                freeDummyFunc,
+                                                freeStringFunc,
+                                                freeContainerBeginFunc,
+                                                freeContainerBeginFunc,
+                                                freeDummyFunc };
+
 void
 tr_bencFree( tr_benc * val )
 {
-    if( val && val->type )
+    if( isSomething( val ) )
     {
         tr_ptrArray a = TR_PTR_ARRAY_INIT;
-        struct WalkFuncs walkFuncs;
-
-        walkFuncs.intFunc = freeDummyFunc;
-        walkFuncs.stringFunc = freeStringFunc;
-        walkFuncs.dictBeginFunc = freeContainerBeginFunc;
-        walkFuncs.listBeginFunc = freeContainerBeginFunc;
-        walkFuncs.containerEndFunc = freeDummyFunc;
-        bencWalk( val, &walkFuncs, &a );
-
+        bencWalk( val, &freeWalkFuncs, &a );
         tr_ptrArrayDestruct( &a, tr_free );
     }
 }
@@ -1111,16 +1189,23 @@ struct ParentState
 
 struct jsonWalk
 {
-    tr_list *          parents;
+    tr_bool doIndent;
+    tr_list * parents;
     struct evbuffer *  out;
 };
 
 static void
 jsonIndent( struct jsonWalk * data )
 {
-    const int width = tr_list_size( data->parents ) * 4;
+    if( data->doIndent )
+    {
+        char buf[1024];
+        const int width = tr_list_size( data->parents ) * 4;
 
-    evbuffer_add_printf( data->out, "\n%*.*s", width, width, " " );
+        buf[0] = '\n';
+        memset( buf+1, ' ', width );
+        evbuffer_add( data->out, buf, 1+width );
+    }
 }
 
 static void
@@ -1132,24 +1217,28 @@ jsonChildFunc( struct jsonWalk * data )
 
         switch( parentState->bencType )
         {
-            case TYPE_DICT:
+            case TR_TYPE_DICT:
             {
                 const int i = parentState->childIndex++;
                 if( !( i % 2 ) )
-                    evbuffer_add_printf( data->out, ": " );
-                else
-                {
-                    evbuffer_add_printf( data->out, ", " );
-                    jsonIndent( data );
+                    evbuffer_add( data->out, ": ", data->doIndent ? 2 : 1 );
+                else {
+                    const tr_bool isLast = parentState->childIndex == parentState->childCount;
+                    if( !isLast ) {
+                        evbuffer_add( data->out, ", ", data->doIndent ? 2 : 1 );
+                        jsonIndent( data );
+                    }
                 }
                 break;
             }
 
-            case TYPE_LIST:
+            case TR_TYPE_LIST:
             {
-                ++parentState->childIndex;
-                evbuffer_add_printf( data->out, ", " );
-                jsonIndent( data );
+                const tr_bool isLast = ++parentState->childIndex == parentState->childCount;
+                if( !isLast ) {
+                    evbuffer_add( data->out, ", ", data->doIndent ? 2 : 1 );
+                    jsonIndent( data );
+                }
                 break;
             }
 
@@ -1188,50 +1277,64 @@ jsonIntFunc( const tr_benc * val,
 }
 
 static void
-jsonStringFunc( const tr_benc * val,
-                void *          vdata )
+jsonBoolFunc( const tr_benc * val, void * vdata )
 {
-    struct jsonWalk *    data = vdata;
-    const unsigned char *it, *end;
+    struct jsonWalk * data = vdata;
 
-    evbuffer_add_printf( data->out, "\"" );
-    for( it = (const unsigned char*)val->val.s.s, end = it + val->val.s.i;
-         it != end; ++it )
+    if( val->val.b )
+        evbuffer_add( data->out, "true", 4 );
+    else
+        evbuffer_add( data->out, "false", 5 );
+
+    jsonChildFunc( data );
+}
+
+static void
+jsonRealFunc( const tr_benc * val, void * vdata )
+{
+    struct jsonWalk * data = vdata;
+    char locale[128];
+
+    if( fabs( val->val.d ) < 0.00001 )
+        evbuffer_add( data->out, "0", 1 );
+    else {
+        /* json requires a '.' decimal point regardless of locale */
+        tr_strlcpy( locale, setlocale( LC_NUMERIC, NULL ), sizeof( locale ) );
+        setlocale( LC_NUMERIC, "POSIX" );
+        evbuffer_add_printf( data->out, "%.4f", val->val.d );
+        setlocale( LC_NUMERIC, locale );
+    }
+
+    jsonChildFunc( data );
+}
+
+static void
+jsonStringFunc( const tr_benc * val, void * vdata )
+{
+    struct jsonWalk * data = vdata;
+    const unsigned char * it = (const unsigned char *) getStr(val);
+    const unsigned char * end = it + val->val.s.len;
+
+    evbuffer_expand( data->out, val->val.s.len + 2 );
+    evbuffer_add( data->out, "\"", 1 );
+
+    for( ; it!=end; ++it )
     {
         switch( *it )
         {
-            case '/':
-                evbuffer_add_printf( data->out, "\\/" ); break;
-
-            case '\b':
-                evbuffer_add_printf( data->out, "\\b" ); break;
-
-            case '\f':
-                evbuffer_add_printf( data->out, "\\f" ); break;
-
-            case '\n':
-                evbuffer_add_printf( data->out, "\\n" ); break;
-
-            case '\r':
-                evbuffer_add_printf( data->out, "\\r" ); break;
-
-            case '\t':
-                evbuffer_add_printf( data->out, "\\t" ); break;
-
-            case '"':
-                evbuffer_add_printf( data->out, "\\\"" ); break;
-
-            case '\\':
-                evbuffer_add_printf( data->out, "\\\\" ); break;
+            case '/': evbuffer_add( data->out, "\\/", 2 ); break;
+            case '\b': evbuffer_add( data->out, "\\b", 2 ); break;
+            case '\f': evbuffer_add( data->out, "\\f", 2 ); break;
+            case '\n': evbuffer_add( data->out, "\\n", 2 ); break;
+            case '\r': evbuffer_add( data->out, "\\r", 2 ); break;
+            case '\t': evbuffer_add( data->out, "\\t", 2 ); break;
+            case '"': evbuffer_add( data->out, "\\\"", 2 ); break;
+            case '\\': evbuffer_add( data->out, "\\\\", 2 ); break;
 
             default:
                 if( isascii( *it ) )
-                {
-                    /*fprintf( stderr, "[%c]\n", *it );*/
-                    evbuffer_add_printf( data->out, "%c", *it );
-                }
-                else
-                {
+                    evbuffer_add( data->out, it, 1 );
+                else {
                     const UTF8 * tmp = it;
                     UTF32        buf = 0;
                     UTF32 *      u32 = &buf;
@@ -1242,11 +1345,10 @@ jsonStringFunc( const tr_benc * val,
                         evbuffer_add_printf( data->out, "\\u%04x", (unsigned int)buf );
                         it = tmp - 1;
                     }
-                    /*fprintf( stderr, "[\\u%04x]\n", buf );*/
                 }
         }
     }
-    evbuffer_add_printf( data->out, "\"" );
+    evbuffer_add( data->out, "\"", 1 );
     jsonChildFunc( data );
 }
 
@@ -1257,7 +1359,7 @@ jsonDictBeginFunc( const tr_benc * val,
     struct jsonWalk * data = vdata;
 
     jsonPushParent( data, val );
-    evbuffer_add_printf( data->out, "{" );
+    evbuffer_add( data->out, "{", 1 );
     if( val->val.l.count )
         jsonIndent( data );
 }
@@ -1270,7 +1372,7 @@ jsonListBeginFunc( const tr_benc * val,
     struct jsonWalk * data = vdata;
 
     jsonPushParent( data, val );
-    evbuffer_add_printf( data->out, "[" );
+    evbuffer_add( data->out, "[", 1 );
     if( nChildren )
         jsonIndent( data );
 }
@@ -1279,57 +1381,26 @@ static void
 jsonContainerEndFunc( const tr_benc * val,
                       void *          vdata )
 {
-    size_t            i;
     struct jsonWalk * data = vdata;
-    char *            str;
     int               emptyContainer = FALSE;
 
-    /* trim out the trailing comma, if any */
-    str = (char*) EVBUFFER_DATA( data->out );
-    for( i = EVBUFFER_LENGTH( data->out ) - 1; i > 0; --i )
-    {
-        if( isspace( str[i] ) ) continue;
-        if( str[i] == ',' )
-            EVBUFFER_LENGTH( data->out ) = i;
-        if( str[i] == '{' || str[i] == '[' )
-            emptyContainer = TRUE;
-        break;
-    }
-
     jsonPopParent( data );
     if( !emptyContainer )
         jsonIndent( data );
     if( tr_bencIsDict( val ) )
-        evbuffer_add_printf( data->out, "}" );
+        evbuffer_add( data->out, "}", 1 );
     else /* list */
-        evbuffer_add_printf( data->out, "]" );
+        evbuffer_add( data->out, "]", 1 );
     jsonChildFunc( data );
 }
 
-char*
-tr_bencSaveAsJSON( const tr_benc * top, struct evbuffer * out )
-{
-    struct WalkFuncs walkFuncs;
-    struct jsonWalk  data;
-
-    evbuffer_drain( out, EVBUFFER_LENGTH( out ) );
-
-    data.out = out;
-    data.parents = NULL;
-
-    walkFuncs.intFunc = jsonIntFunc;
-    walkFuncs.stringFunc = jsonStringFunc;
-    walkFuncs.dictBeginFunc = jsonDictBeginFunc;
-    walkFuncs.listBeginFunc = jsonListBeginFunc;
-    walkFuncs.containerEndFunc = jsonContainerEndFunc;
-
-    bencWalk( top, &walkFuncs, &data );
-
-    if( EVBUFFER_LENGTH( out ) )
-        evbuffer_add_printf( out, "\n" );
-
-    return (char*) EVBUFFER_DATA( out );
-}
+static const struct WalkFuncs jsonWalkFuncs = { jsonIntFunc,
+                                                jsonBoolFunc,
+                                                jsonRealFunc,
+                                                jsonStringFunc,
+                                                jsonDictBeginFunc,
+                                                jsonListBeginFunc,
+                                                jsonContainerEndFunc };
 
 /***
 ****
@@ -1346,8 +1417,8 @@ tr_bencDictSize( const tr_benc * dict )
     return count;
 }
 
-static tr_bool
-tr_bencDictChild( const tr_benc * dict, size_t n, const char ** key, const tr_benc ** val )
+tr_bool
+tr_bencDictChild( tr_benc * dict, size_t n, const char ** key, tr_benc ** val )
 {
     tr_bool success = 0;
 
@@ -1376,23 +1447,34 @@ tr_bencMergeDicts( tr_benc * target, const tr_benc * source )
     for( i=0; i<sourceCount; ++i )
     {
         const char * key;
-        const tr_benc * val;
+        tr_benc * val;
+        tr_benc * t;
 
-        if( tr_bencDictChild( source, i, &key, &val ) )
+        if( tr_bencDictChild( (tr_benc*)source, i, &key, &val ) )
         {
-            int64_t i64;
-            const char * str;
-            tr_benc * t;
-
-            if( tr_bencGetInt( val, &i64 ) )
+            if( tr_bencIsBool( val ) )
             {
-                tr_bencDictRemove( target, key );
-                tr_bencDictAddInt( target, key, i64 );
+                tr_bool boolVal;
+                tr_bencGetBool( val, &boolVal );
+                tr_bencDictAddBool( target, key, boolVal );
             }
-            else if( tr_bencGetStr( val, &str ) )
+            else if( tr_bencIsReal( val ) )
             {
-                tr_bencDictRemove( target, key );
-                tr_bencDictAddStr( target, key, str );
+                double realVal = 0;
+                tr_bencGetReal( val, &realVal );
+                tr_bencDictAddReal( target, key, realVal );
+            }
+            else if( tr_bencIsInt( val ) )
+            {
+                int64_t intVal = 0;
+                tr_bencGetInt( val, &intVal );
+                tr_bencDictAddInt( target, key, intVal );
+            }
+            else if( tr_bencIsString( val ) )
+            {
+                const char * strVal = NULL;
+                tr_bencGetStr( val, &strVal );
+                tr_bencDictAddStr( target, key, strVal );
             }
             else if( tr_bencIsDict( val ) && tr_bencDictFindDict( target, key, &t ) )
             {
@@ -1400,67 +1482,82 @@ tr_bencMergeDicts( tr_benc * target, const tr_benc * source )
             }
             else
             {
-            
                 tr_dbg( "tr_bencMergeDicts skipping \"%s\"", key );
             }
         }
     }
 }
 
-/*** 
-**** 
-***/ 
+/***
+****
+***/
 
-static int
-saveFile( const char * filename,
-          const char * content,
-          size_t       len )
+void
+tr_bencToBuf( const tr_benc * top, tr_fmt_mode mode, struct evbuffer * buf )
 {
-    int    err = 0;
-    FILE * out = NULL;
+    evbuffer_drain( buf, EVBUFFER_LENGTH( buf ) );
 
-    out = fopen( filename, "wb+" );
+    switch( mode )
+    {
+        case TR_FMT_BENC:
+            bencWalk( top, &saveFuncs, buf );
+            break;
 
-    if( !out )
+        case TR_FMT_JSON:
+        case TR_FMT_JSON_LEAN: {
+            struct jsonWalk data;
+            data.doIndent = mode==TR_FMT_JSON;
+            data.out = buf;
+            data.parents = NULL;
+            bencWalk( top, &jsonWalkFuncs, &data );
+            if( EVBUFFER_LENGTH( buf ) )
+                evbuffer_add_printf( buf, "\n" );
+            break;
+        }
+    }
+}
+
+char*
+tr_bencToStr( const tr_benc * top, tr_fmt_mode mode, int * len )
+{
+    char * ret;
+    struct evbuffer * buf = evbuffer_new( );
+    tr_bencToBuf( top, mode, buf );
+    ret = tr_strndup( EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ) );
+    if( len != NULL )
+        *len = (int) EVBUFFER_LENGTH( buf );
+    evbuffer_free( buf );
+    return ret;
+}
+
+int
+tr_bencToFile( const tr_benc * top, tr_fmt_mode mode, const char * filename )
+{
+    int err = 0;
+    FILE * fp = fopen( filename, "wb+" );
+
+    if( fp == NULL )
     {
         err = errno;
         tr_err( _( "Couldn't open \"%1$s\": %2$s" ),
                 filename, tr_strerror( errno ) );
     }
-    else if( fwrite( content, sizeof( char ), len, out ) != (size_t)len )
+    else
     {
-        err = errno;
-        tr_err( _( "Couldn't save file \"%1$s\": %2$s" ),
-               filename, tr_strerror( errno ) );
+        int len;
+        char * str = tr_bencToStr( top, mode, &len );
+
+        if( fwrite( str, 1, len, fp ) == (size_t)len )
+            tr_dbg( "tr_bencToFile saved \"%s\"", filename );
+        else {
+            err = errno;
+            tr_err( _( "Couldn't save file \"%1$s\": %2$s" ), filename, tr_strerror( errno ) );
+        }
+
+        tr_free( str );
+        fclose( fp );
     }
 
-    if( !err )
-        tr_dbg( "tr_bencSaveFile saved \"%s\"", filename );
-    if( out )
-        fclose( out );
-    return err;
-}
-
-int
-tr_bencSaveFile( const char *    filename,
-                 const tr_benc * b )
-{
-    int       len;
-    char *    content = tr_bencSave( b, &len );
-    const int err = saveFile( filename, content, len );
-
-    tr_free( content );
-    return err;
-}
-
-int
-tr_bencSaveJSONFile( const char *    filename,
-                     const tr_benc * b )
-{
-    struct evbuffer * buf = tr_getBuffer( );
-    const char * json = tr_bencSaveAsJSON( b, buf );
-    const int err = saveFile( filename, json, EVBUFFER_LENGTH( buf ) );
-    tr_releaseBuffer( buf );
     return err;
 }
 
@@ -1469,10 +1566,10 @@ tr_bencSaveJSONFile( const char *    filename,
 ***/
 
 int
-tr_bencLoadFile( const char * filename, tr_benc * b )
+tr_bencLoadFile( tr_benc * setme, tr_fmt_mode mode, const char * filename )
 {
-    int       err;
-    size_t    contentLen;
+    int err;
+    size_t contentLen;
     uint8_t * content;
 
     content = tr_loadFile( filename, &contentLen );
@@ -1480,27 +1577,12 @@ tr_bencLoadFile( const char * filename, tr_benc * b )
         err = errno;
     else if( !content )
         err = ENODATA;
-    else
-        err = tr_bencLoad( content, contentLen, b, NULL );
-
-    tr_free( content );
-    return err;
-}
-
-int
-tr_bencLoadJSONFile( const char * filename, tr_benc * b )
-{
-    int        err;
-    size_t     contentLen;
-    uint8_t  * content;
-
-    content = tr_loadFile( filename, &contentLen );
-    if( !content && errno )
-        err = errno;
-    else if( !content )
-        err = ENODATA;
-    else
-        err = tr_jsonParse( content, contentLen, b, NULL );
+    else {
+        if( mode == TR_FMT_BENC )
+            err = tr_bencLoad( content, contentLen, setme, NULL );
+        else
+            err = tr_jsonParse( filename, content, contentLen, setme, NULL );
+    }
 
     tr_free( content );
     return err;
