@@ -6,7 +6,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: crypto.c 7873 2009-02-11 16:07:45Z charles $
+ * $Id: crypto.c 8689 2009-06-14 14:39:51Z charles $
  */
 
 #include <stdlib.h> /* for abs() */
@@ -311,6 +311,8 @@ tr_cryptoRandInt( int upperBound )
     int noise;
     int val;
 
+    assert( upperBound > 0 );
+
     if( RAND_pseudo_bytes ( (unsigned char *) &noise, sizeof noise ) >= 0 )
     {
         val = abs( noise ) % upperBound;
@@ -328,24 +330,89 @@ tr_cryptoRandInt( int upperBound )
 int
 tr_cryptoWeakRandInt( int upperBound )
 {
-    static int init = 0;
+    int val;
+    static tr_bool init = FALSE;
 
     assert( upperBound > 0 );
 
     if( !init )
     {
         srand( tr_date( ) );
-        init = 1;
+        init = TRUE;
     }
 
-    return rand( ) % upperBound;
+    
+    val = rand( ) % upperBound;
+    assert( val >= 0 );
+    assert( val < upperBound );
+    return val;
 }
 
 void
-tr_cryptoRandBuf( unsigned char *buf,
-                  size_t         len )
+tr_cryptoRandBuf( void * buf, size_t len )
 {
-    if( RAND_pseudo_bytes ( buf, len ) != 1 )
+    if( RAND_pseudo_bytes ( (unsigned char*)buf, len ) != 1 )
         logErrorFromSSL( );
 }
 
+/***
+****
+***/
+
+char*
+tr_ssha1( const void * plaintext )
+{
+    static const char * salter = "0123456789"
+                                 "abcdefghijklmnopqrstuvwxyz"
+                                 "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+                                 "./";
+    static const size_t salter_len = 64;
+    static const size_t saltval_len = 8;
+
+    size_t i;
+    char salt[saltval_len];
+    uint8_t sha[SHA_DIGEST_LENGTH];
+    char buf[2*SHA_DIGEST_LENGTH + saltval_len + 2];
+
+    for( i=0; i<saltval_len; ++i )
+        salt[i] = salter[ tr_cryptoRandInt( salter_len ) ];
+
+    tr_sha1( sha, plaintext, strlen( plaintext ), salt, saltval_len, NULL );
+    tr_sha1_to_hex( &buf[1], sha );
+    memcpy( &buf[1+2*SHA_DIGEST_LENGTH], &salt, saltval_len );
+    buf[1+2*SHA_DIGEST_LENGTH + saltval_len] = '\0';
+    buf[0] = '{'; /* signal that this is a hash. this makes saving/restoring
+                     easier */
+
+    return tr_strdup( &buf );
+}
+
+tr_bool
+tr_ssha1_matches( const char * source, const char * pass )
+{
+    char * salt;
+    size_t saltlen;
+    char * hashed;
+    uint8_t buf[SHA_DIGEST_LENGTH];
+    tr_bool result;
+
+    /* extract the salt */
+    saltlen = strlen( source ) - 2*SHA_DIGEST_LENGTH-1;
+    salt = tr_malloc( saltlen );
+    memcpy( salt, source + 2*SHA_DIGEST_LENGTH+1, saltlen );
+
+    /* hash pass + salt */
+    hashed = tr_malloc( 2*SHA_DIGEST_LENGTH + saltlen + 2 );
+    tr_sha1( buf, pass, strlen( pass ), salt, saltlen, NULL );
+    tr_sha1_to_hex( &hashed[1], buf );
+    memcpy( hashed + 1+2*SHA_DIGEST_LENGTH, salt, saltlen );
+    hashed[1+2*SHA_DIGEST_LENGTH + saltlen] = '\0';
+    hashed[0] = '{';
+
+    result = strcmp( source, hashed ) == 0 ? TRUE : FALSE;
+
+    tr_free( hashed );
+    tr_free( salt );
+    
+    return result;
+}
