@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fastresume.c 7855 2009-02-09 17:34:43Z charles $
+ * $Id: fastresume.c 8736 2009-06-23 14:15:23Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -181,7 +181,7 @@ internalIdToPublicBitfield( uint8_t id )
             ret = TR_FR_PROGRESS;      break;
 
         case FR_ID_PRIORITY:
-            ret = TR_FR_PRIORITY;      break;
+            ret = TR_FR_FILE_PRIORITIES; break;
 
         case FR_ID_SPEED:
             ret = TR_FR_SPEEDLIMIT;    break;
@@ -366,7 +366,7 @@ parsePriorities( tr_torrent *    tor,
         tr_free( dnd );
         tr_free( dl );
 
-        ret = TR_FR_PRIORITY;
+        ret = TR_FR_FILE_PRIORITIES;
     }
 
     return ret;
@@ -387,11 +387,11 @@ parseSpeedLimit( tr_torrent *    tor,
         readBytes( &i16, &buf, sizeof( i16 ) );
         tr_torrentSetSpeedLimit( tor, TR_DOWN, i16 );
         readBytes( &i8, &buf, sizeof( i8 ) );
-        tr_torrentSetSpeedMode( tor, TR_DOWN, (tr_speedlimit)i8 );
+        /*tr_torrentSetSpeedMode( tor, TR_DOWN, (tr_speedlimit)i8 );*/
         readBytes( &i16, &buf, sizeof( i16 ) );
         tr_torrentSetSpeedLimit( tor, TR_UP, i16 );
         readBytes( &i8, &buf, sizeof( i8 ) );
-        tr_torrentSetSpeedMode( tor, TR_UP, (tr_speedlimit)i8 );
+        /*tr_torrentSetSpeedMode( tor, TR_UP, (tr_speedlimit)i8 );*/
 
         ret = TR_FR_SPEEDLIMIT;
     }
@@ -550,7 +550,7 @@ fastResumeLoadImpl( tr_torrent * tor,
 
     if( !buf )
         /* %s is the torrent name */
-        tr_torinf( tor, _( "Couldn't read resume file" ) );
+        tr_torinf( tor, "%s", _( "Couldn't read resume file" ) );
     else
     {
         const uint8_t * walk = buf;
@@ -563,7 +563,7 @@ fastResumeLoadImpl( tr_torrent * tor,
                 ret |= parseVersion1 ( tor, walk, end, fieldsToLoad );
             else
                 /* %s is the torrent name */
-                tr_torinf( tor, _( "Couldn't read resume file" ) );
+                tr_torinf( tor, "%s", _( "Couldn't read resume file" ) );
         }
 
         tr_free( buf );
