@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fdlimit.c 7813 2009-01-29 18:18:24Z charles $
+ * $Id: fdlimit.c 8823 2009-07-17 17:23:03Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -43,8 +43,8 @@
  #include <fcntl.h>
 #endif
 
-#ifdef HAVE_FALLOCATE
- #include <linux/falloc.h>
+#ifdef HAVE_XFS_XFS_H
+ #include <xfs/xfs.h>
 #endif
 
 #include <sys/types.h>
@@ -62,7 +62,7 @@
 #include "fdlimit.h"
 #include "list.h"
 #include "net.h"
-#include "platform.h" /* tr_lock */
+#include "platform.h" /* MAX_PATH_LENGTH, TR_PATH_DELIMITER */
 #include "utils.h"
 
 #define dbgmsg( ... ) \
@@ -79,31 +79,23 @@ enum
 {
     NOFILE_BUFFER = 512, /* the process' number of open files is
                             globalMaxPeers + NOFILE_BUFFER */
-
-    SYNC_INTERVAL = 15   /* (arbitrary number) how many seconds to go
-                            between fsync calls for files in heavy use */
 };
 
 struct tr_openfile
 {
-    tr_bool    isCheckedOut;
     tr_bool    isWritable;
-    tr_bool    closeWhenDone;
+    int        torrentId;
     char       filename[MAX_PATH_LENGTH];
     int        fd;
     uint64_t   date;
-    time_t     syncAt;
 };
 
 struct tr_fd_s
 {
     int                   socketCount;
     int                   socketLimit;
-
-    struct tr_openfile  * openFiles;
     int                   openFileLimit;
-
-    tr_lock             * lock;
+    struct tr_openfile  * openFiles;
 };
 
 static struct tr_fd_s * gFd = NULL;
@@ -158,31 +150,47 @@ preallocateFileFull( const char * filename, uint64_t length )
     int fd = open( filename, flags, 0666 );
     if( fd >= 0 )
     {
-        
-# ifdef HAVE_FALLOCATE
-
-        success = !fallocate( fd, FALLOC_FL_KEEP_SIZE, 0, length );
-
-# elif defined(HAVE_POSIX_FALLOCATE)
-
-        success = !posix_fallocate( fd, 0, length );
-
-# elif defined(SYS_DARWIN) 
-
-        fstore_t fst;
-        fst.fst_flags = F_ALLOCATECONTIG;
-        fst.fst_posmode = F_PEOFPOSMODE;
-        fst.fst_offset = 0;
-        fst.fst_length = length;
-        fst.fst_bytesalloc = 0;
-        success = !fcntl( fd, F_PREALLOCATE, &fst );
-
-# else
-
-        #warning no known method to preallocate files on this platform
-        success = 0;
-
+# ifdef HAVE_XFS_XFS_H
+        if( !success && platform_test_xfs_fd( fd ) )
+        {
+            xfs_flock64_t fl;
+            fl.l_whence = 0;
+            fl.l_start = 0;
+            fl.l_len = length;
+            success = !xfsctl( NULL, fd, XFS_IOC_RESVSP64, &fl );
+        }
 # endif
+# ifdef SYS_DARWIN
+        if( !success )
+        {
+            fstore_t fst;
+            fst.fst_flags = F_ALLOCATECONTIG;
+            fst.fst_posmode = F_PEOFPOSMODE;
+            fst.fst_offset = 0;
+            fst.fst_length = length;
+            fst.fst_bytesalloc = 0;
+            success = !fcntl( fd, F_PREALLOCATE, &fst );
+        }
+# endif
+# ifdef HAVE_POSIX_FALLOCATE
+        if( !success )
+        {
+            success = !posix_fallocate( fd, 0, length );
+        }
+# endif
+
+        if( !success ) /* if nothing else works, do it the old-fashioned way */
+        {
+            uint8_t buf[ 4096 ]; 
+            memset( buf, 0, sizeof( buf ) ); 
+            success = TRUE; 
+            while ( success && ( length > 0 ) ) 
+            { 
+                const int thisPass = MIN( length, sizeof( buf ) ); 
+                success = write( fd, buf, thisPass ) == thisPass; 
+                length -= thisPass; 
+            } 
+        }
 
         close( fd );
     }
@@ -192,6 +200,76 @@ preallocateFileFull( const char * filename, uint64_t length )
     return success;
 }
 
+tr_bool
+tr_preallocate_file( const char * filename, uint64_t length )
+{
+    return preallocateFileFull( filename, length );
+}
+
+int
+tr_open_file_for_writing( const char * filename )
+{
+    int flags = O_WRONLY | O_CREAT;
+#ifdef O_BINARY
+    flags |= O_BINARY;
+#endif
+#ifdef O_LARGEFILE
+    flags |= O_LARGEFILE;
+#endif
+    return open( filename, flags, 0666 );
+}
+
+int
+tr_open_file_for_scanning( const char * filename )
+{
+    int fd;
+    int flags;
+
+    /* build the flags */
+    flags = O_RDONLY;
+#ifdef O_SEQUENTIAL
+    flags |= O_SEQUENTIAL;
+#endif
+#ifdef O_BINARY
+    flags |= O_BINARY;
+#endif
+#ifdef O_LARGEFILE
+    flags |= O_LARGEFILE;
+#endif
+
+    /* open the file */
+    fd = open( filename, flags, 0666 );
+    if( fd >= 0 )
+    {
+        /* Set hints about the lookahead buffer and caching. It's okay
+           for these to fail silently, so don't let them affect errno */
+        const int err = errno;
+#ifdef HAVE_POSIX_FADVISE
+        posix_fadvise( fd, 0, 0, POSIX_FADV_SEQUENTIAL );
+#endif
+#ifdef SYS_DARWIN
+        fcntl( fd, F_NOCACHE, 1 );
+        fcntl( fd, F_RDAHEAD, 1 );
+#endif
+        errno = err;
+    }
+
+    return fd;
+}
+
+void
+tr_close_file( int fd )
+{
+#if defined(HAVE_POSIX_FADVISE)
+    /* Set hint about not caching this file.
+       It's okay for this to fail silently, so don't let it affect errno */
+    const int err = errno;
+    posix_fadvise( fd, 0, 0, POSIX_FADV_DONTNEED );
+    errno = err;
+#endif
+    close( fd );
+}
+
 /**
  * returns 0 on success, or an errno value on failure.
  * errno values include ENOENT if the parent folder doesn't exist,
@@ -209,12 +287,12 @@ TrOpenFile( int                      i,
     int                  flags;
     char               * filename;
     struct stat          sb;
-    int                  alreadyExisted;
+    tr_bool              alreadyExisted;
 
     /* confirm the parent folder exists */
     if( stat( folder, &sb ) || !S_ISDIR( sb.st_mode ) )
     {
-        tr_err( _( "Couldn't create \"%1$s\": parent folder \"%2$s\" does not exist" ), torrentFile, folder );
+        tr_err( _( "Couldn't create \"%1$s\": \"%2$s\" is not a folder" ), torrentFile, folder );
         return ENOENT;
     }
 
@@ -241,8 +319,8 @@ TrOpenFile( int                      i,
     
     /* open the file */
     flags = doWrite ? ( O_RDWR | O_CREAT ) : O_RDONLY;
-#ifdef O_RANDOM
-    flags |= O_RANDOM
+#ifdef O_SEQUENTIAL
+    flags |= O_SEQUENTIAL;
 #endif
 #ifdef O_LARGEFILE
     flags |= O_LARGEFILE;
@@ -259,49 +337,49 @@ TrOpenFile( int                      i,
         return err;
     }
 
+    /* If the file already exists and it's too large, truncate it.
+     * This is a fringe case that happens if a torrent's been updated
+     * and one of the updated torrent's files is smaller.
+     * http://trac.transmissionbt.com/ticket/2228
+     * https://bugs.launchpad.net/ubuntu/+source/transmission/+bug/318249
+     */
+    if( alreadyExisted && ( desiredFileSize < (uint64_t)sb.st_size ) )
+        ftruncate( file->fd, desiredFileSize );
+
     if( doWrite && !alreadyExisted && ( preallocationMode == TR_PREALLOCATE_SPARSE ) )
         preallocateFileSparse( file->fd, desiredFileSize );
 
-#if defined( SYS_DARWIN )
-    fcntl( file->fd, F_NOCACHE, 1 );
-    fcntl( file->fd, F_RDAHEAD, 0 );
-#elif defined( HAVE_POSIX_FADVISE )
-    posix_fadvise( file->fd, 0, 0, POSIX_FADV_RANDOM );
+#ifdef HAVE_POSIX_FADVISE
+    /* this doubles the OS level readahead buffer, which in practice
+     * turns out to be a good thing, because many (most?) clients request
+     * chunks of blocks in order */
+    posix_fadvise( file->fd, 0, 0, POSIX_FADV_SEQUENTIAL );
 #endif
 
     tr_free( filename );
     return 0;
 }
 
-static int
+static TR_INLINE tr_bool
 fileIsOpen( const struct tr_openfile * o )
 {
     return o->fd >= 0;
 }
 
 static void
-TrCloseFile( int i )
+TrCloseFile( struct tr_openfile * o )
 {
-    struct tr_openfile * o = &gFd->openFiles[i];
-
-    assert( i >= 0 );
-    assert( i < gFd->openFileLimit );
+    assert( o != NULL );
     assert( fileIsOpen( o ) );
 
-    close( o->fd );
+    tr_close_file( o->fd );
     o->fd = -1;
-    o->isCheckedOut = 0;
-}
-
-static int
-fileIsCheckedOut( const struct tr_openfile * o )
-{
-    return fileIsOpen( o ) && o->isCheckedOut;
 }
 
 /* returns an fd on success, or a -1 on failure and sets errno */
 int
-tr_fdFileCheckout( const char             * folder,
+tr_fdFileCheckout( int                      torrentId,
+                   const char             * folder,
                    const char             * torrentFile,
                    tr_bool                  doWrite,
                    tr_preallocation_mode    preallocationMode,
@@ -311,41 +389,30 @@ tr_fdFileCheckout( const char             * folder,
     struct tr_openfile * o;
     char filename[MAX_PATH_LENGTH];
 
+    assert( torrentId > 0 );
     assert( folder && *folder );
     assert( torrentFile && *torrentFile );
-    assert( doWrite == 0 || doWrite == 1 );
+    assert( tr_isBool( doWrite ) );
 
     tr_snprintf( filename, sizeof( filename ), "%s%c%s", folder, TR_PATH_DELIMITER, torrentFile );
     dbgmsg( "looking for file '%s', writable %c", filename, doWrite ? 'y' : 'n' );
 
-    tr_lockLock( gFd->lock );
-
-    /* Is it already open? */
-    for( i = 0; i < gFd->openFileLimit; ++i )
+    /* is it already open? */
+    for( i=0; i<gFd->openFileLimit; ++i )
     {
         o = &gFd->openFiles[i];
 
         if( !fileIsOpen( o ) )
             continue;
-
+        if( torrentId != o->torrentId )
+            continue;
         if( strcmp( filename, o->filename ) )
             continue;
 
-        if( fileIsCheckedOut( o ) )
-        {
-            dbgmsg( "found it!  it's open, but checked out.  waiting..." );
-            tr_lockUnlock( gFd->lock );
-            tr_wait( 200 );
-            tr_lockLock( gFd->lock );
-            i = -1; /* reloop */
-            continue;
-        }
-
         if( doWrite && !o->isWritable )
         {
-            dbgmsg(
-                "found it!  it's open and available, but isn't writable. closing..." );
-            TrCloseFile( i );
+            dbgmsg( "found it!  it's open and available, but isn't writable. closing..." );
+            TrCloseFile( o );
             break;
         }
 
@@ -354,14 +421,13 @@ tr_fdFileCheckout( const char             * folder,
         break;
     }
 
-    dbgmsg(
-        "it's not already open.  looking for an open slot or an old file." );
+    dbgmsg( "it's not already open.  looking for an open slot or an old file." );
     while( winner < 0 )
     {
         uint64_t date = tr_date( ) + 1;
 
         /* look for the file that's been open longest */
-        for( i = 0; i < gFd->openFileLimit; ++i )
+        for( i=0; i<gFd->openFileLimit; ++i )
         {
             o = &gFd->openFiles[i];
 
@@ -379,22 +445,12 @@ tr_fdFileCheckout( const char             * folder,
             }
         }
 
-        if( winner >= 0 )
+        assert( winner >= 0 );
+       
+        if( fileIsOpen( &gFd->openFiles[winner] ) )
         {
-            if( fileIsOpen( &gFd->openFiles[winner] ) )
-            {
-                dbgmsg( "closing file '%s', slot #%d",
-                        gFd->openFiles[winner].filename,
-                        winner );
-                TrCloseFile( winner );
-            }
-        }
-        else
-        {
-            dbgmsg( "everything's full!  waiting for someone else to finish something" );
-            tr_lockUnlock( gFd->lock );
-            tr_wait( 200 );
-            tr_lockLock( gFd->lock );
+            dbgmsg( "closing file \"%s\"", gFd->openFiles[winner].filename );
+            TrCloseFile( &gFd->openFiles[winner] );
         }
     }
 
@@ -402,9 +458,9 @@ tr_fdFileCheckout( const char             * folder,
     o = &gFd->openFiles[winner];
     if( !fileIsOpen( o ) )
     {
-        const int err = TrOpenFile( winner, folder, torrentFile, doWrite, preallocationMode, desiredFileSize );
+        const int err = TrOpenFile( winner, folder, torrentFile, doWrite,
+                                    preallocationMode, desiredFileSize );
         if( err ) {
-            tr_lockUnlock( gFd->lock );
             errno = err;
             return -1;
         }
@@ -413,80 +469,53 @@ tr_fdFileCheckout( const char             * folder,
                 doWrite ? 'y' : 'n' );
         tr_strlcpy( o->filename, filename, sizeof( o->filename ) );
         o->isWritable = doWrite;
-        o->syncAt = time( NULL ) + SYNC_INTERVAL;
     }
 
     dbgmsg( "checking out '%s' in slot %d", filename, winner );
-    o->isCheckedOut = 1;
-    o->closeWhenDone = 0;
+    o->torrentId = torrentId;
     o->date = tr_date( );
-    tr_lockUnlock( gFd->lock );
     return o->fd;
 }
 
 void
 tr_fdFileReturn( int fd )
 {
-    int i;
+    struct tr_openfile * o;
+    const struct tr_openfile * end;
 
-    tr_lockLock( gFd->lock );
-
-    for( i = 0; i < gFd->openFileLimit; ++i )
+    for( o=gFd->openFiles, end=o+gFd->openFileLimit; o!=end; ++o )
     {
-        struct tr_openfile * o = &gFd->openFiles[i];
         if( o->fd != fd )
             continue;
-
-        dbgmsg( "releasing file '%s' in slot #%d", o->filename, i );
-        o->isCheckedOut = 0;
-        if( o->closeWhenDone )
-            TrCloseFile( i );
-        else if( o->syncAt <= time( NULL ) ) {
-            dbgmsg( "fsync()ing file '%s' in slot #%d", o->filename, i );
-            fsync( o->fd );
-#ifdef HAVE_POSIX_FADVISE
-            /* TODO: test performance with and without this */
-            posix_fadvise( o->fd, 0, 0, POSIX_FADV_DONTNEED );
-#endif
-            o->syncAt = time( NULL ) + SYNC_INTERVAL;
-        }
-
+        dbgmsg( "releasing file \"%s\"", o->filename );
         break;
     }
-
-    tr_lockUnlock( gFd->lock );
 }
 
 void
 tr_fdFileClose( const char * filename )
 {
-    int i;
+    struct tr_openfile * o;
+    const struct tr_openfile * end;
 
-    tr_lockLock( gFd->lock );
-
-    for( i = 0; i < gFd->openFileLimit; ++i )
+    for( o=gFd->openFiles, end=o+gFd->openFileLimit; o!=end; ++o )
     {
-        struct tr_openfile * o = &gFd->openFiles[i];
         if( !fileIsOpen( o ) || strcmp( filename, o->filename ) )
             continue;
-
-        dbgmsg( "tr_fdFileClose closing '%s'", filename );
-
-        if( !o->isCheckedOut )
-        {
-            dbgmsg( "not checked out, so closing it now... '%s'", filename );
-            TrCloseFile( i );
-        }
-        else
-        {
-            dbgmsg(
-                "flagging file '%s', slot #%d to be closed when checked in",
-                gFd->openFiles[i].filename, i );
-            o->closeWhenDone = 1;
-        }
+        dbgmsg( "tr_fdFileClose closing \"%s\"", filename );
+        TrCloseFile( o );
     }
+}
 
-    tr_lockUnlock( gFd->lock );
+void
+tr_fdTorrentClose( int torrentId )
+{
+    struct tr_openfile * o;
+    const struct tr_openfile * end;
+
+    for( o=gFd->openFiles, end=o+gFd->openFileLimit; o!=end; ++o )
+        if( fileIsOpen( o ) && o->torrentId == torrentId )
+            TrCloseFile( o );
 }
 
 /***
@@ -495,7 +524,7 @@ tr_fdFileClose( const char * filename )
 ****
 ***/
 
-static int
+static TR_INLINE int
 getSocketMax( struct tr_fd_s * gFd )
 {
     return gFd->socketLimit;
@@ -506,8 +535,6 @@ tr_fdSocketCreate( int domain, int type )
 {
     int s = -1;
 
-    tr_lockLock( gFd->lock );
-
     if( gFd->socketCount < getSocketMax( gFd ) )
         if( ( s = socket( domain, type, 0 ) ) < 0 )
         {
@@ -516,7 +543,6 @@ tr_fdSocketCreate( int domain, int type )
 #endif
             tr_err( _( "Couldn't create socket: %s" ),
                    tr_strerror( sockerrno ) );
-            s = -sockerrno;
         }
 
     if( s > -1 )
@@ -524,7 +550,6 @@ tr_fdSocketCreate( int domain, int type )
 
     assert( gFd->socketCount >= 0 );
 
-    tr_lockUnlock( gFd->lock );
     return s;
 }
 
@@ -533,64 +558,62 @@ tr_fdSocketAccept( int           b,
                    tr_address  * addr,
                    tr_port     * port )
 {
-    int                s = -1;
-    unsigned int       len;
+    int s;
+    unsigned int len;
     struct sockaddr_storage sock;
 
     assert( addr );
     assert( port );
 
-    tr_lockLock( gFd->lock );
-    if( gFd->socketCount < getSocketMax( gFd ) )
+    len = sizeof( struct sockaddr_storage );
+    s = accept( b, (struct sockaddr *) &sock, &len );
+
+    if( ( s >= 0 ) && gFd->socketCount > getSocketMax( gFd ) )
     {
-        len = sizeof( struct sockaddr_storage );
-        s = accept( b, (struct sockaddr *) &sock, &len );
+        EVUTIL_CLOSESOCKET( s );
+        s = -1;
     }
-    if( s > -1 )
+
+    if( s >= 0 )
     {
         /* "The ss_family field of the sockaddr_storage structure will always 
          * align with the family field of any protocol-specific structure." */ 
         if( sock.ss_family == AF_INET ) 
-        { 
-            struct sockaddr_in * sock4 = (struct sockaddr_in *)&sock; 
+        {
+            struct sockaddr_in *si;
+            union { struct sockaddr_storage dummy; struct sockaddr_in si; } s;
+            s.dummy = sock;
+            si = &s.si;
             addr->type = TR_AF_INET; 
-            addr->addr.addr4.s_addr = sock4->sin_addr.s_addr; 
-            *port = sock4->sin_port; 
+            addr->addr.addr4.s_addr = si->sin_addr.s_addr; 
+            *port = si->sin_port; 
         } 
         else 
         { 
-            struct sockaddr_in6 * sock6 = (struct sockaddr_in6 *)&sock; 
+            struct sockaddr_in6 *si;
+            union { struct sockaddr_storage dummy; struct sockaddr_in6 si; } s;
+            s.dummy = sock;
+            si = &s.si;
             addr->type = TR_AF_INET6; 
-            addr->addr.addr6 = sock6->sin6_addr;
-            *port = sock6->sin6_port; 
+            addr->addr.addr6 = si->sin6_addr;
+            *port = si->sin6_port; 
         } 
         ++gFd->socketCount;
     }
-    tr_lockUnlock( gFd->lock );
 
     return s;
 }
 
-static void
-socketClose( int fd )
-{
-    EVUTIL_CLOSESOCKET( fd );
-}
-
 void
-tr_fdSocketClose( int s )
+tr_fdSocketClose( int fd )
 {
-    tr_lockLock( gFd->lock );
-
-    if( s >= 0 )
+    if( fd >= 0 )
     {
-        socketClose( s );
+        EVUTIL_CLOSESOCKET( fd );
         --gFd->socketCount;
     }
 
     assert( gFd->socketCount >= 0 );
-
-    tr_lockUnlock( gFd->lock );
 }
 
 /***
@@ -608,7 +631,6 @@ tr_fdInit( size_t openFileLimit, size_t socketLimit )
     gFd = tr_new0( struct tr_fd_s, 1 );
     gFd->openFiles = tr_new0( struct tr_openfile, openFileLimit );
     gFd->openFileLimit = openFileLimit;
-    gFd->lock = tr_lockNew( );
 
 #ifdef HAVE_GETRLIMIT
     {
@@ -632,13 +654,12 @@ tr_fdInit( size_t openFileLimit, size_t socketLimit )
 void
 tr_fdClose( void )
 {
-    int i = 0;
+    struct tr_openfile * o;
+    const struct tr_openfile * end;
 
-    for( i = 0; i < gFd->openFileLimit; ++i )
-        if( fileIsOpen( &gFd->openFiles[i] ) )
-            TrCloseFile( i );
-
-    tr_lockFree( gFd->lock );
+    for( o=gFd->openFiles, end=o+gFd->openFileLimit; o!=end; ++o )
+        if( fileIsOpen( o ) )
+            TrCloseFile( o );
 
     tr_free( gFd->openFiles );
     tr_free( gFd );
@@ -657,4 +678,3 @@ tr_fdGetPeerLimit( void )
 {
     return gFd ? gFd->socketLimit : -1;
 }
-
