@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fdlimit.h 7722 2009-01-16 17:31:10Z charles $
+ * $Id: fdlimit.h 8725 2009-06-21 07:36:51Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -29,9 +29,25 @@
 #include "transmission.h"
 #include "net.h"
 
+/**
+ * @addtogroup file_io File IO
+ * @{
+ */
+
 void tr_fdInit( size_t openFileLimit,
                 size_t globalPeerLimit );
 
+int tr_open_file_for_scanning( const char * filename );
+
+int tr_open_file_for_writing( const char * filename );
+
+void tr_close_file( int fd );
+
+tr_bool tr_preallocate_file( const char * filename, uint64_t length );
+
+int64_t tr_lseek( int fd, int64_t offset, int whence );
+
+
 /**
  * Returns an fd to the specified filename.
  *
@@ -51,7 +67,8 @@ void tr_fdInit( size_t openFileLimit,
  * @see tr_fdFileReturn
  * @see tr_fdFileClose
  */
-int  tr_fdFileCheckout( const char             * folder,
+int  tr_fdFileCheckout( int                      torrentId,
+                        const char             * folder,
                         const char             * torrentFile,
                         tr_bool                  doWrite,
                         tr_preallocation_mode    preallocationMode,
@@ -77,6 +94,12 @@ void tr_fdFileReturn( int file );
 void     tr_fdFileClose( const char * filename );
 
 
+/**
+ * Closes all the files associated with a given torrent id
+ */
+void tr_fdTorrentClose( int torrentId );
+
+
 /***********************************************************************
  * Sockets
  **********************************************************************/
@@ -100,3 +123,4 @@ void     tr_fdSetPeerLimit( uint16_t n );
 
 uint16_t tr_fdGetPeerLimit( void );
 
+/* @} */
