@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: list.c 7813 2009-01-29 18:18:24Z charles $
+ * $Id: list.c 8762 2009-06-30 16:40:34Z charles $
  */
 
 #include "transmission.h"
@@ -165,12 +165,6 @@ tr_list_size( const tr_list * list )
  * insert/remove operations
  */
 
-void
-__tr_list_init( struct __tr_list * head )
-{
-    head->next = head;
-    head->prev = head;
-}
 
 void
 __tr_list_insert( struct __tr_list * list,
@@ -192,13 +186,6 @@ __tr_list_splice( struct __tr_list * prev,
 }
 
  
-void
-__tr_list_append( struct __tr_list * head,
-		  struct __tr_list * list)
-{
-    __tr_list_insert( list, head->prev, head );
-}
-
 void
 __tr_list_remove( struct __tr_list * head )
 {
