@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: metainfo.c 7664 2009-01-11 17:46:51Z charles $
+ * $Id: metainfo.c 8776 2009-07-03 14:37:27Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -57,23 +57,35 @@ getTorrentFilename( const tr_session * session,
 }
 
 static char*
-getOldTorrentFilename( const tr_session * session,
-                       const tr_info *   inf )
+getOldTorrentFilename( const tr_session * session, const tr_info * inf )
 {
-    char *            ret;
-    struct evbuffer * buf = tr_getBuffer( );
+    int i;
+    char * path;
+    struct stat sb;
+    const int tagCount = 5;
+    const char * tags[] = { "beos", "cli", "daemon", "macosx", "wx" };
 
-    evbuffer_add_printf( buf, "%s%c%s", tr_getTorrentDir( session ),
-                         TR_PATH_DELIMITER,
-                         inf->hashString );
-    if( session->tag )
-        evbuffer_add_printf( buf, "-%s", session->tag );
+    /* test the beos, cli, daemon, macosx, wx tags */
+    for( i=0; i<tagCount; ++i ) {
+        path = tr_strdup_printf( "%s%c%s-%s", tr_getTorrentDir( session ), '/', inf->hashString, tags[i] );
+        if( !stat( path, &sb ) && ( ( sb.st_mode & S_IFMT ) == S_IFREG ) )
+            return path;
+        tr_free( path );
+    }
 
-    ret = tr_strndup( EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ) );
-    tr_releaseBuffer( buf );
-    return ret;
+    /* test a non-tagged file */
+    path = tr_buildPath( tr_getTorrentDir( session ), inf->hashString, NULL );
+    if( !stat( path, &sb ) && ( ( sb.st_mode & S_IFMT ) == S_IFREG ) )
+        return path;
+    tr_free( path );
+
+    /* return the -gtk form by default, since that's the most common case.
+       don't bother testing stat() on it since this is the last candidate
+       and we don't want to return NULL anyway */
+    return tr_strdup_printf( "%s%c%s-%s", tr_getTorrentDir( session ), '/', inf->hashString, "gtk" );
 }
 
+/* this is for really old versions of T and will probably be removed someday */
 void
 tr_metainfoMigrate( tr_session * session,
                     tr_info *   inf )
@@ -136,7 +148,7 @@ getfile( char        ** setme,
     }
     else
     {
-        struct evbuffer * buf = tr_getBuffer( );
+        struct evbuffer * buf = evbuffer_new( );
         int               n = tr_bencListSize( path );
         int               i;
 
@@ -154,7 +166,7 @@ getfile( char        ** setme,
 
         *setme = tr_utf8clean( (char*)EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ), NULL );
         /* fprintf( stderr, "[%s]\n", *setme ); */
-        tr_releaseBuffer( buf );
+        evbuffer_free( buf );
         err = 0;
     }
 
@@ -230,15 +242,19 @@ announceToScrape( const char * announce )
      * it will be taken as a sign that that tracker doesn't support
      * the scrape convention. If it does, substitute 'scrape' for
      * 'announce' to find the scrape page.  */
-    if( ( ( s =
-               strrchr( announce, '/' ) ) ) && !strncmp( ++s, "announce", 8 ) )
+    if( ( ( s = strrchr( announce, '/' ) ) ) && !strncmp( ++s, "announce", 8 ) )
     {
-        struct evbuffer * buf = tr_getBuffer( );
-        evbuffer_add( buf, announce, s - announce );
-        evbuffer_add( buf, "scrape", 6 );
-        evbuffer_add_printf( buf, "%s", s + 8 );
-        scrape = tr_strdup( EVBUFFER_DATA( buf ) );
-        tr_releaseBuffer( buf );
+        const char * prefix = announce;
+        const size_t prefix_len = s - announce;
+        const char * suffix = s + 8;
+        const size_t suffix_len = strlen( suffix );
+        const size_t alloc_len = prefix_len + 6 + suffix_len + 1;
+        char * walk = scrape = tr_new( char, alloc_len );
+        memcpy( walk, prefix, prefix_len ); walk += prefix_len;
+        memcpy( walk, "scrape", 6 );        walk += 6;
+        memcpy( walk, suffix, suffix_len ); walk += suffix_len;
+        *walk++ = '\0';
+        assert( walk - scrape == (int)alloc_len );
     }
 
     return scrape;
@@ -365,8 +381,8 @@ tr_metainfoParseImpl( const tr_session * session,
         return "info";
     else
     {
-        int    len;
-        char * bstr = tr_bencSave( beInfo, &len );
+        int len;
+        char * bstr = tr_bencToStr( beInfo, TR_FMT_BENC, &len );
         tr_sha1( inf->hash, bstr, len, NULL );
         tr_sha1_to_hex( inf->hashString, inf->hash );
         tr_free( bstr );
@@ -440,7 +456,7 @@ tr_metainfoParseImpl( const tr_session * session,
 
     /* filename of Transmission's copy */
     tr_free( inf->torrent );
-    inf->torrent = getTorrentFilename( session, inf );
+    inf->torrent = session ?  getTorrentFilename( session, inf ) : NULL;
 
     return NULL;
 }
