@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: metainfo.h 4404 2008-01-01 17:20:20Z charles $
+ * $Id: metainfo.h 8637 2009-06-06 17:39:04Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -22,17 +22,25 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
+#ifndef __TRANSMISSION__
+#error only libtransmission should #include this header.
+#endif
+
 #ifndef TR_METAINFO_H
 #define TR_METAINFO_H 1
 
 #include "transmission.h"
 
-struct benc_val_t;
+struct tr_benc;
 
-int tr_metainfoParse( tr_info *, const struct benc_val_s *, const char * tag );
-void tr_metainfoFree( tr_info * inf );
-void tr_metainfoRemoveSaved( const char * hashString, const char * tag );
+int  tr_metainfoParse( const tr_session     * session,
+                       tr_info              * info,
+                       const struct tr_benc * benc );
 
-int tr_metainfoSave( const char *hashString, const char * tag, const uint8_t * metainfo, size_t len );
+void tr_metainfoRemoveSaved( const tr_session * session,
+                             const tr_info    * info );
+
+void tr_metainfoMigrate( tr_session * session,
+                         tr_info    * inf );
 
 #endif
