@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: net.c 7852 2009-02-08 22:53:06Z jhujhiti $
+ * $Id: net.c 8791 2009-07-09 18:14:33Z titer $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -38,7 +38,9 @@
  #include <netdb.h>
  #include <fcntl.h>
 #endif
+#include <unistd.h>
 
+#include <stdarg.h> /* 1.4.x versions of evutil.h need this */
 #include <evutil.h>
 
 #include "transmission.h"
@@ -47,6 +49,7 @@
 #include "net.h"
 #include "peer-io.h"
 #include "platform.h"
+#include "session.h"
 #include "utils.h"
 
 #ifndef IN_MULTICAST
@@ -54,31 +57,30 @@
 #endif
 
 const tr_address tr_in6addr_any = { TR_AF_INET6, { IN6ADDR_ANY_INIT } }; 
-const tr_address tr_inaddr_any = { TR_AF_INET, 
-    { { { { INADDR_ANY, 0x00, 0x00, 0x00 } } } } }; 
+const tr_address tr_inaddr_any = { TR_AF_INET, { { { { INADDR_ANY, 0x00, 0x00, 0x00 } } } } }; 
 
 #ifdef WIN32
 static const char *
-inet_ntop(int af, const void *src, char *dst, socklen_t cnt)
+inet_ntop( int af, const void *src, char *dst, socklen_t cnt )
 {
     if (af == AF_INET)
     {
         struct sockaddr_in in;
-        memset(&in, 0, sizeof(in));
+        memset( &in, 0, sizeof( in ) );
         in.sin_family = AF_INET;
-        memcpy(&in.sin_addr, src, sizeof(struct in_addr));
-        getnameinfo((struct sockaddr *)&in, sizeof(struct
-            sockaddr_in), dst, cnt, NULL, 0, NI_NUMERICHOST);
+        memcpy( &in.sin_addr, src, sizeof( struct in_addr ) );
+        getnameinfo((struct sockaddr *)&in, sizeof(struct sockaddr_in),
+                    dst, cnt, NULL, 0, NI_NUMERICHOST);
         return dst;
     }
     else if (af == AF_INET6)
     {
         struct sockaddr_in6 in;
-        memset(&in, 0, sizeof(in));
+        memset( &in, 0, sizeof( in ) );
         in.sin6_family = AF_INET6;
-        memcpy(&in.sin6_addr, src, sizeof(struct in_addr6));
-        getnameinfo((struct sockaddr *)&in, sizeof(struct
-            sockaddr_in6), dst, cnt, NULL, 0, NI_NUMERICHOST);
+        memcpy( &in.sin6_addr, src, sizeof( struct in_addr6 ) );
+        getnameinfo((struct sockaddr *)&in, sizeof(struct sockaddr_in6),
+                    dst, cnt, NULL, 0, NI_NUMERICHOST);
         return dst;
     }
     return NULL;
@@ -127,40 +129,6 @@ tr_netInit( void )
     }
 }
 
-void
-tr_suspectAddress( const tr_address * a UNUSED, const char * source UNUSED )
-{
-/* this is overkill for a production environment,
- * but useful in the nightly builds, so only compile it into the nightlies */
-#ifdef TR_UNSTABLE
-    /* be really aggressive in what we report */
-    if( a->type == TR_AF_INET && !( ntohl( a->addr.addr4.s_addr ) & 0xff000000 ) )
-        tr_err(  "Funny looking address %s from %s", tr_ntop_non_ts( a ), source );
-    /* /16s taken from ipv6 rib on 21 dec, 2008 */
-    /* this is really, really ugly. expedience over quality */
-    if( a->type == TR_AF_INET6 )
-    {
-        uint16_t slash16;
-        uint16_t valid[] = { 0x339, 0x2002, 0x2003, 0x2400, 0x2401, 0x2402,
-            0x2403, 0x2404, 0x2405, 0x2406, 0x2407, 0x2600, 0x2607, 0x2610,
-            0x2620, 0x2800, 0x2801, 0x2a00, 0x2a01, 0x0a02, 0x2001, 0x0000 };
-        uint16_t *p;
-        tr_bool good = FALSE;
-        p = valid;
-        memcpy( &slash16, &a->addr, 2 );
-        slash16 = ntohs( slash16 );
-        while( *p )
-        {
-            if( slash16 == *p )
-                good = TRUE;
-            p++;
-        }
-        if( !good && !IN6_IS_ADDR_V4MAPPED( &a->addr.addr6 ) )
-            tr_err(  "Funny looking address %s from %s", tr_ntop_non_ts( a ), source );
-    }
-#endif
-}
-
 const char * 
 tr_ntop( const tr_address * src, char * dst, int size ) 
 {
@@ -188,6 +156,7 @@ tr_address *
 tr_pton( const char * src, tr_address * dst ) 
 { 
     int retval = inet_pton( AF_INET, src, &dst->addr ); 
+    assert( dst );
     if( retval < 0 ) 
         return NULL; 
     else if( retval == 0 ) 
@@ -204,18 +173,6 @@ tr_pton( const char * src, tr_address * dst )
     return dst; 
 }
 
-void
-tr_normalizeV4Mapped( tr_address * const addr )
-{
-    assert( tr_isAddress( addr ) );
-
-    if( addr->type == TR_AF_INET6 && IN6_IS_ADDR_V4MAPPED( &addr->addr.addr6 ) )
-    {
-        addr->type = TR_AF_INET;
-        memcpy( &addr->addr.addr4.s_addr, addr->addr.addr6.s6_addr + 12, 4 );
-    }
-}
-
 /* 
  * Compare two tr_address structures. 
  * Returns: 
@@ -226,7 +183,7 @@ tr_normalizeV4Mapped( tr_address * const addr )
 int
 tr_compareAddresses( const tr_address * a, const tr_address * b)
 {
-    int addrlen;
+    static const int sizes[2] = { sizeof(struct in_addr), sizeof(struct in6_addr) };
 
     assert( tr_isAddress( a ) );
     assert( tr_isAddress( b ) );
@@ -235,137 +192,9 @@ tr_compareAddresses( const tr_address * a, const tr_address * b)
     if( a->type != b->type )
         return a->type == TR_AF_INET ? 1 : -1;
 
-    if( a->type == TR_AF_INET ) 
-        addrlen = sizeof( struct in_addr ); 
-    else 
-        addrlen = sizeof( struct in6_addr ); 
-    return memcmp( &a->addr, &b->addr, addrlen );
+    return memcmp( &a->addr, &b->addr, sizes[a->type] );
 } 
 
-tr_bool
-tr_net_hasIPv6( tr_port port )
-{
-    static tr_bool alreadyDone = FALSE;
-    static tr_bool result      = FALSE;
-    int s;
-    if( alreadyDone )
-        return result;
-    s = tr_netBindTCP( &tr_in6addr_any, port, TRUE );
-    if( s >= 0 || -s != EAFNOSUPPORT ) /* we support ipv6 */
-    {
-        result = TRUE;
-        tr_netClose( s );
-    }
-    alreadyDone = TRUE;
-    return result;
-}
-
-/***********************************************************************
- * Socket list housekeeping
- **********************************************************************/
-struct tr_socketList
-{
-    int             socket;
-    tr_address      addr;
-    tr_socketList * next;
-};
-
-tr_socketList *
-tr_socketListAppend( tr_socketList * const head,
-                     const tr_address * const addr )
-{
-    tr_socketList * tmp;
-
-    assert( head );
-    assert( tr_isAddress( addr ) );
-
-    for( tmp = head; tmp->next; tmp = tmp->next );
-    tmp->next = tr_socketListNew( addr );
-    return tmp->next;
-}
-
-tr_socketList *
-tr_socketListNew( const tr_address * const addr )
-{
-    tr_socketList * tmp;
-
-    assert( tr_isAddress( addr ) );
-
-    tmp = tr_new( tr_socketList, 1 );
-    tmp->socket = -1;
-    tmp->addr = *addr;
-    tmp->next = NULL;
-    return tmp;
-}
-
-void
-tr_socketListFree( tr_socketList * const head )
-{
-    assert( head );
-
-    if( head->next )
-        tr_socketListFree( head->next );
-    tr_free( head );
-}
-
-void
-tr_socketListRemove( tr_socketList * const head,
-                     tr_socketList * const el)
-{
-    tr_socketList * tmp;
-
-    assert( head );
-    assert( el );
-
-    for( tmp = head; tmp->next && tmp->next != el; tmp = tmp->next );
-    tmp->next = el->next;
-    el->next = NULL;
-    tr_socketListFree(el);
-}
-
-void
-tr_socketListTruncate( tr_socketList * const head,
-                       tr_socketList * const start )
-{
-    tr_socketList * tmp;
-
-    assert( head );
-    assert( start );
-
-    for( tmp = head; tmp->next && tmp->next != start; tmp = tmp->next );
-    tr_socketListFree( start );
-    tmp->next = NULL;
-}
-
-#if 0
-int
-tr_socketListGetSocket( const tr_socketList * const el )
-{
-    assert( el );
-
-    return el->socket;
-}
-
-const tr_address *
-tr_socketListGetAddress( const tr_socketList * const el )
-{
-    assert( el );
-    return &el->addr;
-}
-#endif
-
-void
-tr_socketListForEach( tr_socketList * const head,
-                      void ( * cb ) ( int * const,
-                                      tr_address * const,
-                                      void * const),
-                      void * const userData )
-{
-    tr_socketList * tmp;
-    for( tmp = head; tmp; tmp = tmp->next )
-        cb( &tmp->socket, &tmp->addr, userData );
-}
-
 /***********************************************************************
  * TCP sockets
  **********************************************************************/
@@ -380,57 +209,16 @@ tr_netSetTOS( int s, int tos )
 #endif
 }
 
-static int
-makeSocketNonBlocking( int fd )
-{
-    if( fd >= 0 )
-    {
-        if( evutil_make_socket_nonblocking( fd ) )
-        {
-            int tmperrno;
-            tr_err( _( "Couldn't create socket: %s" ),
-                   tr_strerror( sockerrno ) );
-            tmperrno = sockerrno;
-            tr_netClose( fd );
-            fd = -tmperrno;
-        }
-    }
-
-    return fd;
-}
-
-static int
-createSocket( int domain, int type )
-{
-    return makeSocketNonBlocking( tr_fdSocketCreate( domain, type ) );
-}
-
-static void
-setSndBuf( tr_session * session UNUSED, int fd UNUSED )
-{
-#if 0
-    if( fd >= 0 )
-    {
-        const int sndbuf = session->so_sndbuf;
-        const int rcvbuf = session->so_rcvbuf;
-        setsockopt( fd, SOL_SOCKET, SO_SNDBUF, &sndbuf, sizeof( sndbuf ) );
-        setsockopt( fd, SOL_SOCKET, SO_RCVBUF, &rcvbuf, sizeof( rcvbuf ) );
-    }
-#endif
-}
-
 static socklen_t
 setup_sockaddr( const tr_address        * addr,
                 tr_port                   port,
                 struct sockaddr_storage * sockaddr)
 {
-    struct sockaddr_in  sock4;
-    struct sockaddr_in6 sock6;
-
     assert( tr_isAddress( addr ) );
 
     if( addr->type == TR_AF_INET )
     {
+        struct sockaddr_in  sock4;
         memset( &sock4, 0, sizeof( sock4 ) );
         sock4.sin_family      = AF_INET;
         sock4.sin_addr.s_addr = addr->addr.addr4.s_addr;
@@ -440,11 +228,12 @@ setup_sockaddr( const tr_address        * addr,
     }
     else
     {
+        struct sockaddr_in6 sock6;
         memset( &sock6, 0, sizeof( sock6 ) );
-        sock6.sin6_family = AF_INET6;
-        sock6.sin6_port = port;
+        sock6.sin6_family   = AF_INET6;
+        sock6.sin6_port     = port;
         sock6.sin6_flowinfo = 0;
-        sock6.sin6_addr = addr->addr.addr6;
+        sock6.sin6_addr     = addr->addr.addr6;
         memcpy( sockaddr, &sock6, sizeof( sock6 ) );
         return sizeof( struct sockaddr_in6 );
     }
@@ -462,11 +251,23 @@ isMulticastAddress( const tr_address * addr )
     return FALSE;
 }
 
+static TR_INLINE tr_bool
+isIPv4MappedOrCompatAddress( const tr_address * addr )
+{
+    if( addr->type == TR_AF_INET6 )
+    {
+        if( IN6_IS_ADDR_V4MAPPED( &addr->addr.addr6 ) ||
+            IN6_IS_ADDR_V4COMPAT( &addr->addr.addr6 ) )
+            return TRUE;
+    }
+    return FALSE;
+}
+
 static TR_INLINE tr_bool
 isIPv6LinkLocalAddress( const tr_address * addr )
 {
     if( addr->type == TR_AF_INET6 &&
-            IN6_IS_ADDR_LINKLOCAL( &addr->addr.addr6 ))
+        IN6_IS_ADDR_LINKLOCAL( &addr->addr.addr6 ) )
         return TRUE;
     return FALSE;
 }
@@ -474,7 +275,8 @@ isIPv6LinkLocalAddress( const tr_address * addr )
 tr_bool
 tr_isValidPeerAddress( const tr_address * addr, tr_port port )
 {
-    if( isMulticastAddress( addr ) || isIPv6LinkLocalAddress( addr ) )
+    if( isMulticastAddress( addr ) || isIPv6LinkLocalAddress( addr ) ||
+        isIPv4MappedOrCompatAddress( addr ) )
         return FALSE;
 
     if( port == 0 )
@@ -488,22 +290,40 @@ tr_netOpenTCP( tr_session        * session,
                const tr_address  * addr,
                tr_port             port )
 {
+    static const int domains[NUM_TR_AF_INET_TYPES] = { AF_INET, AF_INET6 };
     int                     s;
     struct sockaddr_storage sock;
-    const int               type = SOCK_STREAM;
     socklen_t               addrlen;
+    const tr_address      * source_addr;
+    socklen_t               sourcelen;
+    struct sockaddr_storage source_sock;
 
     assert( tr_isAddress( addr ) );
 
-    if( isMulticastAddress( addr ) || isIPv6LinkLocalAddress( addr ))
+    if( isMulticastAddress( addr ) || isIPv6LinkLocalAddress( addr ) )
         return -EINVAL;
 
-    if( ( s = createSocket( ( addr->type == TR_AF_INET ? AF_INET : AF_INET6 ), type ) ) < 0 )
-        return s;
+    s = tr_fdSocketCreate( domains[addr->type], SOCK_STREAM );
+    if( s < 0 )
+        return -1;
 
-    setSndBuf( session, s );
+    if( evutil_make_socket_nonblocking( s ) < 0 ) {
+        tr_netClose( s );
+        return -1;
+    }
 
     addrlen = setup_sockaddr( addr, port, &sock );
+    
+    /* set source address */
+    source_addr = tr_sessionGetPublicAddress( session, addr->type );
+    assert( source_addr );
+    sourcelen = setup_sockaddr( source_addr, 0, &source_sock );
+    if( bind( s, ( struct sockaddr * ) &source_sock, sourcelen ) )
+    {
+        tr_err( _( "Couldn't set source address %s on %d: %s" ),
+                tr_ntop_non_ts( source_addr ), s, tr_strerror( errno ) );
+        return -errno;
+    }
 
     if( ( connect( s, (struct sockaddr *) &sock,
                   addrlen ) < 0 )
@@ -529,68 +349,105 @@ tr_netOpenTCP( tr_session        * session,
     return s;
 }
 
-int
-tr_netBindTCP( const tr_address * addr, tr_port port, tr_bool suppressMsgs )
+static int
+tr_netBindTCPImpl( const tr_address * addr, tr_port port, tr_bool suppressMsgs, int * errOut )
 {
-    int                     s;
+    static const int domains[NUM_TR_AF_INET_TYPES] = { AF_INET, AF_INET6 };
     struct sockaddr_storage sock;
-    const int               type = SOCK_STREAM;
-    int                     addrlen;
-    int                     retval;
-
-#if defined( SO_REUSEADDR ) || defined( SO_REUSEPORT ) || defined( IPV6_V6ONLY )
-    int                optval = 1;
-#endif
+    int fd;
+    int addrlen;
+    int optval;
 
     assert( tr_isAddress( addr ) );
 
-    if( ( s = createSocket( ( addr->type == TR_AF_INET ? AF_INET : AF_INET6 ),
-                            type ) ) < 0 )
-        return s;
+    fd = socket( domains[addr->type], SOCK_STREAM, 0 );
+    if( fd < 0 ) {
+        *errOut = sockerrno;
+        return -1;
+    }
 
-#ifdef SO_REUSEADDR
-    setsockopt( s, SOL_SOCKET, SO_REUSEADDR, (char*)&optval, sizeof( optval ) );
-#endif
+    if( evutil_make_socket_nonblocking( fd ) < 0 ) {
+        *errOut = sockerrno;
+        EVUTIL_CLOSESOCKET( fd );
+        return -1;
+    }
+
+    optval = 1;
+    setsockopt( fd, SOL_SOCKET, SO_KEEPALIVE, &optval, sizeof(optval) );
+    setsockopt( fd, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval) );
 
 #ifdef IPV6_V6ONLY
-    if( addr->type == TR_AF_INET6 && 
-        ( retval = setsockopt( s, IPPROTO_IPV6, IPV6_V6ONLY, &optval,
-                             sizeof( optval ) ) ) == -1 ) {
-        /* the kernel may not support this. if not, ignore it */
-        if( errno != ENOPROTOOPT )
-            return -errno;
-    }
+    if( addr->type == TR_AF_INET6 )
+        if( setsockopt( fd, IPPROTO_IPV6, IPV6_V6ONLY, &optval, sizeof( optval ) ) == -1 )
+            if( sockerrno != ENOPROTOOPT ) { /* if the kernel doesn't support it, ignore it */
+                *errOut = sockerrno;
+                return -1;
+            }
 #endif
 
     addrlen = setup_sockaddr( addr, htons( port ), &sock );
-
-    if( bind( s, (struct sockaddr *) &sock,
-             addrlen ) )
-    {
-        int tmperrno;
+    if( bind( fd, (struct sockaddr *) &sock, addrlen ) ) {
+        const int err = sockerrno;
         if( !suppressMsgs )
-            tr_err( _( "Couldn't bind port %d on %s: %s" ), port,
-                    tr_ntop_non_ts( addr ), tr_strerror( sockerrno ) );
-        tmperrno = sockerrno;
-        tr_netClose( s );
-        return -tmperrno;
+            tr_err( _( "Couldn't bind port %d on %s: %s" ),
+                    port, tr_ntop_non_ts( addr ), tr_strerror( err ) );
+        EVUTIL_CLOSESOCKET( fd );
+        *errOut = err;
+        return -1;
     }
+
     if( !suppressMsgs )
-        tr_dbg(  "Bound socket %d to port %d on %s",
-                 s, port, tr_ntop_non_ts( addr ) );
-    return s;
+        tr_dbg( "Bound socket %d to port %d on %s", fd, port, tr_ntop_non_ts( addr ) );
+
+    if( listen( fd, 128 ) == -1 ) {
+        *errOut = sockerrno;
+        EVUTIL_CLOSESOCKET( fd );
+        return -1;
+    }
+
+    return fd;
 }
 
 int
-tr_netAccept( tr_session  * session,
+tr_netBindTCP( const tr_address * addr, tr_port port, tr_bool suppressMsgs )
+{
+    int unused;
+    return tr_netBindTCPImpl( addr, port, suppressMsgs, &unused );
+}
+
+tr_bool
+tr_net_hasIPv6( tr_port port )
+{
+    static tr_bool result = FALSE;
+    static tr_bool alreadyDone = FALSE;
+
+    if( !alreadyDone )
+    {
+        int err;
+        int fd = tr_netBindTCPImpl( &tr_in6addr_any, port, TRUE, &err );
+        if( fd >= 0 || err != EAFNOSUPPORT ) /* we support ipv6 */
+            result = TRUE;
+        if( fd >= 0 )
+            EVUTIL_CLOSESOCKET( fd );
+        alreadyDone = TRUE;
+    }
+
+    return result;
+}
+
+int
+tr_netAccept( tr_session  * session UNUSED,
               int           b,
               tr_address  * addr,
               tr_port     * port )
 {
-    int fd;
+    int fd = tr_fdSocketAccept( b, addr, port );
+
+    if( fd>=0 && evutil_make_socket_nonblocking(fd)<0 ) {
+        tr_netClose( fd );
+        fd = -1;
+    }
 
-    fd = makeSocketNonBlocking( tr_fdSocketAccept( b, addr, port ) );
-    setSndBuf( session, fd );
     return fd;
 }
 
