@@ -1,4 +1,3 @@
-
 /*
  * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
  *
@@ -8,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-mgr.c 7949 2009-02-25 22:15:19Z charles $
+ * $Id: peer-mgr.c 8782 2009-07-06 12:27:24Z charles $
  */
 
 #include <assert.h>
@@ -20,7 +19,6 @@
 #include <event.h>
 
 #include "transmission.h"
-#include "session.h"
 #include "bandwidth.h"
 #include "bencode.h"
 #include "blocklist.h"
@@ -35,6 +33,7 @@
 #include "peer-mgr.h"
 #include "peer-msgs.h"
 #include "ptrarray.h"
+#include "session.h"
 #include "stats.h" /* tr_statsAddUploaded, tr_statsAddDownloaded */
 #include "torrent.h"
 #include "trevent.h"
@@ -53,7 +52,7 @@ enum
     BANDWIDTH_PERIOD_MSEC = 500,
 
     /* how frequently to age out old piece request lists */
-    REFILL_UPKEEP_PERIOD_MSEC = 10000,
+    REFILL_UPKEEP_PERIOD_MSEC = ( 10 * 1000 ),
 
     /* how frequently to decide which peers live and die */
     RECONNECT_PERIOD_MSEC = 500,
@@ -134,20 +133,20 @@ struct tr_blockIterator
 
 typedef struct tr_torrent_peers
 {
-    tr_bool                    isRunning;
+    struct event               refillTimer;
 
-    uint8_t                    hash[SHA_DIGEST_LENGTH];
-    int                      * pendingRequestCount;
     tr_ptrArray                outgoingHandshakes; /* tr_handshake */
     tr_ptrArray                pool; /* struct peer_atom */
     tr_ptrArray                peers; /* tr_peer */
     tr_ptrArray                webseeds; /* tr_webseed */
-    tr_timer                 * refillTimer;
+
     tr_torrent               * tor;
     tr_peer                  * optimistic; /* the optimistic peer, or NULL if none */
     struct tr_blockIterator  * refillQueue; /* used in refillPulse() */
-
     struct tr_peerMgr        * manager;
+    int                      * pendingRequestCount;
+
+    tr_bool                    isRunning;
 }
 Torrent;
 
@@ -360,16 +359,14 @@ peerDestructor( tr_peer * peer )
 }
 
 static void
-removePeer( Torrent * t,
-            tr_peer * peer )
+removePeer( Torrent * t, tr_peer * peer )
 {
-    tr_peer *          removed;
-    struct peer_atom * atom;
+    tr_peer * removed;
+    struct peer_atom * atom = peer->atom;
 
     assert( torrentIsLocked( t ) );
-
-    atom = getExistingAtom( t, &peer->addr );
     assert( atom );
+
     atom->time = time( NULL );
 
     removed = tr_ptrArrayRemoveSorted( &t->peers, peer, peerCompare );
@@ -390,7 +387,6 @@ static void
 torrentDestructor( void * vt )
 {
     Torrent * t = vt;
-    uint8_t   hash[SHA_DIGEST_LENGTH];
 
     assert( t );
     assert( !t->isRunning );
@@ -398,9 +394,7 @@ torrentDestructor( void * vt )
     assert( tr_ptrArrayEmpty( &t->outgoingHandshakes ) );
     assert( tr_ptrArrayEmpty( &t->peers ) );
 
-    memcpy( hash, t->hash, SHA_DIGEST_LENGTH );
-
-    tr_timerFree( &t->refillTimer );
+    evtimer_del( &t->refillTimer );
 
     blockIteratorFree( &t->refillQueue );
     tr_ptrArrayDestruct( &t->webseeds, (PtrArrayForeachFunc)tr_webseedFree );
@@ -412,6 +406,9 @@ torrentDestructor( void * vt )
     tr_free( t );
 }
 
+
+static void refillPulse( int, short, void* );
+
 static void peerCallbackFunc( void * vpeer,
                               void * vevent,
                               void * vt );
@@ -430,7 +427,8 @@ torrentConstructor( tr_peerMgr * manager,
     t->peers = TR_PTR_ARRAY_INIT;
     t->webseeds = TR_PTR_ARRAY_INIT;
     t->outgoingHandshakes = TR_PTR_ARRAY_INIT;
-    memcpy( t->hash, tor->info.hash, SHA_DIGEST_LENGTH );
+    evtimer_set( &t->refillTimer, refillPulse, t );
+
 
     for( i = 0; i < tor->info.webseedCount; ++i )
     {
@@ -452,28 +450,33 @@ tr_peerMgr*
 tr_peerMgrNew( tr_session * session )
 {
     tr_peerMgr * m = tr_new0( tr_peerMgr, 1 );
-
     m->session = session;
     m->incomingHandshakes = TR_PTR_ARRAY_INIT;
-    m->bandwidthTimer    = tr_timerNew( session, bandwidthPulse, m, BANDWIDTH_PERIOD_MSEC );
-    m->rechokeTimer      = tr_timerNew( session, rechokePulse,   m, RECHOKE_PERIOD_MSEC );
-    m->reconnectTimer    = tr_timerNew( session, reconnectPulse, m, RECONNECT_PERIOD_MSEC );
-    m->refillUpkeepTimer = tr_timerNew( session, refillUpkeep,   m, REFILL_UPKEEP_PERIOD_MSEC );
-
-    rechokePulse( m );
-
     return m;
 }
 
+static void
+deleteTimers( struct tr_peerMgr * m )
+{
+    if( m->bandwidthTimer )
+        tr_timerFree( &m->bandwidthTimer );
+
+    if( m->rechokeTimer )
+        tr_timerFree( &m->rechokeTimer );
+
+    if( m->reconnectTimer )
+        tr_timerFree( &m->reconnectTimer );
+
+    if( m->refillUpkeepTimer )
+        tr_timerFree( &m->refillUpkeepTimer );
+}
+
 void
 tr_peerMgrFree( tr_peerMgr * manager )
 {
     managerLock( manager );
 
-    tr_timerFree( &manager->refillUpkeepTimer );
-    tr_timerFree( &manager->reconnectTimer );
-    tr_timerFree( &manager->rechokeTimer );
-    tr_timerFree( &manager->bandwidthTimer );
+    deleteTimers( manager );
 
     /* free the handshakes.  Abort invokes handshakeDoneCB(), which removes
      * the item from manager->handshakes, so this is a little roundabout... */
@@ -693,7 +696,7 @@ blockIteratorNext( struct tr_blockIterator * i, tr_block_index_t * setme )
         i->blockCount = 0;
         i->blockIndex = 0;
         for( block=b; block!=e; ++block )
-            if( !tr_cpBlockIsComplete( &tor->completion, block ) )
+            if( !tr_cpBlockIsCompleteFast( &tor->completion, block ) )
                 i->blocks[i->blockCount++] = block;
     }
 
@@ -786,8 +789,8 @@ refillUpkeep( void * vmgr )
     return TRUE;
 }
 
-static int
-refillPulse( void * vtorrent )
+static void
+refillPulse( int fd UNUSED, short type UNUSED, void * vtorrent )
 {
     tr_block_index_t block;
     int peerCount;
@@ -799,9 +802,9 @@ refillPulse( void * vtorrent )
     tr_bool hasNext = TRUE;
 
     if( !t->isRunning )
-        return TRUE;
+        return;
     if( tr_torrentIsSeed( t->tor ) )
-        return TRUE;
+        return;
 
     torrentLock( t );
     tordbg( t, "Refilling Request Buffers..." );
@@ -887,9 +890,7 @@ refillPulse( void * vtorrent )
         blockIteratorFree( &t->refillQueue );
     }
 
-    t->refillTimer = NULL;
     torrentUnlock( t );
-    return FALSE;
 }
 
 static void
@@ -911,8 +912,7 @@ broadcastGotBlock( Torrent * t, uint32_t index, uint32_t offset, uint32_t length
 }
 
 static void
-addStrike( Torrent * t,
-           tr_peer * peer )
+addStrike( Torrent * t, tr_peer * peer )
 {
     tordbg( t, "increasing peer %s strike count to %d",
             tr_peerIoAddrStr( &peer->addr,
@@ -920,7 +920,7 @@ addStrike( Torrent * t,
 
     if( ++peer->strikes >= MAX_BAD_PIECES_PER_PEER )
     {
-        struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+        struct peer_atom * atom = peer->atom;
         atom->myflags |= MYFLAG_BANNED;
         peer->doPurge = 1;
         tordbg( t, "banning peer %s", tr_peerIoAddrStr( &atom->addr, atom->port ) );
@@ -941,10 +941,8 @@ gotBadPiece( Torrent *        t,
 static void
 refillSoon( Torrent * t )
 {
-    if( t->refillTimer == NULL )
-        t->refillTimer = tr_timerNew( t->manager->session,
-                                      refillPulse, t,
-                                      REFILL_PERIOD_MSEC );
+    if( !evtimer_pending( &t->refillTimer, NULL ) )
+        tr_timerAdd( &t->refillTimer, 0, REFILL_PERIOD_MSEC );
 }
 
 static void
@@ -1008,10 +1006,8 @@ peerCallbackFunc( void * vpeer, void * vevent, void * vt )
     {
         case TR_PEER_UPLOAD_ONLY:
             /* update our atom */
-            if( peer ) {
-                struct peer_atom * a = getExistingAtom( t, &peer->addr );
-                a->uploadOnly = e->uploadOnly ? UPLOAD_ONLY_YES : UPLOAD_ONLY_NO;
-            }
+            if( peer )
+                peer->atom->uploadOnly = e->uploadOnly ? UPLOAD_ONLY_YES : UPLOAD_ONLY_NO;
             break;
 
         case TR_PEER_NEED_REQ:
@@ -1027,7 +1023,7 @@ peerCallbackFunc( void * vpeer, void * vevent, void * vt )
             const time_t now = time( NULL );
             tr_torrent * tor = t->tor;
 
-            tor->activityDate = now;
+            tr_torrentSetActivityDate( tor, now );
 
             if( e->wasPieceData )
                 tor->uploadedCur += e->length;
@@ -1037,11 +1033,10 @@ peerCallbackFunc( void * vpeer, void * vevent, void * vt )
                 tr_statsAddUploaded( tor->session, e->length );
 
             /* update our atom */
-            if( peer ) {
-                struct peer_atom * a = getExistingAtom( t, &peer->addr );
-                if( e->wasPieceData )
-                    a->piece_data_time = now;
-            }
+            if( peer && e->wasPieceData )
+                peer->atom->piece_data_time = now;
+
+            tor->needsSeedRatioCheck = TRUE;
 
             break;
         }
@@ -1060,7 +1055,8 @@ peerCallbackFunc( void * vpeer, void * vevent, void * vt )
         {
             const time_t now = time( NULL );
             tr_torrent * tor = t->tor;
-            tor->activityDate = now;
+
+            tr_torrentSetActivityDate( tor, now );
 
             /* only add this to downloadedCur if we got it from a peer --
              * webseeds shouldn't count against our ratio.  As one tracker
@@ -1076,11 +1072,8 @@ peerCallbackFunc( void * vpeer, void * vevent, void * vt )
                 tr_statsAddDownloaded( tor->session, e->length );
 
             /* update our atom */
-            if( peer ) {
-                struct peer_atom * a = getExistingAtom( t, &peer->addr );
-                if( e->wasPieceData )
-                    a->piece_data_time = now;
-            }
+            if( peer && e->wasPieceData )
+                peer->atom->piece_data_time = now;
 
             break;
         }
@@ -1089,7 +1082,7 @@ peerCallbackFunc( void * vpeer, void * vevent, void * vt )
         {
             if( peer )
             {
-                struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+                struct peer_atom * atom = peer->atom;
                 const int peerIsSeed = e->progress >= 1.0;
                 if( peerIsSeed ) {
                     tordbg( t, "marking peer %s as a seed", tr_peerIoAddrStr( &atom->addr, atom->port ) );
@@ -1223,7 +1216,7 @@ getMaxPeerCount( const tr_torrent * tor )
 static int
 getPeerCount( const Torrent * t )
 {
-    return tr_ptrArraySize( &t->peers );// + tr_ptrArraySize( &t->outgoingHandshakes );
+    return tr_ptrArraySize( &t->peers );/* + tr_ptrArraySize( &t->outgoingHandshakes ); */
 }
 
 /* FIXME: this is kind of a mess. */
@@ -1314,6 +1307,7 @@ myHandshakeDoneCB( tr_handshake  * handshake,
                 }
 
                 peer->port = port;
+                peer->atom = atom;
                 peer->io = tr_handshakeStealIO( handshake ); /* this steals its refcount too, which is
                                                                 balanced by our unref in peerDestructor()  */
                 tr_peerIoSetParent( peer->io, t->tor->bandwidth );
@@ -1455,7 +1449,6 @@ tr_peerMgrArrayToPex( const void * array,
     
     for( i = 0 ; i < n ; i++ ) {
         memcpy( &pex[i].addr, walk, sizeof( tr_address ) );
-        tr_suspectAddress( &pex[i].addr, "tracker"  );
         memcpy( &pex[i].port, walk + sizeof( tr_address ), 2 );
         pex[i].flags = 0x00;
         walk += sizeof( tr_address ) + 2;
@@ -1552,7 +1545,7 @@ tr_peerMgrGetPeers( tr_torrent      * tor,
             const tr_peer * peer = peers[i];
             if( peer->addr.type == af )
             {
-                const struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+                const struct peer_atom * atom = peer->atom;
 
                 assert( tr_isAddress( &peer->addr ) );
                 walk->addr = peer->addr;
@@ -1576,14 +1569,32 @@ tr_peerMgrGetPeers( tr_torrent      * tor,
     return peersReturning;
 }
 
+static void
+ensureMgrTimersExist( struct tr_peerMgr * m )
+{
+    tr_session * s = m->session;
+
+    if( m->bandwidthTimer == NULL )
+        m->bandwidthTimer = tr_timerNew( s, bandwidthPulse, m, BANDWIDTH_PERIOD_MSEC );
+
+    if( m->rechokeTimer == NULL )
+        m->rechokeTimer = tr_timerNew( s, rechokePulse, m, RECHOKE_PERIOD_MSEC );
+
+    if( m->reconnectTimer == NULL )
+        m->reconnectTimer = tr_timerNew( s, reconnectPulse, m, RECONNECT_PERIOD_MSEC );
+
+    if( m->refillUpkeepTimer == NULL )
+        m->refillUpkeepTimer = tr_timerNew( s, refillUpkeep, m, REFILL_UPKEEP_PERIOD_MSEC );
+}
+
 void
 tr_peerMgrStartTorrent( tr_torrent * tor )
 {
     Torrent * t = tor->torrentPeers;
 
+    assert( t != NULL );
     managerLock( t->manager );
-
-    assert( t );
+    ensureMgrTimersExist( t->manager );
 
     if( !t->isRunning )
     {
@@ -1593,6 +1604,7 @@ tr_peerMgrStartTorrent( tr_torrent * tor )
             refillSoon( t );
     }
 
+    rechokePulse( t->manager );
     managerUnlock( t->manager );
 }
 
@@ -1743,7 +1755,7 @@ tr_peerMgrTorrentStats( tr_torrent       * tor,
     for( i=0; i<size; ++i )
     {
         const tr_peer * peer = peers[i];
-        const struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+        const struct peer_atom * atom = peer->atom;
 
         if( peer->io == NULL ) /* not connected */
             continue;
@@ -1771,6 +1783,25 @@ tr_peerMgrTorrentStats( tr_torrent       * tor,
     managerUnlock( t->manager );
 }
 
+float
+tr_peerMgrGetWebseedSpeed( const tr_torrent * tor, uint64_t now )
+{
+    int i;
+    float tmp;
+    float ret = 0;
+
+    const Torrent * t = tor->torrentPeers;
+    const int n = tr_ptrArraySize( &t->webseeds );
+    const tr_webseed ** webseeds = (const tr_webseed**) tr_ptrArrayBase( &t->webseeds );
+
+    for( i=0; i<n; ++i )
+        if( tr_webseedGetSpeed( webseeds[i], now, &tmp ) )
+            ret += tmp;
+
+    return ret;
+}
+
+
 float*
 tr_peerMgrWebSpeeds( const tr_torrent * tor )
 {
@@ -1823,17 +1854,14 @@ tr_peerMgrPeerStats( const tr_torrent    * tor,
     ret = tr_new0( tr_peer_stat, size );
     now = tr_date( );
 
-    for( i = 0; i < size; ++i )
+    for( i=0; i<size; ++i )
     {
         char *                   pch;
         const tr_peer *          peer = peers[i];
-        const struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+        const struct peer_atom * atom = peer->atom;
         tr_peer_stat *           stat = ret + i;
-        tr_address               norm_addr;
 
-        norm_addr = peer->addr;
-        tr_normalizeV4Mapped( &norm_addr );
-        tr_ntop( &norm_addr, stat->addr, sizeof( stat->addr ) );
+        tr_ntop( &peer->addr, stat->addr, sizeof( stat->addr ) );
         tr_strlcpy( stat->client, ( peer->client ? peer->client : "" ),
                    sizeof( stat->client ) );
         stat->port               = ntohs( peer->port );
@@ -1860,6 +1888,7 @@ tr_peerMgrPeerStats( const tr_torrent    * tor,
         if( !stat->clientIsChoked && !stat->clientIsInterested ) *pch++ = 'K';
         if( !stat->peerIsChoked && !stat->peerIsInterested ) *pch++ = '?';
         if( stat->isEncrypted ) *pch++ = 'E';
+        if( stat->from == TR_PEER_FROM_DHT ) *pch++ = 'H';
         if( stat->from == TR_PEER_FROM_PEX ) *pch++ = 'X';
         if( stat->isIncoming ) *pch++ = 'I';
         *pch = '\0';
@@ -1917,7 +1946,7 @@ isSame( const tr_peer * peer )
 **/
 
 static void
-rechokeTorrent( Torrent * t )
+rechokeTorrent( Torrent * t, const uint64_t now )
 {
     int i, size, unchokedInterested;
     const int peerCount = tr_ptrArraySize( &t->peers );
@@ -1925,7 +1954,6 @@ rechokeTorrent( Torrent * t )
     struct ChokeData * choke = tr_new0( struct ChokeData, peerCount );
     const tr_session * session = t->manager->session;
     const int chokeAll = !tr_torrentIsPieceTransferAllowed( t->tor, TR_CLIENT_TO_PEER );
-    const uint64_t now = tr_date( );
 
     assert( torrentIsLocked( t ) );
 
@@ -1933,7 +1961,7 @@ rechokeTorrent( Torrent * t )
     for( i = 0, size = 0; i < peerCount; ++i )
     {
         tr_peer * peer = peers[i];
-        struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+        struct peer_atom * atom = peer->atom;
 
         if( peer->progress >= 1.0 ) /* choke all seeds */
         {
@@ -2019,13 +2047,15 @@ rechokeTorrent( Torrent * t )
 static int
 rechokePulse( void * vmgr )
 {
+    uint64_t now;
     tr_torrent * tor = NULL;
     tr_peerMgr * mgr = vmgr;
     managerLock( mgr );
 
+    now = tr_date( );
     while(( tor = tr_torrentNext( mgr->session, tor )))
         if( tor->isRunning )
-            rechokeTorrent( tor->torrentPeers );
+            rechokeTorrent( tor->torrentPeers, now );
 
     managerUnlock( mgr );
     return TRUE;
@@ -2052,7 +2082,7 @@ shouldPeerBeClosed( const Torrent    * t,
 {
     const tr_torrent *       tor = t->tor;
     const time_t             now = time( NULL );
-    const struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+    const struct peer_atom * atom = peer->atom;
 
     /* if it's marked for purging, close it */
     if( peer->doPurge )
@@ -2128,8 +2158,7 @@ getPeersToClose( Torrent * t, tr_close_type_t closeType, int * setmeSize )
 }
 
 static int
-compareCandidates( const void * va,
-                   const void * vb )
+compareCandidates( const void * va, const void * vb )
 {
     const struct peer_atom * a = *(const struct peer_atom**) va;
     const struct peer_atom * b = *(const struct peer_atom**) vb;
@@ -2150,8 +2179,8 @@ compareCandidates( const void * va,
     if( a->time != b->time )
         return a->time < b->time ? -1 : 1;
 
-    /* all other things being equal, prefer peers whose
-     * information comes from a more reliable source */
+    /* In order to avoid fragmenting the swarm, peers from trackers and
+     * from the DHT should be preferred to peers from PEX. */
     if( a->from != b->from )
         return a->from < b->from ? -1 : 1;
 
@@ -2256,10 +2285,11 @@ closePeer( Torrent * t, tr_peer * peer )
     assert( t != NULL );
     assert( peer != NULL );
 
+    atom = peer->atom;
+
     /* if we transferred piece data, then they might be good peers,
        so reset their `numFails' weight to zero.  otherwise we connected
        to them fruitlessly, so mark it as another fail */
-    atom = getExistingAtom( t, &peer->addr );
     if( atom->piece_data_time )
         atom->numFails = 0;
     else
@@ -2344,7 +2374,7 @@ reconnectTorrent( Torrent * t )
             tordbg( t, "Starting an OUTGOING connection with %s",
                    tr_peerIoAddrStr( &atom->addr, atom->port ) );
 
-            io = tr_peerIoNewOutgoing( mgr->session, mgr->session->bandwidth, &atom->addr, atom->port, t->hash );
+            io = tr_peerIoNewOutgoing( mgr->session, mgr->session->bandwidth, &atom->addr, atom->port, t->tor->info.hash );
 
             if( io == NULL )
             {
@@ -2379,13 +2409,156 @@ reconnectTorrent( Torrent * t )
     }
 }
 
+struct peer_liveliness
+{
+    tr_peer * peer;
+    void * clientData;
+    time_t pieceDataTime;
+    time_t time;
+    int speed;
+    tr_bool doPurge;
+};
+
+static int
+comparePeerLiveliness( const void * va, const void * vb )
+{
+    const struct peer_liveliness * a = va;
+    const struct peer_liveliness * b = vb;
+
+    if( a->doPurge != b->doPurge )
+        return a->doPurge ? 1 : -1;
+
+    if( a->speed != b->speed ) /* faster goes first */
+        return a->speed > b->speed ? -1 : 1;
+
+    /* the one to give us data more recently goes first */
+    if( a->pieceDataTime != b->pieceDataTime )
+        return a->pieceDataTime > b->pieceDataTime ? -1 : 1;
+
+    /* the one we connected to most recently goes first */
+    if( a->time != b->time )
+        return a->time > b->time ? -1 : 1;
+
+    return 0;
+}
+
+/* FIXME: getPeersToClose() should use this */
+static void
+sortPeersByLiveliness( tr_peer ** peers, void** clientData, int n, uint64_t now )
+{
+    int i;
+    struct peer_liveliness *lives, *l;
+
+    /* build a sortable array of peer + extra info */
+    lives = l = tr_new0( struct peer_liveliness, n );
+    for( i=0; i<n; ++i, ++l ) {
+        tr_peer * p = peers[i];
+        l->peer = p;
+        l->doPurge = p->doPurge;
+        l->pieceDataTime = p->atom->piece_data_time;
+        l->time = p->atom->time;
+        l->speed = 1024.0 * (   tr_peerGetPieceSpeed( p, now, TR_UP )
+                              + tr_peerGetPieceSpeed( p, now, TR_DOWN ) );
+        if( clientData )
+            l->clientData = clientData[i];
+    }
+
+    /* sort 'em */
+    assert( n == ( l - lives ) );
+    qsort( lives, n, sizeof( struct peer_liveliness ), comparePeerLiveliness );
+
+    /* build the peer array */
+    for( i=0, l=lives; i<n; ++i, ++l ) {
+        peers[i] = l->peer;
+        if( clientData )
+            clientData[i] = l->clientData;
+    }
+    assert( n == ( l - lives ) );
+
+    /* cleanup */
+    tr_free( lives );
+}
+
+static void
+enforceTorrentPeerLimit( Torrent * t, uint64_t now )
+{
+    int n = tr_ptrArraySize( &t->peers );
+    const int max = tr_sessionGetPeerLimitPerTorrent( t->tor->session );
+    if( n > max )
+    {
+        void * base = tr_ptrArrayBase( &t->peers );
+        tr_peer ** peers = tr_memdup( base, n*sizeof( tr_peer* ) );
+        sortPeersByLiveliness( peers, NULL, n, now );
+        while( n > max )
+            closePeer( t, peers[--n] );
+        tr_free( peers );
+    }
+}
+
+static void
+enforceSessionPeerLimit( tr_session * session, uint64_t now )
+{
+    int n = 0;
+    tr_torrent * tor = NULL;
+    const int max = tr_sessionGetPeerLimit( session );
+
+    /* count the total number of peers */
+    while(( tor = tr_torrentNext( session, tor )))
+        n += tr_ptrArraySize( &tor->torrentPeers->peers );
+
+    /* if there are too many, prune out the worst */ 
+    if( n > max )
+    {
+        tr_peer ** peers = tr_new( tr_peer*, n );
+        Torrent ** torrents = tr_new( Torrent*, n );
+
+        /* populate the peer array */
+        n = 0;
+        tor = NULL;
+        while(( tor = tr_torrentNext( session, tor ))) {
+            int i;
+            Torrent * t = tor->torrentPeers;
+            const int tn = tr_ptrArraySize( &t->peers );
+            for( i=0; i<tn; ++i, ++n ) {
+                peers[n] = tr_ptrArrayNth( &t->peers, i );
+                torrents[n] = t;
+            }
+        }
+        
+        /* sort 'em */
+        sortPeersByLiveliness( peers, (void**)torrents, n, now );
+
+        /* cull out the crappiest */
+        while( n-- > max )
+            closePeer( torrents[n], peers[n] );
+
+        /* cleanup */
+        tr_free( torrents );
+        tr_free( peers );
+    }
+}
+
+
 static int
 reconnectPulse( void * vmgr )
 {
-    tr_torrent * tor = NULL;
+    tr_torrent * tor;
     tr_peerMgr * mgr = vmgr;
+    uint64_t now;
     managerLock( mgr );
 
+    now = tr_date( );
+
+    /* if we're over the per-torrent peer limits, cull some peers */
+    tor = NULL;
+    while(( tor = tr_torrentNext( mgr->session, tor )))
+        if( tor->isRunning )
+            enforceTorrentPeerLimit( tor->torrentPeers, now );
+
+    /* if we're over the per-session peer limits, cull some peers */
+    enforceSessionPeerLimit( mgr->session, now );
+
+    tor = NULL;
     while(( tor = tr_torrentNext( mgr->session, tor )))
         if( tor->isRunning )
             reconnectTorrent( tor->torrentPeers );
@@ -2421,6 +2594,7 @@ pumpAllPeers( tr_peerMgr * mgr )
 static int
 bandwidthPulse( void * vmgr )
 {
+    tr_torrent * tor = NULL;
     tr_peerMgr * mgr = vmgr;
     managerLock( mgr );
 
@@ -2431,6 +2605,14 @@ bandwidthPulse( void * vmgr )
     tr_bandwidthAllocate( mgr->session->bandwidth, TR_UP, BANDWIDTH_PERIOD_MSEC );
     tr_bandwidthAllocate( mgr->session->bandwidth, TR_DOWN, BANDWIDTH_PERIOD_MSEC );
 
+    /* possibly stop torrents that have seeded enough */
+    while(( tor = tr_torrentNext( mgr->session, tor ))) {
+        if( tor->needsSeedRatioCheck ) {
+            tor->needsSeedRatioCheck = FALSE;
+            tr_torrentCheckSeedRatio( tor );
+        }
+    }
+
     managerUnlock( mgr );
     return TRUE;
 }
