@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-msgs.c 7771 2009-01-22 15:00:29Z charles $
+ * $Id: peer-msgs.c 8767 2009-07-01 14:58:57Z charles $
  */
 
 #include <assert.h>
@@ -20,7 +20,6 @@
 #include <event.h>
 
 #include "transmission.h"
-#include "session.h"
 #include "bencode.h"
 #include "completion.h"
 #include "crypto.h"
@@ -34,10 +33,12 @@
 #include "platform.h" /* MAX_STACK_ARRAY_SIZE */
 #include "ratecontrol.h"
 #include "request-list.h"
+#include "session.h"
 #include "stats.h"
 #include "torrent.h"
-#include "trevent.h"
+#include "tr-dht.h"
 #include "utils.h"
+#include "version.h"
 
 /**
 ***
@@ -75,7 +76,7 @@ enum
     /* idle seconds before we send a keepalive */
     KEEPALIVE_INTERVAL_SECS = 100,
 
-    PEX_INTERVAL            = ( 90 * 1000 ), /* msec between sendPex() calls */
+    PEX_INTERVAL_SECS       = 90, /* sec between sendPex() calls */
 
 
     MAX_BLOCK_SIZE          = ( 1024 * 16 ),
@@ -143,7 +144,12 @@ struct tr_peermsgs
     tr_bool         peerSupportsPex;
     tr_bool         clientSentLtepHandshake;
     tr_bool         peerSentLtepHandshake;
-    tr_bool         haveFastSet;
+    /*tr_bool         haveFastSet;*/
+
+    /* how long the outMessages batch should be allowed to grow before
+     * it's flushed -- some messages (like requests >:) should be sent
+     * very quickly; others aren't as urgent. */
+    int8_t          outMessagesBatchPeriod;
 
     uint8_t         state;
     uint8_t         ut_pex_id;
@@ -151,17 +157,13 @@ struct tr_peermsgs
     uint16_t        pexCount6;
     uint16_t        maxActiveRequests;
 
+#if 0
     size_t                 fastsetSize;
     tr_piece_index_t       fastset[MAX_FAST_SET_SIZE];
-
-    /* how long the outMessages batch should be allowed to grow before
-     * it's flushed -- some messages (like requests >:) should be sent
-     * very quickly; others aren't as urgent. */
-    int                    outMessagesBatchPeriod;
+#endif
 
     tr_peer *              peer;
 
-    tr_session *           session;
     tr_torrent *           torrent;
 
     tr_publisher           publisher;
@@ -172,11 +174,10 @@ struct tr_peermsgs
     struct request_list    clientAskedFor;
     struct request_list    clientWillAskFor;
 
-    tr_timer             * pexTimer;
     tr_pex               * pex;
     tr_pex               * pex6;
 
-    time_t                 clientSentPexAt;
+    /*time_t                 clientSentPexAt;*/
     time_t                 clientSentAnythingAt;
 
     /* when we started batching the outMessages */
@@ -188,8 +189,16 @@ struct tr_peermsgs
        supplied a reqq argument, it's stored here.  otherwise the
        value is zero and should be ignored. */
     int64_t               reqq;
+
+    struct event          pexTimer;
 };
 
+static TR_INLINE tr_session*
+getSession( struct tr_peermsgs * msgs )
+{
+    return msgs->torrent->session;
+}
+
 /**
 ***
 **/
@@ -205,7 +214,7 @@ myDebug( const char * file, int line,
     {
         va_list           args;
         char              timestr[64];
-        struct evbuffer * buf = tr_getBuffer( );
+        struct evbuffer * buf = evbuffer_new( );
         char *            base = tr_basename( file );
 
         evbuffer_add_printf( buf, "[%s] %s - %s [%s]: ",
@@ -217,10 +226,11 @@ myDebug( const char * file, int line,
         evbuffer_add_vprintf( buf, fmt, args );
         va_end( args );
         evbuffer_add_printf( buf, " (%s:%d)\n", base, line );
+        /* FIXME(libevent2) tr_getLog() should return an fd, then use evbuffer_write() here */
         fwrite( EVBUFFER_DATA( buf ), 1, EVBUFFER_LENGTH( buf ), fp );
 
         tr_free( base );
-        tr_releaseBuffer( buf );
+        evbuffer_free( buf );
     }
 }
 
@@ -305,6 +315,18 @@ protocolSendCancel( tr_peermsgs               * msgs,
     pokeBatchPeriod( msgs, IMMEDIATE_PRIORITY_INTERVAL_SECS );
 }
 
+static void
+protocolSendPort(tr_peermsgs *msgs, uint16_t port)
+{
+    tr_peerIo       * io  = msgs->peer->io;
+    struct evbuffer * out = msgs->outMessages;
+
+    dbgmsg( msgs, "sending Port %u", port);
+    tr_peerIoWriteUint32( io, out, 3 );
+    tr_peerIoWriteUint8 ( io, out, BT_PORT );
+    tr_peerIoWriteUint16( io, out, port);
+}
+
 static void
 protocolSendHave( tr_peermsgs * msgs,
                   uint32_t      index )
@@ -526,12 +548,16 @@ tr_generateAllowedSet( tr_piece_index_t * setmePieces,
 
     if( addr->type == TR_AF_INET )
     {
-        uint8_t w[SHA_DIGEST_LENGTH + 4];
+        uint8_t w[SHA_DIGEST_LENGTH + 4], *walk=w;
         uint8_t x[SHA_DIGEST_LENGTH];
 
-        *(uint32_t*)w = ntohl( htonl( addr->addr.addr4.s_addr ) & 0xffffff00 );   /* (1) */
-        memcpy( w + 4, infohash, SHA_DIGEST_LENGTH );                /* (2) */
-        tr_sha1( x, w, sizeof( w ), NULL );                          /* (3) */
+        uint32_t ui32 = ntohl( htonl( addr->addr.addr4.s_addr ) & 0xffffff00 );   /* (1) */
+        memcpy( w, &ui32, sizeof( uint32_t ) );
+        walk += sizeof( uint32_t );
+        memcpy( walk, infohash, SHA_DIGEST_LENGTH );                 /* (2) */
+        walk += SHA_DIGEST_LENGTH;
+        tr_sha1( x, w, walk-w, NULL );                               /* (3) */
+        assert( sizeof( w ) == walk-w );
 
         while( setSize<desiredSetSize )
         {
@@ -587,7 +613,7 @@ updateFastSet( tr_peermsgs * msgs UNUSED )
 ***  INTEREST
 **/
 
-static int
+static tr_bool
 isPieceInteresting( const tr_peermsgs * msgs,
                     tr_piece_index_t    piece )
 {
@@ -599,7 +625,7 @@ isPieceInteresting( const tr_peermsgs * msgs,
 }
 
 /* "interested" means we'll ask for piece data if they unchoke us */
-static int
+static tr_bool
 isPeerInteresting( const tr_peermsgs * msgs )
 {
     tr_piece_index_t    i;
@@ -657,7 +683,7 @@ updateInterest( tr_peermsgs * msgs )
         fireNeedReq( msgs );
 }
 
-static int
+static tr_bool
 popNextRequest( tr_peermsgs *         msgs,
                 struct peer_request * setme )
 {
@@ -718,7 +744,7 @@ tr_peerMsgsHave( tr_peermsgs * msgs,
 ***
 **/
 
-static int
+static tr_bool
 reqIsValid( const tr_peermsgs * peer,
             uint32_t            index,
             uint32_t            offset,
@@ -727,7 +753,7 @@ reqIsValid( const tr_peermsgs * peer,
     return tr_torrentReqIsValid( peer->torrent, index, offset, length );
 }
 
-static int
+static tr_bool
 requestIsValid( const tr_peermsgs * msgs, const struct peer_request * req )
 {
     return reqIsValid( msgs, req->index, req->offset, req->length );
@@ -824,7 +850,7 @@ pumpRequestQueue( tr_peermsgs * msgs, const time_t now )
         fireNeedReq( msgs );
 }
 
-static TR_INLINE int
+static TR_INLINE tr_bool
 requestQueueIsFull( const tr_peermsgs * msgs )
 {
     const int req_max = msgs->maxActiveRequests;
@@ -841,7 +867,6 @@ tr_peerMsgsAddRequest( tr_peermsgs *    msgs,
 
     assert( msgs );
     assert( msgs->torrent );
-    assert( reqIsValid( msgs, index, offset, length ) );
 
     /**
     ***  Reasons to decline the request
@@ -853,16 +878,16 @@ tr_peerMsgsAddRequest( tr_peermsgs *    msgs,
         return TR_ADDREQ_CLIENT_CHOKED;
     }
 
-    /* peer doesn't have this piece */
-    if( !tr_bitfieldHas( msgs->peer->have, index ) )
-        return TR_ADDREQ_MISSING;
-
     /* peer's queue is full */
     if( requestQueueIsFull( msgs ) ) {
         dbgmsg( msgs, "declining request because we're full" );
         return TR_ADDREQ_FULL;
     }
 
+    /* peer doesn't have this piece */
+    if( !tr_bitfieldHas( msgs->peer->have, index ) )
+        return TR_ADDREQ_MISSING;
+
     /* have we already asked for this piece? */
     req.index = index;
     req.offset = offset;
@@ -971,14 +996,14 @@ sendLtepHandshake( tr_peermsgs * msgs )
         pex = 1;
 
     tr_bencInitDict( &val, 5 );
-    tr_bencDictAddInt( &val, "e", msgs->session->encryptionMode != TR_CLEAR_PREFERRED );
-    tr_bencDictAddInt( &val, "p", tr_sessionGetPeerPort( msgs->session ) );
+    tr_bencDictAddInt( &val, "e", getSession(msgs)->encryptionMode != TR_CLEAR_PREFERRED );
+    tr_bencDictAddInt( &val, "p", tr_sessionGetPeerPort( getSession(msgs) ) );
     tr_bencDictAddInt( &val, "upload_only", tr_torrentIsSeed( msgs->torrent ) );
     tr_bencDictAddStr( &val, "v", TR_NAME " " USERAGENT_PREFIX );
     m  = tr_bencDictAddDict( &val, "m", 1 );
     if( pex )
         tr_bencDictAddInt( m, "ut_pex", TR_LTEP_PEX );
-    buf = tr_bencSave( &val, &len );
+    buf = tr_bencToStr( &val, TR_FMT_BENC, &len );
 
     tr_peerIoWriteUint32( msgs->peer->io, out, 2 * sizeof( uint8_t ) + len );
     tr_peerIoWriteUint8 ( msgs->peer->io, out, BT_LTEP );
@@ -1223,7 +1248,7 @@ peerMadeRequest( tr_peermsgs *               msgs,
         protocolSendReject( msgs, req );
 }
 
-static int
+static tr_bool
 messageLengthIsCorrect( const tr_peermsgs * msg, uint8_t id, uint32_t len )
 {
     switch( id )
@@ -1319,7 +1344,8 @@ readBtPiece( tr_peermsgs      * msgs,
         err = clientGotBlock( msgs, EVBUFFER_DATA( msgs->incoming.block ), req );
 
         /* cleanup */
-        evbuffer_drain( msgs->incoming.block, EVBUFFER_LENGTH( msgs->incoming.block ) );
+        evbuffer_free( msgs->incoming.block );
+        msgs->incoming.block = evbuffer_new( );
         req->length = 0;
         msgs->state = AWAITING_BT_LENGTH;
         if( !err )
@@ -1429,7 +1455,9 @@ readBtMessage( tr_peermsgs * msgs, struct evbuffer * inbuf, size_t inlen )
 
         case BT_PORT:
             dbgmsg( msgs, "Got a BT_PORT" );
-            tr_peerIoReadUint16( msgs->peer->io, inbuf, &msgs->peer->port );
+            tr_peerIoReadUint16( msgs->peer->io, inbuf, &msgs->peer->dht_port );
+            if( msgs->peer->dht_port > 0 )
+                tr_dhtAddNode( getSession(msgs), &msgs->peer->addr, msgs->peer->dht_port, 0 );
             break;
 
         case BT_FEXT_SUGGEST:
@@ -1596,7 +1624,9 @@ didWrite( tr_peerIo * io UNUSED, size_t bytesWritten, int wasPieceData, void * v
 {
     tr_peermsgs * msgs = vmsgs;
     firePeerGotData( msgs, bytesWritten, wasPieceData );
-    peerPulse( msgs );
+
+    if ( tr_isPeerIo( io ) && io->userData )
+        peerPulse( msgs );
 }
 
 static ReadState
@@ -1701,27 +1731,43 @@ fillOutputBuffer( tr_peermsgs * msgs, time_t now )
         if( requestIsValid( msgs, &req )
             && tr_cpPieceIsComplete( &msgs->torrent->completion, req.index ) )
         {
+            /* FIXME(libevent2) use evbuffer_reserve_space() + evbuffer_commit_space() */
             int err;
-            static uint8_t buf[MAX_BLOCK_SIZE];
+            const uint32_t msglen = 4 + 1 + 4 + 4 + req.length;
+            struct evbuffer * out;
+            tr_peerIo * io = msgs->peer->io;
 
-            /* send a block */
-            if(( err = tr_ioRead( msgs->torrent, req.index, req.offset, req.length, buf ))) {
-                fireError( msgs, err );
-                bytesWritten = 0;
-                msgs = NULL;
-            } else {
-                tr_peerIo * io = msgs->peer->io;
-                struct evbuffer * out = tr_getBuffer( );
-                dbgmsg( msgs, "sending block %u:%u->%u", req.index, req.offset, req.length );
-                tr_peerIoWriteUint32( io, out, sizeof( uint8_t ) + 2 * sizeof( uint32_t ) + req.length );
-                tr_peerIoWriteUint8 ( io, out, BT_PIECE );
-                tr_peerIoWriteUint32( io, out, req.index );
-                tr_peerIoWriteUint32( io, out, req.offset );
-                tr_peerIoWriteBytes ( io, out, buf, req.length );
+            out = evbuffer_new( );
+            evbuffer_expand( out, msglen );
+
+            tr_peerIoWriteUint32( io, out, sizeof( uint8_t ) + 2 * sizeof( uint32_t ) + req.length );
+            tr_peerIoWriteUint8 ( io, out, BT_PIECE );
+            tr_peerIoWriteUint32( io, out, req.index );
+            tr_peerIoWriteUint32( io, out, req.offset );
+
+            err = tr_ioRead( msgs->torrent, req.index, req.offset, req.length, EVBUFFER_DATA(out)+EVBUFFER_LENGTH(out) );
+            if( err )
+            {
+                if( fext )
+                    protocolSendReject( msgs, &req );
+            }
+            else
+            {
+                dbgmsg( msgs, "sending block %u:%u->%u", req.index, req.offset, req.length ); 
+                EVBUFFER_LENGTH(out) += req.length;
+                assert( EVBUFFER_LENGTH( out ) == msglen );
                 tr_peerIoWriteBuf( io, out, TRUE );
                 bytesWritten += EVBUFFER_LENGTH( out );
                 msgs->clientSentAnythingAt = now;
-                tr_releaseBuffer( out );
+            }
+
+            evbuffer_free( out );
+
+            if( err )
+            {
+                fireError( msgs, err );
+                bytesWritten = 0;
+                msgs = NULL;
             }
         }
         else if( fext ) /* peer needs a reject message */
@@ -1795,7 +1841,7 @@ sendBitfield( tr_peermsgs * msgs )
 
     field = tr_bitfieldDup( tr_cpPieceBitfield( &msgs->torrent->completion ) );
 
-    if( tr_sessionIsLazyBitfieldEnabled( msgs->session ) )
+    if( tr_sessionIsLazyBitfieldEnabled( getSession( msgs ) ) )
     {
         /** Lazy bitfields aren't a high priority or secure, so I'm opting for
             speed over a truly random sample -- let's limit the pool size to
@@ -1826,6 +1872,7 @@ sendBitfield( tr_peermsgs * msgs )
     tr_peerIoWriteUint32( msgs->peer->io, out,
                           sizeof( uint8_t ) + field->byteCount );
     tr_peerIoWriteUint8 ( msgs->peer->io, out, BT_BITFIELD );
+    /* FIXME(libevent2): use evbuffer_add_reference() */
     tr_peerIoWriteBytes ( msgs->peer->io, out, field->bits, field->byteCount );
     dbgmsg( msgs, "sending bitfield... outMessage size is now %zu",
             EVBUFFER_LENGTH( out ) );
@@ -1986,7 +2033,6 @@ sendPex( tr_peermsgs * msgs )
             tmp = walk = tr_new( uint8_t, diffs.addedCount * 6 );
             for( i = 0; i < diffs.addedCount; ++i )
             {
-                tr_suspectAddress( &diffs.added[i].addr, "pex" );
                 memcpy( walk, &diffs.added[i].addr.addr, 4 ); walk += 4;
                 memcpy( walk, &diffs.added[i].port, 2 ); walk += 2;
             }
@@ -2017,7 +2063,6 @@ sendPex( tr_peermsgs * msgs )
             tmp = walk = tr_new( uint8_t, diffs6.addedCount * 18 );
             for( i = 0; i < diffs6.addedCount; ++i )
             {
-                tr_suspectAddress( &diffs6.added[i].addr, "pex6" );
                 memcpy( walk, &diffs6.added[i].addr.addr.addr6.s6_addr, 16 );
                 walk += 16;
                 memcpy( walk, &diffs6.added[i].port, 2 );
@@ -2049,7 +2094,7 @@ sendPex( tr_peermsgs * msgs )
             tr_free( tmp );
 
             /* write the pex message */
-            benc = tr_bencSave( &val, &bencLen );
+            benc = tr_bencToStr( &val, TR_FMT_BENC, &bencLen );
             tr_peerIoWriteUint32( io, out, 2 * sizeof( uint8_t ) + bencLen );
             tr_peerIoWriteUint8 ( io, out, BT_LTEP );
             tr_peerIoWriteUint8 ( io, out, msgs->ut_pex_id );
@@ -2070,15 +2115,18 @@ sendPex( tr_peermsgs * msgs )
         tr_free( diffs6.dropped );
         tr_free( newPex6 );
 
-        msgs->clientSentPexAt = time( NULL );
+        /*msgs->clientSentPexAt = time( NULL );*/
     }
 }
 
-static TR_INLINE int
-pexPulse( void * vpeer )
+static void
+pexPulse( int foo UNUSED, short bar UNUSED, void * vmsgs )
 {
-    sendPex( vpeer );
-    return TRUE;
+    struct tr_peermsgs * msgs = vmsgs;
+
+    sendPex( msgs );
+
+    tr_timerAdd( &msgs->pexTimer, PEX_INTERVAL_SECS, 0 );
 }
 
 /**
@@ -2100,7 +2148,6 @@ tr_peerMsgsNew( struct tr_torrent * torrent,
     m = tr_new0( tr_peermsgs, 1 );
     m->publisher = TR_PUBLISHER_INIT;
     m->peer = peer;
-    m->session = torrent->session;
     m->torrent = torrent;
     m->peer->clientIsChoked = 1;
     m->peer->peerIsChoked = 1;
@@ -2108,7 +2155,6 @@ tr_peerMsgsNew( struct tr_torrent * torrent,
     m->peer->peerIsInterested = 0;
     m->peer->have = tr_bitfieldNew( torrent->info.pieceCount );
     m->state = AWAITING_BT_LENGTH;
-    m->pexTimer = tr_timerNew( m->session, pexPulse, m, PEX_INTERVAL );
     m->outMessages = evbuffer_new( );
     m->outMessagesBatchedAt = 0;
     m->outMessagesBatchPeriod = LOW_PRIORITY_INTERVAL_SECS;
@@ -2116,6 +2162,8 @@ tr_peerMsgsNew( struct tr_torrent * torrent,
     m->peerAskedFor = REQUEST_LIST_INIT;
     m->clientAskedFor = REQUEST_LIST_INIT;
     m->clientWillAskFor = REQUEST_LIST_INIT;
+    evtimer_set( &m->pexTimer, pexPulse, m );
+    tr_timerAdd( &m->pexTimer, PEX_INTERVAL_SECS, 0 );
     peer->msgs = m;
 
     *setme = tr_publisherSubscribe( &m->publisher, func, userData );
@@ -2123,6 +2171,9 @@ tr_peerMsgsNew( struct tr_torrent * torrent,
     if( tr_peerIoSupportsLTEP( peer->io ) )
         sendLtepHandshake( m );
 
+    if(tr_peerIoSupportsDHT(peer->io))
+        protocolSendPort(m, tr_dhtPort(torrent->session));
+
     tellPeerWhatWeHave( m );
 
     tr_peerIoSetIOFuncs( m->peer->io, canRead, didWrite, gotError, m );
@@ -2136,7 +2187,7 @@ tr_peerMsgsFree( tr_peermsgs* msgs )
 {
     if( msgs )
     {
-        tr_timerFree( &msgs->pexTimer );
+        evtimer_del( &msgs->pexTimer );
         tr_publisherDestruct( &msgs->publisher );
         reqListClear( &msgs->clientWillAskFor );
         reqListClear( &msgs->clientAskedFor );
