@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: platform.c 7664 2009-01-11 17:46:51Z charles $
+ * $Id: platform.c 8792 2009-07-09 18:18:14Z titer $
  */
 
 #ifdef WIN32
@@ -17,8 +17,10 @@
  #ifdef SYS_DARWIN
   #include <CoreFoundation/CoreFoundation.h>
  #endif
-
- #define _XOPEN_SOURCE 500  /* needed for recursive locks. */
+ #ifdef __HAIKU__
+  #include <FindDirectory.h>
+ #endif
+ #define _XOPEN_SOURCE 600  /* needed for recursive locks. */
  #ifndef __USE_UNIX98
   #define __USE_UNIX98 /* some older Linuxes need it spelt out for them */
  #endif
@@ -271,6 +273,10 @@ getOldConfigDir( void )
         char appdata[MAX_PATH]; /* SHGetFolderPath() requires MAX_PATH */
         SHGetFolderPath( NULL, CSIDL_APPDATA, NULL, 0, appdata );
         path = tr_buildPath( appdata, "Transmission", NULL );
+#elif defined( __HAIKU__ )
+        char buf[MAX_PATH_LENGTH];
+        find_directory( B_USER_SETTINGS_DIRECTORY, -1, true, buf, sizeof(buf) );
+        path = tr_buildPath( buf, "Transmission", NULL );
 #else
         path = tr_buildPath( getHomeDir( ), ".transmission", NULL );
 #endif
@@ -431,6 +437,10 @@ tr_getDefaultConfigDir( const char * appname )
             char appdata[MAX_PATH]; /* SHGetFolderPath() requires MAX_PATH */
             SHGetFolderPath( NULL, CSIDL_APPDATA, NULL, 0, appdata );
             s = tr_buildPath( appdata, appname, NULL );
+#elif defined( __HAIKU__ )
+            char buf[MAX_PATH_LENGTH];
+            find_directory( B_USER_SETTINGS_DIRECTORY, -1, true, buf, sizeof(buf) );
+            s = tr_buildPath( buf, appname, NULL );
 #else
             if( ( s = getenv( "XDG_CONFIG_HOME" ) ) )
                 s = tr_buildPath( s, appname, NULL );
@@ -446,12 +456,57 @@ tr_getDefaultConfigDir( const char * appname )
 const char*
 tr_getDefaultDownloadDir( void )
 {
-    static char * s = NULL;
+    static char * user_dir = NULL;
 
-    if( s == NULL )
-        s = tr_buildPath( getHomeDir( ), "Downloads", NULL );
+    if( user_dir == NULL )
+    {
+        const char * config_home;
+        char * config_file;
+        char * content;
+        size_t content_len;
 
-    return s;
+        /* figure out where to look for user-dirs.dirs */
+        config_home = getenv( "XDG_CONFIG_HOME" );
+        if( config_home && *config_home )
+            config_file = tr_buildPath( config_home, "user-dirs.dirs", NULL );
+        else
+            config_file = tr_buildPath( getHomeDir( ), ".config", "user-dirs.dirs", NULL );
+
+        /* read in user-dirs.dirs and look for the download dir entry */
+        content = (char *) tr_loadFile( config_file, &content_len );
+        if( content && content_len>0 )
+        {
+            const char * key = "XDG_DOWNLOAD_DIR=\"";
+            char * line = strstr( content, key );
+            if( line != NULL )
+            {
+                char * value = line + strlen( key );
+                char * end = strchr( value, '"' );
+
+                if( end )
+                {
+                    *end = '\0';
+
+                    if( !memcmp( value, "$HOME/", 6 ) )
+                        user_dir = tr_buildPath( getHomeDir( ), value+6, NULL );
+                    else
+                        user_dir = tr_strdup( value );
+                }
+            }
+        }
+
+        if( user_dir == NULL )
+#ifdef __HAIKU__
+            user_dir = tr_buildPath( getHomeDir( ), "Desktop", NULL );
+#else
+            user_dir = tr_buildPath( getHomeDir( ), "Downloads", NULL );
+#endif
+
+        tr_free( content );
+        tr_free( config_file );
+    }
+
+    return user_dir;
 }
 
 /***
@@ -488,7 +543,7 @@ tr_getClutchDir( const tr_session * session UNUSED )
         else
         {
 
-#ifdef SYS_DARWIN
+#ifdef SYS_DARWIN /* on Mac, look in the app package first, then the Application Support folder (for daemon, etc) */
 
             CFURLRef appURL = CFBundleCopyBundleURL( CFBundleGetMainBundle( ) );
             CFStringRef appRef = CFURLCopyFileSystemPath( appURL,
@@ -499,6 +554,17 @@ tr_getClutchDir( const tr_session * session UNUSED )
             CFRelease( appRef );
 
             s = tr_buildPath( appString, "Contents", "Resources", "web", NULL );
+            
+            if( !isClutchDir( s ) ) {
+                tr_free( s );
+                
+                /* Fallback to the Application Support folder */
+                s = tr_buildPath( tr_sessionGetConfigDir( session ), "web", NULL );
+                if( !isClutchDir( s ) ) {
+                    tr_free( s );
+                    s = NULL;
+                }
+            }
 
 #elif defined( WIN32 )
 
