@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: tracker.c 7897 2009-02-18 02:11:15Z charles $
+ * $Id: tracker.c 8767 2009-07-01 14:58:57Z charles $
  */
 
 #include <assert.h>
@@ -17,15 +17,16 @@
 #include <event.h>
 
 #include "transmission.h"
-#include "session.h"
 #include "bencode.h"
-#include "crypto.h"
 #include "completion.h"
+#include "crypto.h"
 #include "net.h"
 #include "publish.h"
 #include "resume.h"
+#include "session.h"
 #include "torrent.h"
 #include "tracker.h"
+#include "tr-dht.h"
 #include "trevent.h"
 #include "utils.h"
 #include "web.h"
@@ -41,7 +42,7 @@ enum
     HTTP_OK = 200,
 
     /* seconds between tracker pulses */
-    PULSE_INTERVAL_MSEC = 1000,
+    PULSE_INTERVAL_MSEC = 1500,
 
     /* unless the tracker says otherwise, rescrape this frequently */
     DEFAULT_SCRAPE_INTERVAL_SEC = ( 60 * 15 ),
@@ -226,7 +227,7 @@ publishWarning( tr_tracker * t,
 static void
 publishNewPeers( tr_tracker * t,
                  int          allAreSeeds,
-                 void *       compact,
+                 const void * compact,
                  int          compactLen )
 {
     tr_tracker_event event = emptyEvent;
@@ -242,11 +243,12 @@ publishNewPeers( tr_tracker * t,
 static void
 publishNewPeersCompact( tr_tracker * t,
                         int          allAreSeeds,
-                        void       * compact,
+                        const void * compact,
                         int          compactLen )
 {
     int i;
-    uint8_t *array, *walk, *compactWalk;
+    const uint8_t *compactWalk;
+    uint8_t *array, *walk;
     const int peerCount = compactLen / 6;
     const int arrayLen = peerCount * ( sizeof( tr_address ) + 2 );
     tr_address addr;
@@ -259,7 +261,6 @@ publishNewPeersCompact( tr_tracker * t,
     {
         memcpy( &addr.addr.addr4, compactWalk, 4 );
         memcpy( &port, compactWalk + 4, 2 );
-        tr_suspectAddress( &addr, "compact" );
         
         memcpy( walk, &addr, sizeof( addr ) );
         memcpy( walk + sizeof( addr ), &port, 2 );
@@ -274,11 +275,12 @@ publishNewPeersCompact( tr_tracker * t,
 static void
 publishNewPeersCompact6( tr_tracker * t,
                          int          allAreSeeds,
-                         void       * compact,
+                         const void * compact,
                          int          compactLen )
 {
     int i;
-    uint8_t *array, *walk, *compactWalk;
+    const uint8_t *compactWalk;
+    uint8_t *array, *walk;
     const int peerCount = compactLen / 18;
     const int arrayLen = peerCount * ( sizeof( tr_address ) + 2 );
     tr_address addr;
@@ -292,7 +294,6 @@ publishNewPeersCompact6( tr_tracker * t,
         memcpy( &addr.addr.addr6, compactWalk, 16 );
         memcpy( &port, compactWalk + 16, 2 );
         compactWalk += 18;
-        tr_suspectAddress( &addr, "compact6" );
         
         memcpy( walk, &addr, sizeof( addr ) );
         memcpy( walk + sizeof( addr ), &port, 2 );
@@ -346,7 +347,7 @@ parseOldPeers( tr_benc * bePeers,
     uint8_t * array, *walk;
     const int peerCount = bePeers->val.l.count;
 
-    assert( bePeers->type == TYPE_LIST );
+    assert( tr_bencIsList( bePeers ) );
 
     array = tr_new( uint8_t, peerCount * ( sizeof( tr_address ) + 2 ) );
 
@@ -368,7 +369,6 @@ parseOldPeers( tr_benc * bePeers,
             continue;
 
         memcpy( walk, &addr, sizeof( tr_address ) );
-        tr_suspectAddress( &addr, "old tracker" );
         port = htons( itmp );
         memcpy( walk + sizeof( tr_address ), &port, 2 );
         walk += sizeof( tr_address ) + 2;
@@ -434,79 +434,75 @@ onTrackerResponse( tr_session * session,
             int64_t      i;
             int          incomplete = -1;
             const char * str;
+            const uint8_t * raw;
+            size_t rawlen;
 
             success = TRUE;
             t->retryAnnounceIntervalSec = FIRST_SCRAPE_RETRY_INTERVAL_SEC;
 
-            if( ( tr_bencDictFindStr( &benc, "failure reason", &str ) ) )
+            if( tr_bencDictFindStr( &benc, "failure reason", &str ) )
             {
                 publishMessage( t, str, TR_TRACKER_ERROR );
                 success = FALSE;
             }
 
-            if( ( tr_bencDictFindStr( &benc, "warning message", &str ) ) )
+            if( tr_bencDictFindStr( &benc, "warning message", &str ) )
                 publishWarning( t, str );
 
-            if( ( tr_bencDictFindInt( &benc, "interval", &i ) ) )
+            if( tr_bencDictFindInt( &benc, "interval", &i ) )
             {
                 dbgmsg( t->name, "setting interval to %d", (int)i );
                 t->announceIntervalSec = i;
             }
 
-            if( ( tr_bencDictFindInt( &benc, "min interval", &i ) ) )
+            if( tr_bencDictFindInt( &benc, "min interval", &i ) )
             {
                 dbgmsg( t->name, "setting min interval to %d", (int)i );
                 t->announceMinIntervalSec = i;
             }
 
-            if( ( tr_bencDictFindStr( &benc, "tracker id", &str ) ) )
+            if( tr_bencDictFindStr( &benc, "tracker id", &str ) )
                 t->trackerID = tr_strdup( str );
 
-            if( ( tr_bencDictFindInt( &benc, "complete", &i ) ) )
+            if( tr_bencDictFindInt( &benc, "complete", &i ) )
             {
                 ++scrapeFields;
                 t->seederCount = i;
             }
 
-            if( ( tr_bencDictFindInt( &benc, "incomplete", &i ) ) )
+            if( tr_bencDictFindInt( &benc, "incomplete", &i ) )
             {
                 ++scrapeFields;
                 t->leecherCount = incomplete = i;
             }
 
-            if( ( tr_bencDictFindInt( &benc, "downloaded", &i ) ) )
+            if( tr_bencDictFindInt( &benc, "downloaded", &i ) )
             {
                 ++scrapeFields;
                 t->timesDownloaded = i;
             }
 
-            if( ( tmp = tr_bencDictFind( &benc, "peers" ) ) )
+            if( tr_bencDictFindRaw( &benc, "peers", &raw, &rawlen ) )
             {
+                /* "compact" extension */
                 const int allAreSeeds = incomplete == 0;
-
-                if( tmp->type == TYPE_STR ) /* "compact" extension */
-                {
-                    publishNewPeersCompact( t, allAreSeeds, tmp->val.s.s,
-                                            tmp->val.s.i );
-                }
-                else if( tmp->type == TYPE_LIST ) /* original protocol */
-                {
-                    size_t    byteCount = 0;
-                    uint8_t * array = parseOldPeers( tmp, &byteCount );
-                    publishNewPeers( t, allAreSeeds, array, byteCount );
-                    tr_free( array );
-                }
+                publishNewPeersCompact( t, allAreSeeds, raw, rawlen );
             }
-            
-            if( ( tmp = tr_bencDictFind( &benc, "peers6" ) ) )
+            else if( tr_bencDictFindList( &benc, "peers", &tmp ) )
             {
+                /* original version of peers */
                 const int allAreSeeds = incomplete == 0;
-                
-                if( tmp->type == TYPE_STR ) /* "compact" extension */
-                {
-                    publishNewPeersCompact6( t, allAreSeeds, tmp->val.s.s,
-                                             tmp->val.s.i );
-                }
+                size_t byteCount = 0;
+                uint8_t * array = parseOldPeers( tmp, &byteCount );
+                publishNewPeers( t, allAreSeeds, array, byteCount );
+                tr_free( array );
+            }
+
+            if( tr_bencDictFindRaw( &benc, "peers6", &raw, &rawlen ) )
+            {
+                /* "compact" extension */
+                const int allAreSeeds = incomplete == 0;
+                publishNewPeersCompact6( t, allAreSeeds, raw, rawlen );
             }
         }
 
@@ -626,34 +622,34 @@ onScrapeResponse( tr_session * session,
                                              &benc, NULL );
         if( bencLoaded && tr_bencDictFindDict( &benc, "files", &files ) )
         {
-            size_t i;
-            for( i = 0; i < files->val.l.count; i += 2 )
+            const char * key;
+            tr_benc * val;
+            int i = 0;
+            while( tr_bencDictChild( files, i++, &key, &val ))
             {
-                int64_t        itmp;
-                const uint8_t* hash =
-                    (const uint8_t*) files->val.l.vals[i].val.s.s;
-                tr_benc *      flags;
-                tr_benc *      tordict = &files->val.l.vals[i + 1];
-                if( memcmp( t->hash, hash, SHA_DIGEST_LENGTH ) )
+                int64_t intVal;
+                tr_benc * flags;
+
+                if( memcmp( t->hash, key, SHA_DIGEST_LENGTH ) )
                     continue;
 
                 publishErrorClear( t );
 
-                if( ( tr_bencDictFindInt( tordict, "complete", &itmp ) ) )
-                    t->seederCount = itmp;
+                if( ( tr_bencDictFindInt( val, "complete", &intVal ) ) )
+                    t->seederCount = intVal;
 
-                if( ( tr_bencDictFindInt( tordict, "incomplete", &itmp ) ) )
-                    t->leecherCount = itmp;
+                if( ( tr_bencDictFindInt( val, "incomplete", &intVal ) ) )
+                    t->leecherCount = intVal;
 
-                if( ( tr_bencDictFindInt( tordict, "downloaded", &itmp ) ) )
-                    t->timesDownloaded = itmp;
+                if( ( tr_bencDictFindInt( val, "downloaded", &intVal ) ) )
+                    t->timesDownloaded = intVal;
 
-                if( ( tr_bencDictFindInt( tordict, "downloaders", &itmp ) ) )
-                    t->downloaderCount = itmp;
+                if( ( tr_bencDictFindInt( val, "downloaders", &intVal ) ) )
+                    t->downloaderCount = intVal;
 
-                if( tr_bencDictFindDict( tordict, "flags", &flags ) )
-                    if( ( tr_bencDictFindInt( flags, "min_request_interval", &itmp ) ) )
-                        t->scrapeIntervalSec = i;
+                if( tr_bencDictFindDict( val, "flags", &flags ) )
+                    if( ( tr_bencDictFindInt( flags, "min_request_interval", &intVal ) ) )
+                        t->scrapeIntervalSec = intVal;
 
                 /* as per ticket #1045, safeguard against trackers returning
                  * a very low min_request_interval... */
@@ -758,6 +754,7 @@ buildTrackerRequestURI( tr_tracker *       t,
                               "&corrupt=%" PRIu64
                               "&left=%" PRIu64
                               "&compact=1"
+                              "&supportcrypto=1"
                               "&numwant=%d"
                               "&key=%s",
                               strchr( ann, '?' ) ? '&' : '?',
@@ -771,6 +768,9 @@ buildTrackerRequestURI( tr_tracker *       t,
                               numwant,
                               t->key_param );
 
+    if( t->session->encryptionMode == TR_ENCRYPTION_REQUIRED )
+        evbuffer_add_printf( buf, "&requirecrypto=1" );
+
     if( eventName && *eventName )
         evbuffer_add_printf( buf, "&event=%s", eventName );
 
@@ -980,6 +980,18 @@ trackerPulse( void * vsession )
             t->manualAnnounceAllowedAt = TR_TRACKER_BUSY;
             enqueueRequest( session, t, TR_REQ_REANNOUNCE );
         }
+
+        if( tor->dhtAnnounceAt <= now ) {
+            int rc = 1;
+            if( tor->isRunning && tr_torrentAllowsDHT(tor) )
+                rc = tr_dhtAnnounce(tor, 1);
+            if(rc == 0)
+                /* The DHT is not ready yet.  Try again soon. */
+                tor->dhtAnnounceAt = now + 5 + tr_cryptoWeakRandInt( 5 );
+            else
+                /* We should announce at least once every 30 minutes. */
+                tor->dhtAnnounceAt = now + 25 * 60 + tr_cryptoWeakRandInt( 3 * 60 );
+        }
     }
 
     if( th->runningCount )
