@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: trevent.c 7897 2009-02-18 02:11:15Z charles $
+ * $Id: trevent.c 8681 2009-06-12 23:01:35Z charles $
  */
 
 #include <assert.h>
@@ -235,7 +235,8 @@ libeventThreadFunc( void * veh )
     event_set_log_callback( logFunc );
 
     /* loop until all the events are done */
-    event_dispatch( );
+    while( !eh->die )
+        event_dispatch( );
 
     /* shut down the thread */
     tr_lockFree( eh->lock );
@@ -302,10 +303,12 @@ timerCallback( int    fd UNUSED,
     more = ( *timer->func )( timer->user_data );
     timer->inCallback = 0;
 
-    if( more )
-        evtimer_add( &timer->event, &timer->tv );
-    else
+    if( !more )
         tr_timerFree( &timer );
+    else {
+        assert( tr_isTimeval( &timer->tv ) );
+        evtimer_add( &timer->event, &timer->tv );
+    }
 }
 
 void
