@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: upnp.h 7664 2009-01-11 17:46:51Z charles $
+ * $Id: upnp.h 8561 2009-05-29 19:17:12Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -17,6 +17,11 @@
 #ifndef TR_UPNP_H
 #define TR_UPNP_H 1
 
+/** 
+ * @addtogroup port_forwarding Port Forwarding
+ * @{
+ */
+
 typedef struct tr_upnp tr_upnp;
 
 tr_upnp * tr_upnpInit( void );
@@ -25,6 +30,7 @@ void      tr_upnpClose( tr_upnp * );
 
 int       tr_upnpPulse(         tr_upnp *,
                             int port,
-                            int isEnabled );
-
+                            int isEnabled,
+                            int doPortCheck );
+/* @} */
 #endif
