@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: utils.h 7855 2009-02-09 17:34:43Z charles $
+ * $Id: utils.h 8825 2009-07-17 18:00:29Z charles $
  */
 
 #ifndef TR_UTILS_H
@@ -31,6 +31,11 @@ extern "C" {
 ****
 ***/
 
+/**
+ * @addtogroup utils Utilities
+ * @{ 
+ */
+
 #ifndef FALSE
  #define FALSE 0
 #endif
@@ -192,13 +197,15 @@ char*          tr_getLogTimeStr( char * buf,
 int            tr_wildmat( const char * text,
                            const char * pattern );
 
-/** a portability wrapper for basename(). */
-char*          tr_basename( const char * path ) TR_GNUC_MALLOC;
+/** @brief Portability wrapper for basename() that uses the system implementation if available */
+char* tr_basename( const char * path ) TR_GNUC_MALLOC;
 
-/** a portability wrapper for dirname(). */
-char*          tr_dirname( const char * path ) TR_GNUC_MALLOC;
+/** @brief Portability wrapper for dirname() that uses the system implementation if available */
+char* tr_dirname( const char * path ) TR_GNUC_MALLOC;
 
 /**
+ * @brief Portability wrapper for mkdir()
+ *
  * a portability wrapper around mkdir().
  * On WIN32, the `permissions' argument is unused.
  *
@@ -214,62 +221,55 @@ int            tr_mkdir( const char * path,
  * @return zero on success, or -1 if an error occurred
  * (in which case errno is set appropriately).
  */
-int            tr_mkdirp( const char * path,
-                          int          permissions );
+int tr_mkdirp( const char * path, int permissions );
 
 
 /**
- * Loads a file and returns its contents.
+ * @brief Loads a file and returns its contents.
  * On failure, NULL is returned and errno is set.
  */
-uint8_t*       tr_loadFile( const char * filename,
-                            size_t *     size ) TR_GNUC_MALLOC;
+uint8_t* tr_loadFile( const char * filename, size_t * size ) TR_GNUC_MALLOC;
 
 
-/* creates a filename from a series of elements using the
-   correct separator for filenames. */
-char*          tr_buildPath( const char * first_element, ... )
-                                              TR_GNUC_NULL_TERMINATED
-                                              TR_GNUC_MALLOC;
+/** @brief build a filename from a series of elements using the
+           platform's correct directory separator. */
+char* tr_buildPath( const char * first_element, ... ) TR_GNUC_NULL_TERMINATED
+                                                      TR_GNUC_MALLOC;
 
 struct timeval;
 
-void tr_timevalMsec( uint64_t           milliseconds,
-                     struct timeval   * setme );
+tr_bool tr_isTimeval( const struct timeval * tv );
+
+void tr_timevalMsec( uint64_t milliseconds, struct timeval * setme );
+
+void tr_timevalSet( struct timeval * setme, int seconds, int microseconds );
+
+struct event;
+
+void tr_timerAdd( struct event * timer, int seconds, int milliseconds );
 
 
-/* return the current date in milliseconds */
+/** @brief return the current date in milliseconds */
 uint64_t       tr_date( void );
 
-/* wait the specified number of milliseconds */
+/** @brief wait the specified number of milliseconds */
 void           tr_wait( uint64_t delay_milliseconds );
 
-char*          tr_utf8clean( const char  * str,
-                             int           max_len,
-                             tr_bool     * err );
+/**
+ * @brief make a copy of 'str' whose non-utf8 content has been corrected or stripped
+ * @param str the string to make a clean copy of
+ * @param len the length of the string to copy.  If -1, the entire string is used.
+ * @param err if an error occurs and err is non-NULL, it's set to TRUE.
+ */
+char* tr_utf8clean( const char * str, int len, tr_bool * err ) TR_GNUC_MALLOC;
 
 
 /***
 ****
 ***/
 
-struct evbuffer;
-
-/** @brief pool of reusable buffers
-    @see tr_releaseBuffer() */
-struct evbuffer * tr_getBuffer( void );
-
-/** @brief return a buffer to the pool
-    @see tr_getBuffer() */
-void tr_releaseBuffer( struct evbuffer * buf );
-
-
-/***
-****
-***/
-
-/* Sometimes the system defines MAX/MIN, sometimes not. In the latter
-   case, define those here since we will use them */
+/* Sometimes the system defines MAX/MIN, sometimes not.
+   In the latter case, define those here since we will use them */
 #ifndef MAX
  #define MAX( a, b ) ( ( a ) > ( b ) ? ( a ) : ( b ) )
 #endif
@@ -337,18 +337,22 @@ char*       tr_base64_decode( const void * input,
                               int          inlen,
                               int *        outlen ) TR_GNUC_MALLOC;
 
-size_t      tr_strlcpy( char *       dst,
-                        const void * src,
-                        size_t       siz );
+/** @brief Portability wrapper for strlcpy() that uses the system implementation if available */
+size_t tr_strlcpy( char * dst, const void * src, size_t siz );
 
-int         tr_snprintf( char *       buf,
-                         size_t       buflen,
-                         const char * fmt,
-                         ... ) TR_GNUC_PRINTF( 3, 4 );
+/** @brief Portability wrapper for snprintf() that uses the system implementation if available */
+int tr_snprintf( char * buf, size_t buflen,
+                 const char * fmt, ... ) TR_GNUC_PRINTF( 3, 4 );
 
 const char* tr_strerror( int );
 
-char*       tr_strstrip( char * str );
+/** @brief strips leading and trailing whitspace from a string
+    @return the stripped string */
+char* tr_strstrip( char * str );
+
+/** @brief Portability wrapper for memmem() that uses the system implementation if available */
+const char* tr_memmem( const char * haystack, size_t haystack_len,
+                       const char * needle, size_t needle_len );
 
 /***
 ****
@@ -379,87 +383,49 @@ int  tr_httpParseURL( const char * url,
                       int *        setme_port,
                       char **      setme_path );
 
-
-/***
-****
-***/
-
-typedef struct tr_bitfield
-{
-    uint8_t *  bits;
-    size_t     bitCount;
-    size_t     byteCount;
-}
-tr_bitfield;
-
-tr_bitfield* tr_bitfieldConstruct( tr_bitfield*, size_t bitcount );
-
-tr_bitfield* tr_bitfieldDestruct( tr_bitfield* );
-
-static TR_INLINE tr_bitfield* tr_bitfieldNew( size_t bitcount )
-{
-    return tr_bitfieldConstruct( tr_new0( tr_bitfield, 1 ), bitcount );
-}
-
-static TR_INLINE void tr_bitfieldFree( tr_bitfield * b )
-{
-    tr_free( tr_bitfieldDestruct( b ) );
-}
-
-tr_bitfield* tr_bitfieldDup( const tr_bitfield* ) TR_GNUC_MALLOC;
-
-void         tr_bitfieldClear( tr_bitfield* );
-
-int          tr_bitfieldAdd( tr_bitfield*, size_t bit );
-
-int          tr_bitfieldRem( tr_bitfield*, size_t bit );
-
-int          tr_bitfieldAddRange( tr_bitfield *, size_t begin, size_t end );
-
-int          tr_bitfieldRemRange( tr_bitfield*, size_t begin, size_t end );
-
-void         tr_bitfieldDifference( tr_bitfield *, const tr_bitfield * );
-
-int          tr_bitfieldIsEmpty( const tr_bitfield* );
-
-size_t       tr_bitfieldCountTrueBits( const tr_bitfield* );
-
-tr_bitfield* tr_bitfieldOr( tr_bitfield*, const tr_bitfield* );
-
-/** A stripped-down version of bitfieldHas to be used
-    for speed when you're looping quickly.  This version
-    has none of tr_bitfieldHas()'s safety checks, so you
-    need to call tr_bitfieldTestFast() first before you
-    start looping. */
-static TR_INLINE tr_bool tr_bitfieldHasFast( const tr_bitfield * b, const size_t nth )
-{
-    return ( b->bits[nth>>3u] << ( nth & 7u ) & 0x80 ) != 0;
-}
-
-/** @param high the highest nth bit you're going to access */
-static TR_INLINE tr_bool tr_bitfieldTestFast( const tr_bitfield * b, const size_t high )
-{
-    return ( b != NULL )
-        && ( b->bits != NULL )
-        && ( high < b->bitCount );
-}
-
-static TR_INLINE tr_bool tr_bitfieldHas( const tr_bitfield * b, size_t nth )
-{
-    return tr_bitfieldTestFast( b, nth ) && tr_bitfieldHasFast( b, nth );
-}
-
 double tr_getRatio( double numerator, double denominator );
 
-int* tr_parseNumberRange( const char * str, int str_len, int * setmeCount );
+int* tr_parseNumberRange( const char * str, int str_len, int * setmeCount ) TR_GNUC_MALLOC;
 
 
 int tr_ptr2int( void* );
 
 void* tr_int2ptr( int );
 
+/* truncate a double value at a given number of decimal places.
+   this can be used to prevent a printf() call from rounding up:
+   call with the decimal_places argument equal to the number of
+   decimal places in the printf()'s precision:
+   
+   - printf("%.2f%%",           99.999    ) ==> "100.00%"
+
+   - printf("%.2f%%", tr_truncd(99.999, 2)) ==>  "99.99%"
+               ^                        ^
+               |   These should match   |
+               +------------------------+
+*/
+double tr_truncd( double x, int decimal_places );
+
+/**
+ * @param buf the buffer to write the string to
+ * @param buflef buf's size
+ * @param ratio the ratio to convert to a string
+ * @param the string represntation of "infinity"
+ */
+char* tr_strratio( char * buf, size_t buflen, double ratio, const char * infinity );
+
+/** @brief Portability wrapper for localtime_r() that uses the system implementation if available */
+struct tm * tr_localtime_r( const time_t *_clock, struct tm *_result );
+
+
+/** on success, return 0.  on failure, return -1 and set errno */
+int tr_moveFile( const char * oldpath, const char * newpath );
+
+
 #ifdef __cplusplus
 }
 #endif
 
+/** @} */
+
 #endif
