@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: web.c 7906 2009-02-18 16:16:26Z charles $
+ * $Id: web.c 8767 2009-07-01 14:58:57Z charles $
  */
 
 #include <assert.h>
@@ -19,11 +19,12 @@
 #include <curl/curl.h>
 
 #include "transmission.h"
-#include "session.h"
 #include "list.h"
 #include "net.h" /* socklen_t */
+#include "session.h"
 #include "trevent.h"
 #include "utils.h"
+#include "version.h"
 #include "web.h"
 
 /* Use curl_multi_socket_action() instead of curl_multi_perform()
@@ -72,6 +73,7 @@ struct tr_web
     CURLM * multi;
     tr_session * session;
     struct event timer_event;
+    tr_list * easy_queue;
     tr_list * fds;
 };
 
@@ -82,7 +84,7 @@ struct tr_web
 static struct tr_web_sockinfo *
 getSockinfo( tr_web * web, int fd, tr_bool createIfMissing )
 {
-    tr_list * l = web->fds;
+    tr_list * l;
 
     for( l=web->fds; l!=NULL; l=l->next ) {
         struct tr_web_sockinfo * s =  l->data;
@@ -214,7 +216,10 @@ addTask( void * vtask )
         else /* don't set encoding on webseeds; it messes up binary data */
             curl_easy_setopt( easy, CURLOPT_ENCODING, "" );
 
-        {
+        if( web->still_running >= MAX_CONCURRENT_TASKS ) {
+            tr_list_append( &web->easy_queue, easy );
+            dbgmsg( ">> enqueueing a task... size is now %d", tr_list_size( web->easy_queue ) );
+        } else {
             const CURLMcode mcode = curl_multi_add_handle( web->multi, easy );
             tr_assert( mcode == CURLM_OK, "curl_multi_add_handle() failed: %d (%s)", mcode, curl_multi_strerror( mcode ) );
             if( mcode == CURLM_OK )
@@ -324,6 +329,27 @@ restart_timer( tr_web * g )
     evtimer_add( &g->timer_event, &interval );
 }
 
+static void
+add_tasks_from_queue( tr_web * g )
+{
+    while( ( g->still_running < MAX_CONCURRENT_TASKS ) 
+        && ( tr_list_size( g->easy_queue ) > 0 ) )
+    {
+        CURL * easy = tr_list_pop_front( &g->easy_queue );
+        if( easy )
+        {
+            const CURLMcode rc = curl_multi_add_handle( g->multi, easy );
+            if( rc != CURLM_OK )
+                tr_err( "%s", curl_multi_strerror( rc ) );
+            else {
+                dbgmsg( "pumped the task queue, %d remain",
+                        tr_list_size( g->easy_queue ) );
+                ++g->still_running;
+            }
+        }
+    }
+}
+
 static void
 web_close( tr_web * g )
 {
@@ -372,6 +398,8 @@ tr_multi_perform( tr_web * g, int fd )
 
     remove_finished_tasks( g );
 
+    add_tasks_from_queue( g );
+
     if( !g->still_running ) {
         assert( tr_list_size( g->fds ) == 0 );
         stop_timer( g );
