@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: FileOutlineView.m 5383 2008-03-25 14:10:05Z livings124 $
+ * $Id: FileOutlineView.m 8533 2009-05-26 00:29:36Z livings124 $
  * 
- * Copyright (c) 2007-2008 Transmission authors and contributors
+ * Copyright (c) 2007-2009 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -22,11 +22,13 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
+#import "InfoWindowController.h"
 #import "FileOutlineView.h"
 #import "FileNameCell.h"
 #import "FilePriorityCell.h"
 #import "Torrent.h"
-#import "CTGradient.h"
+#import "FileListNode.h"
+#import "QuickLookController.h"
 
 @implementation FileOutlineView
 
@@ -45,15 +47,15 @@
     
     NSColor * endingColor = [NSColor colorWithCalibratedRed: 217.0/255.0 green: 250.0/255.0 blue: 211.0/255.0 alpha: 1.0];
     NSColor * beginningColor = [endingColor blendedColorWithFraction: 0.3 ofColor: [NSColor whiteColor]];
-    fHighPriorityGradient = [[CTGradient gradientWithBeginningColor: beginningColor endingColor: endingColor] retain];
+    fHighPriorityGradient = [[NSGradient alloc] initWithStartingColor: beginningColor endingColor: endingColor];
     
     endingColor = [NSColor colorWithCalibratedRed: 255.0/255.0 green: 243.0/255.0 blue: 206.0/255.0 alpha: 1.0];
     beginningColor = [endingColor blendedColorWithFraction: 0.3 ofColor: [NSColor whiteColor]];
-    fLowPriorityGradient = [[CTGradient gradientWithBeginningColor: beginningColor endingColor: endingColor] retain];
+    fLowPriorityGradient = [[NSGradient alloc] initWithStartingColor: beginningColor endingColor: endingColor];
     
     endingColor = [NSColor colorWithCalibratedRed: 225.0/255.0 green: 218.0/255.0 blue: 255.0/255.0 alpha: 1.0];
     beginningColor = [endingColor blendedColorWithFraction: 0.3 ofColor: [NSColor whiteColor]];
-    fMixedPriorityGradient = [[CTGradient gradientWithBeginningColor: beginningColor endingColor: endingColor] retain];
+    fMixedPriorityGradient = [[NSGradient alloc] initWithStartingColor: beginningColor endingColor: endingColor];
     
     fMouseRow = -1;
 }
@@ -83,6 +85,22 @@
     [super mouseDown: event];
 }
 
+- (void) keyDown: (NSEvent *) event
+{
+    const unichar firstChar = [[event charactersIgnoringModifiers] characterAtIndex: 0];
+    
+    //don't allow quick look on add window
+    if (firstChar == ' ' && [[[self window] windowController] isKindOfClass: [InfoWindowController class]])
+        [[QuickLookController quickLook] toggleQuickLook];
+    else if (firstChar == NSRightArrowFunctionKey)
+        [[QuickLookController quickLook] pressRight];
+    else if (firstChar == NSLeftArrowFunctionKey)
+        [[QuickLookController quickLook] pressLeft];
+    else;
+    
+    [super keyDown: event];  
+}
+
 - (NSMenu *) menuForEvent: (NSEvent *) event
 {
     int row = [self rowAtPoint: [self convertPoint: [event locationInWindow] fromView: nil]];
@@ -98,13 +116,20 @@
     return [self menu];
 }
 
+- (NSRect) iconRectForRow: (int) row
+{
+    FileNameCell * cell = (FileNameCell *)[self preparedCellAtColumn: [self columnWithIdentifier: @"Name"] row: row];
+    NSRect iconRect = [cell imageRectForBounds: [self rectOfRow: row]];
+    
+    iconRect.origin.x += [self indentationPerLevel] * (float)([self levelForRow: row] + 1);
+    return iconRect;
+}
+
 - (void) updateTrackingAreas
 {
     [super updateTrackingAreas];
     
-    NSEnumerator * enumerator = [[self trackingAreas] objectEnumerator];
-    NSTrackingArea * area;
-    while ((area = [enumerator nextObject]))
+    for (NSTrackingArea * area in [self trackingAreas])
     {
         if ([area owner] == self && [[area userInfo] objectForKey: @"Row"])
             [self removeTrackingArea: area];
@@ -114,11 +139,9 @@
     if (visibleRows.length == 0)
         return;
     
-    int col = [self columnWithIdentifier: @"Priority"];
     NSPoint mouseLocation = [self convertPoint: [[self window] convertScreenToBase: [NSEvent mouseLocation]] fromView: nil];
     
-    int row;
-    for (row = visibleRows.location; row < NSMaxRange(visibleRows); row++)
+    for (NSInteger row = visibleRows.location, col = [self columnWithIdentifier: @"Priority"]; row < NSMaxRange(visibleRows); row++)
     {
         FilePriorityCell * cell = (FilePriorityCell *)[self preparedCellAtColumn: col row: row];
         
@@ -153,16 +176,16 @@
     }
 }
 
-- (void) drawRow: (int) row clipRect: (NSRect) clipRect
+- (void) drawRow: (NSInteger) row clipRect: (NSRect) clipRect
 {
     if (![self isRowSelected: row])
     {
         NSDictionary * item = [self itemAtRow: row]; 
-        NSIndexSet * indexes = [item objectForKey: @"Indexes"];
+        NSIndexSet * indexes = [(FileListNode *)item indexes];
         
         if ([fTorrent checkForFiles: indexes] != NSOffState)
         {
-            CTGradient * gradient = nil;
+            NSGradient * gradient = nil;
             
             NSSet * priorities = [fTorrent filePrioritiesForIndexes: indexes];
             int count = [priorities count];
@@ -186,7 +209,7 @@
             {
                 NSRect rect = [self rectOfRow: row];
                 rect.size.height -= 1.0;
-                [gradient fillRect: rect angle: 90];
+                [gradient drawInRect: rect angle: 90];
             }
         }
     }
