@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: InfoWindowController.m 7016 2008-11-02 01:07:01Z livings124 $
+ * $Id: InfoWindowController.m 8800 2009-07-14 00:11:50Z livings124 $
  *
- * Copyright (c) 2006-2008 Transmission authors and contributors
+ * Copyright (c) 2006-2009 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,10 +28,10 @@
 #import "FileOutlineView.h"
 #import "FileOutlineController.h"
 #import "FileListNode.h"
+#import "PeerProgressIndicatorCell.h"
 #import "TrackerTableView.h"
 #import "PiecesView.h"
 #import "QuickLookController.h"
-#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #include "utils.h" //tr_getRatio()
 
@@ -51,6 +51,10 @@
 #define OPTION_POPUP_NO_LIMIT 1
 #define OPTION_POPUP_LIMIT 2
 
+#define OPTION_POPUP_PRIORITY_HIGH 0
+#define OPTION_POPUP_PRIORITY_NORMAL 1
+#define OPTION_POPUP_PRIORITY_LOW 2
+
 #define INVALID -99
 
 #define TRACKER_ADD_TAG 0
@@ -166,26 +170,17 @@ typedef enum
                                             sortDescriptorPrototype]]];
     
     //set table header tool tips
-    if ([NSApp isOnLeopardOrBetter])
-    {
-        [[fPeerTable tableColumnWithIdentifier: @"Encryption"] setHeaderToolTip: NSLocalizedString(@"Encrypted Connection",
-                                                                            "inspector -> peer table -> header tool tip")];
-        [[fPeerTable tableColumnWithIdentifier: @"Progress"] setHeaderToolTip: NSLocalizedString(@"Available",
-                                                                            "inspector -> peer table -> header tool tip")];
-        [[fPeerTable tableColumnWithIdentifier: @"UL To"] setHeaderToolTip: NSLocalizedString(@"Uploading To Peer",
-                                                                            "inspector -> peer table -> header tool tip")];
-        [[fPeerTable tableColumnWithIdentifier: @"DL From"] setHeaderToolTip: NSLocalizedString(@"Downloading From Peer",
-                                                                            "inspector -> peer table -> header tool tip")];
-        
-        [[fWebSeedTable tableColumnWithIdentifier: @"DL From"] setHeaderToolTip: NSLocalizedString(@"Downloading From Web Seed",
-                                                                            "inspector -> web seed table -> header tool tip")];
-    }
-    else
-    {
-        [fTrackerAddRemoveControl sizeToFit];
-        [fTrackerAddRemoveControl setLabel: @"+" forSegment: TRACKER_ADD_TAG];
-        [fTrackerAddRemoveControl setLabel: @"-" forSegment: TRACKER_REMOVE_TAG];
-    }
+    [[fPeerTable tableColumnWithIdentifier: @"Encryption"] setHeaderToolTip: NSLocalizedString(@"Encrypted Connection",
+                                                                        "inspector -> peer table -> header tool tip")];
+    [[fPeerTable tableColumnWithIdentifier: @"Progress"] setHeaderToolTip: NSLocalizedString(@"Available",
+                                                                        "inspector -> peer table -> header tool tip")];
+    [[fPeerTable tableColumnWithIdentifier: @"UL To"] setHeaderToolTip: NSLocalizedString(@"Uploading To Peer",
+                                                                        "inspector -> peer table -> header tool tip")];
+    [[fPeerTable tableColumnWithIdentifier: @"DL From"] setHeaderToolTip: NSLocalizedString(@"Downloading From Peer",
+                                                                        "inspector -> peer table -> header tool tip")];
+    
+    [[fWebSeedTable tableColumnWithIdentifier: @"DL From"] setHeaderToolTip: NSLocalizedString(@"Downloading From Web Seed",
+                                                                        "inspector -> web seed table -> header tool tip")];
     
     //prepare for animating peer table and web seed table
     NSRect webSeedTableFrame = [[fWebSeedTable enclosingScrollView] frame];
@@ -247,17 +242,14 @@ typedef enum
     {
         if (numberSelected > 0)
         {
-            [fImageView setImage: [NSImage imageNamed: [NSApp isOnLeopardOrBetter]
-                                    ? NSImageNameMultipleDocuments : @"NSApplicationIcon"]];
+            [fImageView setImage: [NSImage imageNamed: NSImageNameMultipleDocuments]];
             
             [fNameField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%d Torrents Selected",
                                             "Inspector -> selected torrents"), numberSelected]];
         
             uint64_t size = 0;
             NSInteger fileCount = 0;
-            NSEnumerator * enumerator = [torrents objectEnumerator];
-            Torrent * torrent;
-            while ((torrent = [enumerator nextObject]))
+            for (Torrent * torrent in torrents)
             {
                 size += [torrent size];
                 fileCount += [torrent fileCount];
@@ -286,18 +278,24 @@ typedef enum
             [fRatioField setStringValue: @""];
             
             //options fields
-            [fUploadLimitPopUp setEnabled: NO];
-            [fUploadLimitPopUp selectItemAtIndex: -1];
-            [fUploadLimitField setHidden: YES];
-            [fUploadLimitLabel setHidden: YES];
+            [fUploadLimitCheck setEnabled: NO];
+            [fUploadLimitCheck setState: NSOffState];
+            [fUploadLimitField setEnabled: NO];
+            [fUploadLimitLabel setEnabled: NO];
             [fUploadLimitField setStringValue: @""];
             
-            [fDownloadLimitPopUp setEnabled: NO];
-            [fDownloadLimitPopUp selectItemAtIndex: -1];
-            [fDownloadLimitField setHidden: YES];
-            [fDownloadLimitLabel setHidden: YES];
+            [fDownloadLimitCheck setEnabled: NO];
+            [fDownloadLimitCheck setState: NSOffState];
+            [fDownloadLimitField setEnabled: NO];
+            [fDownloadLimitLabel setEnabled: NO];
             [fDownloadLimitField setStringValue: @""];
             
+            [fGlobalLimitCheck setEnabled: NO];
+            [fGlobalLimitCheck setState: NSOffState];
+            
+            [fPriorityPopUp setEnabled: NO];
+            [fPriorityPopUp selectItemAtIndex: -1];
+            
             [fRatioPopUp setEnabled: NO];
             [fRatioPopUp selectItemAtIndex: -1];
             [fRatioLimitField setHidden: YES];
@@ -393,6 +391,8 @@ typedef enum
         
         [fTrackerAddRemoveControl setEnabled: NO forSegment: TRACKER_ADD_TAG];
         [fTrackerAddRemoveControl setEnabled: NO forSegment: TRACKER_REMOVE_TAG];
+        
+        [fFileFilterField setEnabled: NO];
     }
     else
     {
@@ -430,7 +430,7 @@ typedef enum
         [fHashField setStringValue: hashString];
         [fHashField setToolTip: hashString];
         [fSecureField setStringValue: [torrent privateTorrent]
-                        ? NSLocalizedString(@"Private Torrent, PEX automatically disabled", "Inspector -> private torrent")
+                        ? NSLocalizedString(@"Private Torrent, PEX and DHT automatically disabled", "Inspector -> private torrent")
                         : NSLocalizedString(@"Public Torrent", "Inspector -> private torrent")];
         
         NSString * commentString = [torrent comment];
@@ -491,8 +491,12 @@ typedef enum
         
         [fTrackerAddRemoveControl setEnabled: YES forSegment: TRACKER_ADD_TAG];
         [fTrackerAddRemoveControl setEnabled: NO forSegment: TRACKER_REMOVE_TAG];
+        
+        [fFileFilterField setEnabled: [torrent isFolder]];
     }
     
+    [fFileFilterField setStringValue: @""];
+    
     //update stats and settings
     [self updateInfoStats];
     [self updateOptions];
@@ -533,67 +537,59 @@ typedef enum
     NSEnumerator * enumerator = [fTorrents objectEnumerator];
     Torrent * torrent = [enumerator nextObject]; //first torrent
     
-    NSInteger uploadSpeedMode = [torrent speedMode: YES],
+    NSInteger uploadUseSpeedLimit = [torrent usesSpeedLimit: YES] ? NSOnState : NSOffState,
                 uploadSpeedLimit = [torrent speedLimit: YES],
-                downloadSpeedMode = [torrent speedMode: NO],
-                downloadSpeedLimit = [torrent speedLimit: NO];
+                downloadUseSpeedLimit = [torrent usesSpeedLimit: NO] ? NSOnState : NSOffState,
+                downloadSpeedLimit = [torrent speedLimit: NO],
+                globalUseSpeedLimit = [torrent usesGlobalSpeedLimit] ? NSOnState : NSOffState;
     
     while ((torrent = [enumerator nextObject])
-            && (uploadSpeedMode != INVALID || uploadSpeedLimit != INVALID
-                || downloadSpeedMode != INVALID || downloadSpeedLimit != INVALID))
+            && (uploadUseSpeedLimit != NSMixedState || uploadSpeedLimit != INVALID
+                || downloadUseSpeedLimit != NSMixedState || downloadSpeedLimit != INVALID
+                || globalUseSpeedLimit != NSMixedState))
     {
-        if (uploadSpeedMode != INVALID && uploadSpeedMode != [torrent speedMode: YES])
-            uploadSpeedMode = INVALID;
+        if (uploadUseSpeedLimit != NSMixedState && uploadUseSpeedLimit != ([torrent usesSpeedLimit: YES] ? NSOnState : NSOffState))
+            uploadUseSpeedLimit = NSMixedState;
         
         if (uploadSpeedLimit != INVALID && uploadSpeedLimit != [torrent speedLimit: YES])
             uploadSpeedLimit = INVALID;
         
-        if (downloadSpeedMode != INVALID && downloadSpeedMode != [torrent speedMode: NO])
-            downloadSpeedMode = INVALID;
+        if (downloadUseSpeedLimit != NSMixedState && downloadUseSpeedLimit != ([torrent usesSpeedLimit: NO] ? NSOnState : NSOffState))
+            downloadUseSpeedLimit = NSMixedState;
         
         if (downloadSpeedLimit != INVALID && downloadSpeedLimit != [torrent speedLimit: NO])
             downloadSpeedLimit = INVALID;
+        
+        if (globalUseSpeedLimit != NSMixedState && globalUseSpeedLimit != ([torrent usesGlobalSpeedLimit] ? NSOnState : NSOffState))
+            globalUseSpeedLimit = NSMixedState;
     }
     
     //set upload view
-    NSInteger index;
-    if (uploadSpeedMode == TR_SPEEDLIMIT_SINGLE)
-        index = OPTION_POPUP_LIMIT;
-    else if (uploadSpeedMode == TR_SPEEDLIMIT_UNLIMITED)
-        index = OPTION_POPUP_NO_LIMIT;
-    else if (uploadSpeedMode == TR_SPEEDLIMIT_GLOBAL)
-        index = OPTION_POPUP_GLOBAL;
-    else
-        index = -1;
-    [fUploadLimitPopUp selectItemAtIndex: index];
-    [fUploadLimitPopUp setEnabled: YES];
+    [fUploadLimitCheck setState: uploadUseSpeedLimit];
+    [fUploadLimitCheck setEnabled: YES];
     
-    [fUploadLimitLabel setHidden: uploadSpeedMode != TR_SPEEDLIMIT_SINGLE];
-    [fUploadLimitField setHidden: uploadSpeedMode != TR_SPEEDLIMIT_SINGLE];
+    [fUploadLimitLabel setEnabled: uploadUseSpeedLimit == NSOnState];
+    [fUploadLimitField setEnabled: uploadUseSpeedLimit == NSOnState];
     if (uploadSpeedLimit != INVALID)
         [fUploadLimitField setIntValue: uploadSpeedLimit];
     else
         [fUploadLimitField setStringValue: @""];
     
     //set download view
-    if (downloadSpeedMode == TR_SPEEDLIMIT_SINGLE)
-        index = OPTION_POPUP_LIMIT;
-    else if (downloadSpeedMode == TR_SPEEDLIMIT_UNLIMITED)
-        index = OPTION_POPUP_NO_LIMIT;
-    else if (downloadSpeedMode == TR_SPEEDLIMIT_GLOBAL)
-        index = OPTION_POPUP_GLOBAL;
-    else
-        index = -1;
-    [fDownloadLimitPopUp selectItemAtIndex: index];
-    [fDownloadLimitPopUp setEnabled: YES];
+    [fDownloadLimitCheck setState: downloadUseSpeedLimit];
+    [fDownloadLimitCheck setEnabled: YES];
     
-    [fDownloadLimitLabel setHidden: downloadSpeedMode != TR_SPEEDLIMIT_SINGLE];
-    [fDownloadLimitField setHidden: downloadSpeedMode != TR_SPEEDLIMIT_SINGLE];
+    [fDownloadLimitLabel setEnabled: downloadUseSpeedLimit == NSOnState];
+    [fDownloadLimitField setEnabled: downloadUseSpeedLimit == NSOnState];
     if (downloadSpeedLimit != INVALID)
         [fDownloadLimitField setIntValue: downloadSpeedLimit];
     else
         [fDownloadLimitField setStringValue: @""];
     
+    //set global check
+    [fGlobalLimitCheck setState: globalUseSpeedLimit];
+    [fGlobalLimitCheck setEnabled: YES];
+    
     //get ratio info
     enumerator = [fTorrents objectEnumerator];
     torrent = [enumerator nextObject]; //first torrent
@@ -601,7 +597,7 @@ typedef enum
     NSInteger checkRatio = [torrent ratioSetting];
     CGFloat ratioLimit = [torrent ratioLimit];
     
-    while ((torrent = [enumerator nextObject]) && (checkRatio != INVALID || checkRatio != INVALID))
+    while ((torrent = [enumerator nextObject]) && (checkRatio != INVALID || ratioLimit != INVALID))
     {
         if (checkRatio != INVALID && checkRatio != [torrent ratioSetting])
             checkRatio = INVALID;
@@ -611,23 +607,48 @@ typedef enum
     }
     
     //set ratio view
-    if (checkRatio == NSOnState)
+    NSInteger index;
+    if (checkRatio == TR_RATIOLIMIT_SINGLE)
         index = OPTION_POPUP_LIMIT;
-    else if (checkRatio == NSOffState)
+    else if (checkRatio == TR_RATIOLIMIT_UNLIMITED)
         index = OPTION_POPUP_NO_LIMIT;
-    else if (checkRatio == NSMixedState)
+    else if (checkRatio == TR_RATIOLIMIT_GLOBAL)
         index = OPTION_POPUP_GLOBAL;
     else
         index = -1;
     [fRatioPopUp selectItemAtIndex: index];
     [fRatioPopUp setEnabled: YES];
     
-    [fRatioLimitField setHidden: checkRatio != NSOnState];
+    [fRatioLimitField setHidden: checkRatio != TR_RATIOLIMIT_SINGLE];
     if (ratioLimit != INVALID)
         [fRatioLimitField setFloatValue: ratioLimit];
     else
         [fRatioLimitField setStringValue: @""];
     
+    //get priority info
+    enumerator = [fTorrents objectEnumerator];
+    torrent = [enumerator nextObject]; //first torrent
+    
+    NSInteger priority = [torrent priority];
+    
+    while ((torrent = [enumerator nextObject]) && priority != INVALID)
+    {
+        if (priority != INVALID && priority != [torrent priority])
+            priority = INVALID;
+    }
+    
+    //set priority view
+    if (priority == TR_PRI_HIGH)
+        index = OPTION_POPUP_PRIORITY_HIGH;
+    else if (priority == TR_PRI_NORMAL)
+        index = OPTION_POPUP_PRIORITY_NORMAL;
+    else if (priority == TR_PRI_LOW)
+        index = OPTION_POPUP_PRIORITY_LOW;
+    else
+        index = -1;
+    [fPriorityPopUp selectItemAtIndex: index];
+    [fPriorityPopUp setEnabled: YES];
+    
     //get peer info
     enumerator = [fTorrents objectEnumerator];
     torrent = [enumerator nextObject]; //first torrent
@@ -910,6 +931,21 @@ typedef enum
     return nil;
 }
 
+- (void) tableView: (NSTableView *) tableView willDisplayCell: (id) cell forTableColumn: (NSTableColumn *) tableColumn
+    row: (NSInteger) row
+{
+    if (tableView == fPeerTable)
+    {
+        NSString * ident = [tableColumn identifier];
+        
+        if  ([ident isEqualToString: @"Progress"])
+        {
+            NSDictionary * peer = [fPeers objectAtIndex: row];
+            [(PeerProgressIndicatorCell *)cell setSeed: [[peer objectForKey: @"Seed"] boolValue]];
+        }
+    }
+}
+
 - (void) tableView: (NSTableView *) tableView didClickTableColumn: (NSTableColumn *) tableColumn
 {
     if (tableView == fPeerTable)
@@ -964,8 +1000,13 @@ typedef enum
         NSDictionary * peer = [fPeers objectAtIndex: row];
         NSMutableArray * components = [NSMutableArray arrayWithCapacity: 5];
         
-        [components addObject: [NSString localizedStringWithFormat: NSLocalizedString(@"Progress: %.1f%%",
-            "Inspector -> Peers tab -> table row tooltip"), [[peer objectForKey: @"Progress"] floatValue] * 100.0]];
+        CGFloat progress = [[peer objectForKey: @"Progress"] floatValue];
+        NSString * progressString = [NSString localizedStringWithFormat: NSLocalizedString(@"Progress: %.1f%%",
+                                        "Inspector -> Peers tab -> table row tooltip"), progress * 100.0];
+        if (progress < 1.0 && [[peer objectForKey: @"Seed"] boolValue])
+            progressString = [progressString stringByAppendingFormat: @" (%@)", NSLocalizedString(@"Partial Seed",
+                                "Inspector -> Peers tab -> table row tooltip")];
+        [components addObject: progressString];
         
         if ([[peer objectForKey: @"Encryption"] boolValue])
             [components addObject: NSLocalizedString(@"Encrypted Connection", "Inspector -> Peers tab -> table row tooltip")];
@@ -993,38 +1034,39 @@ typedef enum
             case TR_PEER_FROM_PEX:
                 [components addObject: NSLocalizedString(@"From: peer exchange", "Inspector -> Peers tab -> table row tooltip")];
                 break;
+            case TR_PEER_FROM_DHT:
+                [components addObject: NSLocalizedString(@"From: distributed hash table", "Inspector -> Peers tab -> table row tooltip")];
+                break;
         }
         
         //determing status strings from flags 
-        NSMutableArray * statusArray = [NSMutableArray arrayWithCapacity: 3];
+        NSMutableArray * statusArray = [NSMutableArray arrayWithCapacity: 6];
         NSString * flags = [peer objectForKey: @"Flags"];
         
         if ([flags rangeOfString: @"D"].location != NSNotFound)
             [statusArray addObject: NSLocalizedString(@"Currently downloading (interested and not choked)",
                 "Inspector -> peer -> status")];
-        else if ([flags rangeOfString: @"d"].location != NSNotFound)
+        if ([flags rangeOfString: @"d"].location != NSNotFound)
             [statusArray addObject: NSLocalizedString(@"You want to download, but peer does not want to send (interested and choked)",
                 "Inspector -> peer -> status")];
-        else;
-        
         if ([flags rangeOfString: @"U"].location != NSNotFound)
             [statusArray addObject: NSLocalizedString(@"Currently uploading (interested and not choked)",
                 "Inspector -> peer -> status")];
-        else if ([flags rangeOfString: @"u"].location != NSNotFound)
+        if ([flags rangeOfString: @"u"].location != NSNotFound)
             [statusArray addObject: NSLocalizedString(@"Peer wants you to upload, but you do not want to (interested and choked)",
                 "Inspector -> peer -> status")];
-        else;
-        
         if ([flags rangeOfString: @"K"].location != NSNotFound)
             [statusArray addObject: NSLocalizedString(@"Peer is unchoking you, but you are not interested",
                 "Inspector -> peer -> status")];
-        
         if ([flags rangeOfString: @"?"].location != NSNotFound)
             [statusArray addObject: NSLocalizedString(@"You unchoked the peer, but the peer is not interested",
                 "Inspector -> peer -> status")];
         
         if ([statusArray count] > 0)
-            [components addObject: [@"\n" stringByAppendingString: [statusArray componentsJoinedByString: @"\n\n"]]];
+        {
+            NSString * statusStrings = [statusArray componentsJoinedByString: @"\n\n"];
+            [components addObject: [@"\n" stringByAppendingString: statusStrings]];
+        }
         
         return [components componentsJoinedByString: @"\n"];
     }
@@ -1164,111 +1206,124 @@ typedef enum
         [[fTorrents objectAtIndex: 0] revealData];
 }
 
-- (void) setSpeedMode: (id) sender
+- (void) setFileFilterText: (id) sender
 {
-    BOOL upload = sender == fUploadLimitPopUp;
-    NSInteger mode;
-    switch ([sender indexOfSelectedItem])
-    {
-        case OPTION_POPUP_LIMIT:
-            mode = TR_SPEEDLIMIT_SINGLE;
-            break;
-        case OPTION_POPUP_NO_LIMIT:
-            mode = TR_SPEEDLIMIT_UNLIMITED;
-            break;
-        case OPTION_POPUP_GLOBAL:
-            mode = TR_SPEEDLIMIT_GLOBAL;
-            break;
-        default:
-            return;
-    }
+    [fFileController setFilterText: [sender stringValue]];
+}
+
+- (void) setUseSpeedLimit: (id) sender
+{
+    const BOOL upload = sender == fUploadLimitCheck;
     
-    Torrent * torrent;
-    NSEnumerator * enumerator = [fTorrents objectEnumerator];
-    while ((torrent = [enumerator nextObject]))
-        [torrent setSpeedMode: mode upload: upload];
+    if ([sender state] == NSMixedState)
+        [sender setState: NSOnState];
+    const BOOL limit = [sender state] == NSOnState;
+    
+    for (Torrent * torrent in fTorrents)
+        [torrent setUseSpeedLimit: limit upload: upload];
     
     NSTextField * field = upload ? fUploadLimitField : fDownloadLimitField;
-    
-    BOOL single = mode == TR_SPEEDLIMIT_SINGLE;
-    [field setHidden: !single];
-    if (single)
+    [field setEnabled: limit];
+    if (limit)
     {
         [field selectText: self];
-        [[self window] makeKeyAndOrderFront:self];
+        [[self window] makeKeyAndOrderFront: self];
     }
     
     NSTextField * label = upload ? fUploadLimitLabel : fDownloadLimitLabel;
-    [label setHidden: !single];
+    [label setEnabled: limit];
+}
+
+- (void) setUseGlobalSpeedLimit: (id) sender
+{
+    if ([sender state] == NSMixedState)
+        [sender setState: NSOnState];
+    const BOOL limit = [sender state] == NSOnState;
+    
+    for (Torrent * torrent in fTorrents)
+        [torrent setUseGlobalSpeedLimit: limit];
 }
 
 - (void) setSpeedLimit: (id) sender
 {
-    BOOL upload = sender == fUploadLimitField;
-    NSInteger limit = [sender intValue];
+    const BOOL upload = sender == fUploadLimitField;
+    const NSInteger limit = [sender intValue];
     
-    Torrent * torrent;
-    NSEnumerator * enumerator = [fTorrents objectEnumerator];
-    
-    while ((torrent = [enumerator nextObject]))
+    for (Torrent * torrent in fTorrents)
         [torrent setSpeedLimit: limit upload: upload];
 }
 
 - (void) setRatioSetting: (id) sender
 {
     NSInteger setting;
+    bool single = NO;
     switch ([sender indexOfSelectedItem])
     {
         case OPTION_POPUP_LIMIT:
-            setting = NSOnState;
+            setting = TR_RATIOLIMIT_SINGLE;
+            single = YES;
             break;
         case OPTION_POPUP_NO_LIMIT:
-            setting = NSOffState;
+            setting = TR_RATIOLIMIT_UNLIMITED;
             break;
         case OPTION_POPUP_GLOBAL:
-            setting = NSMixedState;
+            setting = TR_RATIOLIMIT_GLOBAL;
             break;
         default:
             return;
     }
     
-    Torrent * torrent;
-    NSEnumerator * enumerator = [fTorrents objectEnumerator];
-    while ((torrent = [enumerator nextObject]))
+    for (Torrent * torrent in fTorrents)
         [torrent setRatioSetting: setting];
     
-    BOOL single = setting == NSOnState;
     [fRatioLimitField setHidden: !single];
     if (single)
     {
         [fRatioLimitField selectText: self];
-        [[self window] makeKeyAndOrderFront:self];
+        [[self window] makeKeyAndOrderFront: self];
     }
-    
-    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
 }
 
 - (void) setRatioLimit: (id) sender
 {
     CGFloat limit = [sender floatValue];
     
-    Torrent * torrent;
-    NSEnumerator * enumerator = [fTorrents objectEnumerator];
-    while ((torrent = [enumerator nextObject]))
+    for (Torrent * torrent in fTorrents)
         [torrent setRatioLimit: limit];
 }
 
+- (void) setPriority: (id) sender
+{
+    tr_priority_t priority;
+    switch ([sender indexOfSelectedItem])
+    {
+        case OPTION_POPUP_PRIORITY_HIGH:
+            priority = TR_PRI_HIGH;
+            break;
+        case OPTION_POPUP_PRIORITY_NORMAL:
+            priority = TR_PRI_NORMAL;
+            break;
+        case OPTION_POPUP_PRIORITY_LOW:
+            priority = TR_PRI_LOW;
+            break;
+        default:
+            return;
+    }
+    
+    for (Torrent * torrent in fTorrents)
+        [torrent setPriority: priority];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
+}
+
 - (void) setPeersConnectLimit: (id) sender
 {
     NSInteger limit = [sender intValue];
     
-    Torrent * torrent;
-    NSEnumerator * enumerator = [fTorrents objectEnumerator];
-    while ((torrent = [enumerator nextObject]))
+    for (Torrent * torrent in fTorrents)
         [torrent setMaxPeerConnect: limit];
 }
 
-
 - (BOOL) control: (NSControl *) control textShouldBeginEditing: (NSText *) fieldEditor
 {
     [fInitialString release];
@@ -1315,9 +1370,7 @@ typedef enum
     
     uint64_t have = 0, haveVerified = 0, downloadedTotal = 0, uploadedTotal = 0, failedHash = 0;
     NSDate * lastActivity = nil;
-    Torrent * torrent;
-    NSEnumerator * enumerator = [fTorrents objectEnumerator];
-    while ((torrent = [enumerator nextObject]))
+    for (Torrent * torrent in fTorrents)
     {
         have += [torrent haveTotal];
         haveVerified += [torrent haveVerified];
@@ -1350,7 +1403,7 @@ typedef enum
     
     if (numberSelected == 1)
     {
-        torrent = [fTorrents objectAtIndex: 0];
+        Torrent * torrent = [fTorrents objectAtIndex: 0];
         
         [fStateField setStringValue: [torrent stateString]];
         
@@ -1472,7 +1525,7 @@ typedef enum
         
         if (total > 0)
         {
-            NSMutableArray * components = [NSMutableArray arrayWithCapacity: 4];
+            NSMutableArray * components = [NSMutableArray arrayWithCapacity: 5];
             NSInteger count;
             if ((count = [torrent totalPeersTracker]) > 0)
                 [components addObject: [NSString stringWithFormat:
@@ -1480,12 +1533,15 @@ typedef enum
             if ((count = [torrent totalPeersIncoming]) > 0)
                 [components addObject: [NSString stringWithFormat:
                                         NSLocalizedString(@"%d incoming", "Inspector -> Peers tab -> peers"), count]];
-            if ((count = [torrent totalPeersPex]) > 0)
-                [components addObject: [NSString stringWithFormat:
-                                        NSLocalizedString(@"%d PEX", "Inspector -> Peers tab -> peers"), count]];
             if ((count = [torrent totalPeersCache]) > 0)
                 [components addObject: [NSString stringWithFormat:
                                         NSLocalizedString(@"%d cache", "Inspector -> Peers tab -> peers"), count]];
+            if ((count = [torrent totalPeersPex]) > 0)
+                [components addObject: [NSString stringWithFormat:
+                                        NSLocalizedString(@"%d PEX", "Inspector -> Peers tab -> peers"), count]];
+            if ((count = [torrent totalPeersDHT]) > 0)
+                [components addObject: [NSString stringWithFormat:
+                                        NSLocalizedString(@"%d DHT", "Inspector -> Peers tab -> peers"), count]];
             
             connected = [connected stringByAppendingFormat: @": %@", [components componentsJoinedByString: @", "]];
         }
@@ -1645,7 +1701,7 @@ typedef enum
     [[self window] makeKeyWindow];
     
     NSUInteger index = 1;
-    if ([[fTorrents objectAtIndex: 0] hasAddedTrackers])
+    if ([fTrackers count] > 0 && [[fTorrents objectAtIndex: 0] hasAddedTrackers])
     {
         for (; index < [fTrackers count]; index++)
             if ([[fTrackers objectAtIndex: index] isKindOfClass: [NSNumber class]])
@@ -1668,7 +1724,7 @@ typedef enum
     NSUInteger i = 0, numberBuiltIn = 0;
     while (i < [fTrackers count])
     {
-        BOOL builtIn = i != 0 || [[fTrackers objectAtIndex: i] intValue] != 0;
+        const BOOL builtIn = i != 0 || [[fTrackers objectAtIndex: i] intValue] != 0;
         
         //if a group is selected, remove all trackers in the group
         if ([indexes containsIndex: i])
@@ -1701,6 +1757,7 @@ typedef enum
         }
     }
     
+    #warning show warning and allow?
     if ([fTrackers count] == [indexes count])
     {
         NSBeep();
@@ -1733,14 +1790,10 @@ typedef enum
         [alert addButtonWithTitle: NSLocalizedString(@"Remove", "Remove built-in tracker alert -> button")];
         [alert addButtonWithTitle: NSLocalizedString(@"Cancel", "Remove built-in tracker alert -> button")];
         
-        BOOL onLeopard = [NSApp isOnLeopardOrBetter];
-        if (onLeopard)
-            [alert setShowsSuppressionButton: YES];
-        else
-            [alert addButtonWithTitle: NSLocalizedString(@"Don't Alert Again", "Remove built-in tracker alert -> button")];
+        [alert setShowsSuppressionButton: YES];
 
         NSInteger result = [alert runModal];
-        if ((onLeopard ? [[alert suppressionButton] state] == NSOnState : result == NSAlertThirdButtonReturn))
+        if ([[alert suppressionButton] state] == NSOnState)
             [[NSUserDefaults standardUserDefaults] setBool: NO forKey: @"WarningRemoveBuiltInTracker"];
         [alert release];
         
