@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: TorrentTableView.m 5734 2008-05-01 02:20:53Z livings124 $
+ * $Id: TorrentTableView.m 8261 2009-04-20 03:36:35Z livings124 $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) 2005-2009 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,37 +25,33 @@
 #import "TorrentTableView.h"
 #import "TorrentCell.h"
 #import "Torrent.h"
-#import "NSApplicationAdditions.h"
+#import "TorrentGroup.h"
+#import "FileListNode.h"
+#import "QuickLookController.h"
 
-#define MAX_GROUP (INT_MAX-10)
+#define MAX_GROUP 999999
 
 #define ACTION_MENU_GLOBAL_TAG 101
 #define ACTION_MENU_UNLIMITED_TAG 102
 #define ACTION_MENU_LIMIT_TAG 103
 
-#define PIECE_CHANGE 0.1
-#define PIECE_TIME 0.005
+#define ACTION_MENU_PRIORITY_HIGH_TAG 101
+#define ACTION_MENU_PRIORITY_NORMAL_TAG 102
+#define ACTION_MENU_PRIORITY_LOW_TAG 103
 
-#define GROUP_SPEED_IMAGE_COLUMN_WIDTH 8.0
-#define GROUP_RATIO_IMAGE_COLUMN_WIDTH 10.0
+#define GROUP_SPEED_IMAGE_COLUMN_WIDTH 8.0f
+#define GROUP_RATIO_IMAGE_COLUMN_WIDTH 10.0f
+
+#define TOGGLE_PROGRESS_SECONDS 0.175
 
 @interface TorrentTableView (Private)
 
-- (BOOL) pointInControlRect: (NSPoint) point;
-- (BOOL) pointInRevealRect: (NSPoint) point;
-- (BOOL) pointInActionRect: (NSPoint) point;
-
-- (BOOL) pointInProgressRect: (NSPoint) point;
-- (BOOL) pointInMinimalStatusRect: (NSPoint) point;
-
 - (BOOL) pointInGroupStatusRect: (NSPoint) point;
 
 - (void) setGroupStatusColumns;
 
 - (void) createFileMenu: (NSMenu *) menu forFiles: (NSArray *) files;
 
-- (void) resizePiecesBarIncrement;
-
 @end
 
 @implementation TorrentTableView
@@ -68,25 +64,6 @@
         
         fTorrentCell = [[TorrentCell alloc] init];
         
-        if (![NSApp isOnLeopardOrBetter])
-        {
-            NSTableColumn * groupColumn = [self tableColumnWithIdentifier: @"Group"];
-            [self setOutlineTableColumn: groupColumn];
-            
-            //remove all unnecessary columns
-            int i;
-            for (i = [[self tableColumns] count]-1; i >= 0; i--)
-            {
-                NSTableColumn * column = [[self tableColumns] objectAtIndex: i];
-                if (column != groupColumn)
-                    [self removeTableColumn: column];
-            }
-            
-            [self sizeLastColumnToFit];
-            
-            [groupColumn setDataCell: fTorrentCell];
-        }
-        
         NSData * groupData = [fDefaults dataForKey: @"CollapsedGroups"];
         if (groupData)
             fCollapsedGroups = [[NSUnarchiver unarchiveObjectWithData: groupData] mutableCopy];
@@ -100,7 +77,7 @@
         
         [self setDelegate: self];
         
-        fPiecesBarPercent = [fDefaults boolForKey: @"PiecesBar"] ? 1.0 : 0.0;
+        fPiecesBarPercent = [fDefaults boolForKey: @"PiecesBar"] ? 1.0f : 0.0f;
     }
     
     return self;
@@ -110,7 +87,7 @@
 {
     [fCollapsedGroups release];
     
-    [fPiecesBarTimer invalidate];
+    [fPiecesBarAnimation release];
     [fMenuTorrent release];
     
     [fSelectedValues release];
@@ -126,17 +103,17 @@
     [self setGroupStatusColumns];
 }
 
-- (BOOL) isGroupCollapsed: (int) value
+- (BOOL) isGroupCollapsed: (NSInteger) value
 {
-    if (value < 0)
+    if (value == -1)
         value = MAX_GROUP;
     
     return [fCollapsedGroups containsIndex: value];
 }
 
-- (void) removeCollapsedGroup: (int) value
+- (void) removeCollapsedGroup: (NSInteger) value
 {
-    if (value < 0)
+    if (value == -1)
         value = MAX_GROUP;
     
     [fCollapsedGroups removeIndex: value];
@@ -178,18 +155,16 @@
     {
         [cell setRepresentedObject: item];
         
-        int row = [self rowForItem: item];
-        if ([NSApp isOnLeopardOrBetter])
-        {
-            [cell setControlHover: row == fMouseControlRow];
-            [cell setRevealHover: row == fMouseRevealRow];
-            [cell setActionHover: row == fMouseActionRow];
-        }
+        const NSInteger row = [self rowForItem: item];
+        [cell setControlHover: row == fMouseControlRow];
+        [cell setRevealHover: row == fMouseRevealRow];
+        [cell setActionHover: row == fMouseActionRow];
         [cell setActionPushed: row == fActionPushedRow];
     }
     else
     {
-        if ([[tableColumn identifier] isEqualToString: @"UL Image"] || [[tableColumn identifier] isEqualToString: @"DL Image"])
+        NSString * ident = [tableColumn identifier];
+        if ([ident isEqualToString: @"UL Image"] || [ident isEqualToString: @"DL Image"])
         {
             //ensure arrows are white only when selected
             NSImage * image = [cell image];
@@ -206,7 +181,7 @@
 
 - (NSRect) frameOfCellAtColumn: (NSInteger) column row: (NSInteger) row
 {
-    if (column == -1 || ![NSApp isOnLeopardOrBetter])
+    if (column == -1)
         return [self rectOfRow: row];
     else
     {
@@ -214,7 +189,7 @@
         
         //adjust placement for proper vertical alignment
         if (column == [self columnWithIdentifier: @"Group"])
-            rect.size.height -= 1.0;
+            rect.size.height -= 1.0f;
         
         return rect;
     }
@@ -237,7 +212,7 @@
                 : NSLocalizedString(@"Upload speed", "Torrent table -> group row -> tooltip");
     else if (ident)
     {
-        int count = [[item objectForKey: @"Torrents"] count];
+        NSInteger count = [[item torrents] count];
         if (count == 1)
             return NSLocalizedString(@"1 transfer", "Torrent table -> group row -> tooltip");
         else
@@ -257,17 +232,14 @@
         return;
     
     NSPoint mouseLocation = [self convertPoint: [[self window] convertScreenToBase: [NSEvent mouseLocation]] fromView: nil];
-    
-    NSUInteger row;
-    for (row = rows.location; row < NSMaxRange(rows); row++)
+    for (NSUInteger row = rows.location; row < NSMaxRange(rows); row++)
     {
         if (![[self itemAtRow: row] isKindOfClass: [Torrent class]])
             continue;
         
         NSDictionary * userInfo = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: row] forKey: @"Row"];
         TorrentCell * cell = (TorrentCell *)[self preparedCellAtColumn: -1 row: row];
-        [cell addTrackingAreasForView: self inRect: [self rectOfRow: row] withUserInfo: userInfo
-                mouseLocation: mouseLocation];
+        [cell addTrackingAreasForView: self inRect: [self rectOfRow: row] withUserInfo: userInfo mouseLocation: mouseLocation];
     }
 }
 
@@ -277,33 +249,28 @@
     fMouseRevealRow = -1;
     fMouseActionRow = -1;
     
-    if (![NSApp isOnLeopardOrBetter])
-        return;
-    
-    NSEnumerator * enumerator = [[self trackingAreas] objectEnumerator];
-    NSTrackingArea * area;
-    while ((area = [enumerator nextObject]))
+    for (NSTrackingArea * area in [self trackingAreas])
     {
         if ([area owner] == self && [[area userInfo] objectForKey: @"Row"])
             [self removeTrackingArea: area];
     }
 }
 
-- (void) setControlButtonHover: (int) row
+- (void) setControlButtonHover: (NSInteger) row
 {
     fMouseControlRow = row;
     if (row >= 0)
         [self setNeedsDisplayInRect: [self rectOfRow: row]];
 }
 
-- (void) setRevealButtonHover: (int) row
+- (void) setRevealButtonHover: (NSInteger) row
 {
     fMouseRevealRow = row;
     if (row >= 0)
         [self setNeedsDisplayInRect: [self rectOfRow: row]];
 }
 
-- (void) setActionButtonHover: (int) row
+- (void) setActionButtonHover: (NSInteger) row
 {
     fMouseActionRow = row;
     if (row >= 0)
@@ -317,7 +284,7 @@
     NSNumber * row;
     if ((row = [dict objectForKey: @"Row"]))
     {
-        int rowVal = [row intValue];
+        NSInteger rowVal = [row intValue];
         NSString * type = [dict objectForKey: @"Type"];
         if ([type isEqualToString: @"Action"])
             fMouseActionRow = rowVal;
@@ -357,23 +324,31 @@
 
 - (void) outlineViewItemDidExpand: (NSNotification *) notification
 {
-    int value = [[[[notification userInfo] objectForKey: @"NSObject"] objectForKey: @"Group"] intValue];
-    [fCollapsedGroups removeIndex: value >= 0 ? value : MAX_GROUP];
+    NSInteger value = [[[notification userInfo] objectForKey: @"NSObject"] groupIndex];
+    if (value < 0)
+        value = MAX_GROUP;
     
-    [[NSNotificationCenter defaultCenter] postNotificationName: @"OutlineExpandCollapse" object: self];
+    if ([fCollapsedGroups containsIndex: value])
+    {
+        [fCollapsedGroups removeIndex: value];
+        [[NSNotificationCenter defaultCenter] postNotificationName: @"OutlineExpandCollapse" object: self];
+    }
 }
 
 - (void) outlineViewItemDidCollapse: (NSNotification *) notification
 {
-    int value = [[[[notification userInfo] objectForKey: @"NSObject"] objectForKey: @"Group"] intValue];
-    [fCollapsedGroups addIndex: value >= 0 ? value : MAX_GROUP];
+    NSInteger value = [[[notification userInfo] objectForKey: @"NSObject"] groupIndex];
+    if (value < 0)
+        value = MAX_GROUP;
     
+    [fCollapsedGroups addIndex: value];
     [[NSNotificationCenter defaultCenter] postNotificationName: @"OutlineExpandCollapse" object: self];
 }
 
 - (void) mouseDown: (NSEvent *) event
 {
     NSPoint point = [self convertPoint: [event locationInWindow] fromView: nil];
+    const NSInteger row = [self rowAtPoint: point];
     
     //check to toggle group status before anything else
     if ([self pointInGroupStatusRect: point])
@@ -384,8 +359,7 @@
         return;
     }
     
-    BOOL pushed = [self pointInControlRect: point] || [self pointInRevealRect: point] || [self pointInActionRect: point]
-                    || [self pointInProgressRect: point] || [self pointInMinimalStatusRect: point];
+    const BOOL pushed = row != -1 && (fMouseActionRow == row || fMouseRevealRow == row || fMouseControlRow == row);
     
     //if pushing a button, don't change the selected rows
     if (pushed)
@@ -397,10 +371,8 @@
     fSelectedValues = nil;
     
     //avoid weird behavior when showing menu by doing this after mouse down
-    if ([self pointInActionRect: point])
+    if (row != -1 && fMouseActionRow == row)
     {
-        int row = [self rowAtPoint: point];
-        
         fActionPushedRow = row;
         [self setNeedsDisplayInRect: [self rectOfRow: row]]; //ensure button is pushed down
         
@@ -412,7 +384,6 @@
     else if (!pushed && [event clickCount] == 2) //double click
     {
         id item = nil;
-        int row = [self rowAtPoint: point];
         if (row != -1)
             item = [self itemAtRow: row];
         
@@ -433,24 +404,24 @@
 {
     NSMutableIndexSet * indexSet = [NSMutableIndexSet indexSet];
     
-    NSEnumerator * enumerator = [values objectEnumerator];
-    id item;
-    while ((item = [enumerator nextObject]))
+    for (id item in values)
     {
         if ([item isKindOfClass: [Torrent class]])
         {
-            NSUInteger index = [self rowForItem: item];
+            NSInteger index = [self rowForItem: item];
             if (index != -1)
                 [indexSet addIndex: index];
         }
         else
         {
-            NSNumber * group = [item objectForKey: @"Group"];
-            int i;
-            for (i = 0; i < [self numberOfRows]; i++)
+            NSInteger group = [item groupIndex];
+            for (NSInteger i = 0; i < [self numberOfRows]; i++)
             {
+                if ([indexSet containsIndex: i])
+                    continue;
+                
                 id tableItem = [self itemAtRow: i];
-                if (![tableItem isKindOfClass: [Torrent class]] && [group isEqualToNumber: [tableItem objectForKey: @"Group"]])
+                if (![tableItem isKindOfClass: [Torrent class]] && group == [tableItem groupIndex])
                 {
                     [indexSet addIndex: i];
                     break;
@@ -467,8 +438,7 @@
     NSIndexSet * selectedIndexes = [self selectedRowIndexes];
     NSMutableArray * values = [NSMutableArray arrayWithCapacity: [selectedIndexes count]];
     
-    NSUInteger i;
-    for (i = [selectedIndexes firstIndex]; i != NSNotFound; i = [selectedIndexes indexGreaterThanIndex: i])
+    for (NSUInteger i = [selectedIndexes firstIndex]; i != NSNotFound; i = [selectedIndexes indexGreaterThanIndex: i])
         [values addObject: [self itemAtRow: i]];
     
     return values;
@@ -477,19 +447,20 @@
 - (NSArray *) selectedTorrents
 {
     NSIndexSet * selectedIndexes = [self selectedRowIndexes];
-    NSMutableArray * torrents = [NSMutableArray array];
+    NSMutableArray * torrents = [NSMutableArray arrayWithCapacity: [selectedIndexes count]]; //take a shot at guessing capacity
     
-    NSUInteger i;
-    for (i = [selectedIndexes firstIndex]; i != NSNotFound; i = [selectedIndexes indexGreaterThanIndex: i])
+    for (NSUInteger i = [selectedIndexes firstIndex]; i != NSNotFound; i = [selectedIndexes indexGreaterThanIndex: i])
     {
         id item = [self itemAtRow: i];
         if ([item isKindOfClass: [Torrent class]])
-        {
-            if (![torrents containsObject: item])
-                [torrents addObject: item];
-        }
+            [torrents addObject: item];
         else
-            [torrents addObjectsFromArray: [item objectForKey: @"Torrents"]];
+        {
+            NSArray * groupTorrents = [item torrents];
+            [torrents addObjectsFromArray: groupTorrents];
+            if ([self isItemExpanded: item])
+                i +=[groupTorrents count];
+        }
     }
     
     return torrents;
@@ -497,7 +468,7 @@
 
 - (NSMenu *) menuForEvent: (NSEvent *) event
 {
-    int row = [self rowAtPoint: [self convertPoint: [event locationInWindow] fromView: nil]];
+    NSInteger row = [self rowAtPoint: [self convertPoint: [event locationInWindow] fromView: nil]];
     if (row >= 0)
     {
         if (![self isRowSelected: row])
@@ -521,11 +492,28 @@
 //option-command-f will focus the filter bar's search field
 - (void) keyDown: (NSEvent *) event
 {
-    if ([[event charactersIgnoringModifiers] isEqualToString: @"f"] && [event modifierFlags] & NSAlternateKeyMask
-        && [event modifierFlags] & NSCommandKeyMask)
+    const unichar firstChar = [[event charactersIgnoringModifiers] characterAtIndex: 0];
+    
+    if (firstChar == 'f' && [event modifierFlags] & NSAlternateKeyMask && [event modifierFlags] & NSCommandKeyMask)
         [fController focusFilterField];
     else
+    {
+        //handle quicklook
+        if (firstChar == ' ')
+            [[QuickLookController quickLook] toggleQuickLook];
+        else if (firstChar == NSRightArrowFunctionKey)
+            [[QuickLookController quickLook] pressRight];
+        else if (firstChar == NSLeftArrowFunctionKey)
+            [[QuickLookController quickLook] pressLeft];
+        else;
+        
         [super keyDown: event];
+    }
+}
+
+- (NSRect) iconRectForRow: (NSInteger) row
+{
+    return [fTorrentCell iconRectForBounds: [self rectOfRow: row]];
 }
 
 - (void) paste: (id) sender
@@ -562,20 +550,23 @@
 
 - (void) displayTorrentMenuForEvent: (NSEvent *) event
 {
-    int row = [self rowAtPoint: [self convertPoint: [event locationInWindow] fromView: nil]];
+    const NSInteger row = [self rowAtPoint: [self convertPoint: [event locationInWindow] fromView: nil]];
     if (row < 0)
         return;
     
-    NSInteger numberOfNonFileItems = [fActionMenu numberOfItems];
+    const NSInteger numberOfNonFileItems = [fActionMenu numberOfItems];
     
     //update file action menu
     fMenuTorrent = [[self itemAtRow: row] retain];
     [self createFileMenu: fActionMenu forFiles: [fMenuTorrent fileList]];
     
+    //update global limit check
+    [fGlobalLimitItem setState: [fMenuTorrent usesGlobalSpeedLimit] ? NSOnState : NSOffState];
+    
     //place menu below button
     NSRect rect = [fTorrentCell iconRectForBounds: [self rectOfRow: row]];
     NSPoint location = rect.origin;
-    location.y += rect.size.height + 5.0;
+    location.y += rect.size.height + 5.0f;
     location = [self convertPoint: location toView: nil];
     
     NSEvent * newEvent = [NSEvent mouseEventWithType: [event type] location: location
@@ -584,8 +575,7 @@
     
     [NSMenu popUpContextMenu: fActionMenu withEvent: newEvent forView: self];
     
-    NSInteger i;
-    for (i = [fActionMenu numberOfItems]-1; i >= numberOfNonFileItems; i--)
+    for (NSInteger i = [fActionMenu numberOfItems]-1; i >= numberOfNonFileItems; i--)
         [fActionMenu removeItemAtIndex: i];
     
     [fMenuTorrent release];
@@ -601,12 +591,11 @@
     if (menu == fUploadMenu || menu == fDownloadMenu)
     {
         NSMenuItem * item;
-        if ([menu numberOfItems] == 4)
+        if ([menu numberOfItems] == 3)
         {
-            const int speedLimitActionValue[] = { 0, 5, 10, 20, 30, 40, 50, 75, 100, 150, 200, 250, 500, 750, -1 };
+            const NSInteger speedLimitActionValue[] = { 0, 5, 10, 20, 30, 40, 50, 75, 100, 150, 200, 250, 500, 750, -1 };
             
-            int i;
-            for (i = 0; speedLimitActionValue[i] != -1; i++)
+            for (NSInteger i = 0; speedLimitActionValue[i] != -1; i++)
             {
                 item = [[NSMenuItem alloc] initWithTitle: [NSString stringWithFormat: NSLocalizedString(@"%d KB/s",
                         "Action menu -> upload/download limit"), speedLimitActionValue[i]] action: @selector(setQuickLimit:)
@@ -618,29 +607,25 @@
             }
         }
         
-        BOOL upload = menu == fUploadMenu;
-        int mode = [fMenuTorrent speedMode: upload];
+        const BOOL upload = menu == fUploadMenu;
+        const BOOL limit = [fMenuTorrent usesSpeedLimit: upload];
         
         item = [menu itemWithTag: ACTION_MENU_LIMIT_TAG];
-        [item setState: mode == TR_SPEEDLIMIT_SINGLE ? NSOnState : NSOffState];
+        [item setState: limit ? NSOnState : NSOffState];
         [item setTitle: [NSString stringWithFormat: NSLocalizedString(@"Limit (%d KB/s)",
                             "torrent action menu -> upload/download limit"), [fMenuTorrent speedLimit: upload]]];
         
         item = [menu itemWithTag: ACTION_MENU_UNLIMITED_TAG];
-        [item setState: mode == TR_SPEEDLIMIT_UNLIMITED ? NSOnState : NSOffState];
-        
-        item = [menu itemWithTag: ACTION_MENU_GLOBAL_TAG];
-        [item setState: mode == TR_SPEEDLIMIT_GLOBAL ? NSOnState : NSOffState];
+        [item setState: !limit ? NSOnState : NSOffState];
     }
     else if (menu == fRatioMenu)
     {
         NSMenuItem * item;
         if ([menu numberOfItems] == 4)
         {
-            const float ratioLimitActionValue[] = { 0.25, 0.5, 0.75, 1.0, 1.5, 2.0, 3.0, -1.0 };
+            const CGFloat ratioLimitActionValue[] = { 0.25f, 0.5f, 0.75f, 1.0f, 1.5f, 2.0f, 3.0f, -1.0f };
             
-            int i;
-            for (i = 0; ratioLimitActionValue[i] != -1.0; i++)
+            for (NSInteger i = 0; ratioLimitActionValue[i] != -1.0f; i++)
             {
                 item = [[NSMenuItem alloc] initWithTitle: [NSString localizedStringWithFormat: @"%.2f", ratioLimitActionValue[i]]
                         action: @selector(setQuickRatio:) keyEquivalent: @""];
@@ -651,75 +636,145 @@
             }
         }
         
-        int mode = [fMenuTorrent ratioSetting];
+        const tr_ratiolimit mode = [fMenuTorrent ratioSetting];
         
         item = [menu itemWithTag: ACTION_MENU_LIMIT_TAG];
-        [item setState: mode == NSOnState ? NSOnState : NSOffState];
+        [item setState: mode == TR_RATIOLIMIT_SINGLE ? NSOnState : NSOffState];
         [item setTitle: [NSString localizedStringWithFormat: NSLocalizedString(@"Stop at Ratio (%.2f)",
             "torrent action menu -> ratio stop"), [fMenuTorrent ratioLimit]]];
         
         item = [menu itemWithTag: ACTION_MENU_UNLIMITED_TAG];
-        [item setState: mode == NSOffState ? NSOnState : NSOffState];
+        [item setState: mode == TR_RATIOLIMIT_UNLIMITED ? NSOnState : NSOffState];
         
         item = [menu itemWithTag: ACTION_MENU_GLOBAL_TAG];
-        [item setState: mode == NSMixedState ? NSOnState : NSOffState];
+        [item setState: mode == TR_RATIOLIMIT_GLOBAL ? NSOnState : NSOffState];
     }
-    else  //assume the menu is part of the file list
+    else if (menu == fPriorityMenu)
+    {
+        const tr_priority_t priority = [fMenuTorrent priority];
+        
+        NSMenuItem * item = [menu itemWithTag: ACTION_MENU_PRIORITY_HIGH_TAG];
+        [item setState: priority == TR_PRI_HIGH ? NSOnState : NSOffState];
+        
+        item = [menu itemWithTag: ACTION_MENU_PRIORITY_NORMAL_TAG];
+        [item setState: priority == TR_PRI_NORMAL ? NSOnState : NSOffState];
+        
+        item = [menu itemWithTag: ACTION_MENU_PRIORITY_LOW_TAG];
+        [item setState: priority == TR_PRI_LOW ? NSOnState : NSOffState];
+    }
+    else //assume the menu is part of the file list
     {
         if ([menu numberOfItems] > 0)
             return;
         
         NSMenu * supermenu = [menu supermenu];
-        [self createFileMenu: menu forFiles: [[[supermenu itemAtIndex: [supermenu indexOfItemWithSubmenu: menu]]
-                                                    representedObject] objectForKey: @"Children"]];
+        [self createFileMenu: menu forFiles: [(FileListNode *)[[supermenu itemAtIndex: [supermenu indexOfItemWithSubmenu: menu]]
+                                                representedObject] children]];
     }
 }
 
+//alternating rows - first row after group row is white
+- (void) highlightSelectionInClipRect: (NSRect) clipRect
+{
+    NSColor * altColor = [[NSColor controlAlternatingRowBackgroundColors] objectAtIndex: 1];
+    [altColor set];
+    
+    NSRect visibleRect = clipRect;
+    NSRange rows = [self rowsInRect: visibleRect];
+    BOOL start = YES;
+    
+    if (rows.length > 0)
+    {
+        //determine what the first row color should be
+        if ([[self itemAtRow: rows.location] isKindOfClass: [Torrent class]])
+        {
+            for (NSInteger i = rows.location-1; i>=0; i--)
+            {
+                if (![[self itemAtRow: i] isKindOfClass: [Torrent class]])
+                    break;
+                start = !start;
+            }
+        }
+        else
+        {
+            rows.location++;
+            rows.length--;
+        }
+        
+        NSInteger i;
+        for (i = rows.location; i < NSMaxRange(rows); i++)
+        {
+            if (![[self itemAtRow: i] isKindOfClass: [Torrent class]])
+            {
+                start = YES;
+                continue;
+            }
+            
+            if (!start && ![self isRowSelected: i])
+                NSRectFill([self rectOfRow: i]);
+            
+            start = !start;
+        }
+        
+        CGFloat newY = NSMaxY([self rectOfRow: i-1]);
+        visibleRect.size.height -= newY - visibleRect.origin.y;
+        visibleRect.origin.y = newY;
+    }
+        
+    //remaining visible rows continue alternating
+    const CGFloat height = [self rowHeight] + [self intercellSpacing].height;
+    const NSInteger numberOfRects = ceil(visibleRect.size.height / height);
+    
+    visibleRect.size.height = height;
+    if (start)
+        visibleRect.origin.y += height;
+    
+    for (NSInteger i = start ? 1 : 0; i < numberOfRects; i += 2)
+    {
+        NSRectFill(visibleRect);
+        visibleRect.origin.y += 2.0 * height;
+    }
+    
+    [super highlightSelectionInClipRect: clipRect];
+}
+
 - (void) setQuickLimitMode: (id) sender
 {
-    int mode;
-    switch ([sender tag])
-    {
-        case ACTION_MENU_UNLIMITED_TAG:
-            mode = TR_SPEEDLIMIT_UNLIMITED;
-            break;
-        case ACTION_MENU_LIMIT_TAG:
-            mode = TR_SPEEDLIMIT_SINGLE;
-            break;
-        case ACTION_MENU_GLOBAL_TAG:
-            mode = TR_SPEEDLIMIT_GLOBAL;
-            break;
-        default:
-            return;
-    }
-    
-    [fMenuTorrent setSpeedMode: mode upload: [sender menu] == fUploadMenu];
+    const BOOL limit = [sender tag] == ACTION_MENU_LIMIT_TAG;
+    [fMenuTorrent setUseSpeedLimit: limit upload: [sender menu] == fUploadMenu];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptions" object: nil];
 }
 
 - (void) setQuickLimit: (id) sender
 {
-    BOOL upload = [sender menu] == fUploadMenu;
-    [fMenuTorrent setSpeedMode: TR_SPEEDLIMIT_SINGLE upload: upload];
+    const BOOL upload = [sender menu] == fUploadMenu;
+    [fMenuTorrent setUseSpeedLimit: YES upload: upload];
     [fMenuTorrent setSpeedLimit: [[sender representedObject] intValue] upload: upload];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptions" object: nil];
 }
 
+- (void) setGlobalLimit: (id) sender
+{
+    [fMenuTorrent setUseGlobalSpeedLimit: [sender state] != NSOnState];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptions" object: nil];
+}
+
 - (void) setQuickRatioMode: (id) sender
 {
-    int mode;
+    tr_ratiolimit mode;
     switch ([sender tag])
     {
         case ACTION_MENU_UNLIMITED_TAG:
-            mode = NSOffState;
+            mode = TR_RATIOLIMIT_UNLIMITED;
             break;
         case ACTION_MENU_LIMIT_TAG:
-            mode = NSOnState;
+            mode = TR_RATIOLIMIT_SINGLE;
             break;
         case ACTION_MENU_GLOBAL_TAG:
-            mode = NSMixedState;
+            mode = TR_RATIOLIMIT_GLOBAL;
             break;
         default:
             return;
@@ -732,15 +787,38 @@
 
 - (void) setQuickRatio: (id) sender
 {
-    [fMenuTorrent setRatioSetting: NSOnState];
+    [fMenuTorrent setRatioSetting: TR_RATIOLIMIT_SINGLE];
     [fMenuTorrent setRatioLimit: [[sender representedObject] floatValue]];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptions" object: nil];
 }
 
+- (void) setPriority: (id) sender
+{
+    tr_priority_t priority;
+    switch ([sender tag])
+    {
+        case ACTION_MENU_PRIORITY_HIGH_TAG:
+            priority = TR_PRI_HIGH;
+            break;
+        case ACTION_MENU_PRIORITY_NORMAL_TAG:
+            priority = TR_PRI_NORMAL;
+            break;
+        case ACTION_MENU_PRIORITY_LOW_TAG:
+            priority = TR_PRI_LOW;
+            break;
+        default:
+            return;
+    }
+    
+    [fMenuTorrent setPriority: priority];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
+}
+
 - (void) checkFile: (id) sender
 {
-    NSIndexSet * indexSet = [[sender representedObject] objectForKey: @"Indexes"];
+    NSIndexSet * indexSet = [(FileListNode *)[sender representedObject] indexes];
     [fMenuTorrent setFileCheckState: [sender state] != NSOnState ? NSOnState : NSOffState forIndexes: indexSet];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateStats" object: nil];
@@ -753,18 +831,45 @@
 
 - (void) togglePiecesBar
 {
-    [self resizePiecesBarIncrement];
+    //stop previous animation
+    if (fPiecesBarAnimation)
+        [fPiecesBarAnimation release];
     
-    if (!fPiecesBarTimer)
+    NSMutableArray * progressMarks = [NSMutableArray arrayWithCapacity: 16];
+    for (NSAnimationProgress i = 0.0625f; i <= 1.0f; i += 0.0625f)
+        [progressMarks addObject: [NSNumber numberWithFloat: i]];
+    
+    fPiecesBarAnimation = [[NSAnimation alloc] initWithDuration: TOGGLE_PROGRESS_SECONDS animationCurve: NSAnimationEaseIn];
+    [fPiecesBarAnimation setAnimationBlockingMode: NSAnimationNonblocking];
+    [fPiecesBarAnimation setProgressMarks: progressMarks];
+    [fPiecesBarAnimation setDelegate: self];
+    
+    [fPiecesBarAnimation startAnimation];
+}
+
+- (void) animationDidEnd: (NSAnimation *) animation
+{
+    if (animation == fPiecesBarAnimation)
     {
-        fPiecesBarTimer = [NSTimer scheduledTimerWithTimeInterval: PIECE_TIME target: self
-                            selector: @selector(resizePiecesBarIncrement) userInfo: nil repeats: YES];
-        [[NSRunLoop currentRunLoop] addTimer: fPiecesBarTimer forMode: NSModalPanelRunLoopMode];
-        [[NSRunLoop currentRunLoop] addTimer: fPiecesBarTimer forMode: NSEventTrackingRunLoopMode];
+        [fPiecesBarAnimation release];
+        fPiecesBarAnimation = nil;
     }
 }
 
-- (float) piecesBarPercent
+- (void) animation: (NSAnimation *) animation didReachProgressMark: (NSAnimationProgress) progress
+{
+    if (animation == fPiecesBarAnimation)
+    {
+        if ([fDefaults boolForKey: @"PiecesBar"])
+            fPiecesBarPercent = progress;
+        else
+            fPiecesBarPercent = 1.0f - progress;
+        
+        [self reloadData];
+    }
+}
+
+- (CGFloat) piecesBarPercent
 {
     return fPiecesBarPercent;
 }
@@ -773,71 +878,9 @@
 
 @implementation TorrentTableView (Private)
 
-- (BOOL) pointInControlRect: (NSPoint) point
-{
-    int row = [self rowAtPoint: point];
-    if (row < 0 || ![[self itemAtRow: row] isKindOfClass: [Torrent class]])
-        return NO;
-    
-    return NSPointInRect(point, [fTorrentCell controlButtonRectForBounds: [self rectOfRow: row]]);
-}
-
-- (BOOL) pointInRevealRect: (NSPoint) point
-{
-    int row = [self rowAtPoint: point];
-    if (row < 0 || ![[self itemAtRow: row] isKindOfClass: [Torrent class]])
-        return NO;
-    
-    return NSPointInRect(point, [fTorrentCell revealButtonRectForBounds: [self rectOfRow: row]]);
-}
-
-- (BOOL) pointInActionRect: (NSPoint) point
-{
-    int row = [self rowAtPoint: point];
-    if (row < 0 || ![[self itemAtRow: row] isKindOfClass: [Torrent class]])
-        return NO;
-    
-    return NSPointInRect(point, [fTorrentCell iconRectForBounds: [self rectOfRow: row]]);
-}
-
-- (BOOL) pointInProgressRect: (NSPoint) point
-{
-    int row = [self rowAtPoint: point];
-    if (row < 0 || ![[self itemAtRow: row] isKindOfClass: [Torrent class]] || [fDefaults boolForKey: @"SmallView"]
-        || ![[self itemAtRow: row] folder])
-        return NO;
-    
-    TorrentCell * cell;
-    if ([NSApp isOnLeopardOrBetter])
-        cell = (TorrentCell *)[self preparedCellAtColumn: -1 row: row];
-    else
-    {
-        cell = fTorrentCell;
-        [cell setRepresentedObject: [self itemAtRow: row]];
-    }
-    return NSPointInRect(point, [cell progressRectForBounds: [self rectOfRow: row]]);
-}
-
-- (BOOL) pointInMinimalStatusRect: (NSPoint) point
-{
-    int row = [self rowAtPoint: point];
-    if (row < 0 || ![[self itemAtRow: row] isKindOfClass: [Torrent class]] || ![fDefaults boolForKey: @"SmallView"])
-        return NO;
-    
-    TorrentCell * cell;
-    if ([NSApp isOnLeopardOrBetter])
-        cell = (TorrentCell *)[self preparedCellAtColumn: -1 row: row];
-    else
-    {
-        cell = fTorrentCell;
-        [cell setRepresentedObject: [self itemAtRow: row]];
-    }
-    return NSPointInRect(point, [cell minimalStatusRectForBounds: [self rectOfRow: row]]);
-}
-
 - (BOOL) pointInGroupStatusRect: (NSPoint) point
 {
-    int row = [self rowAtPoint: point];
+    NSInteger row = [self rowAtPoint: point];
     if (row < 0 || [[self itemAtRow: row] isKindOfClass: [Torrent class]])
         return NO;
     
@@ -855,7 +898,7 @@
     
     //change size of image column
     NSTableColumn * ulImageTableColumn = [self tableColumnWithIdentifier: @"UL Image"];
-    float oldWidth = [ulImageTableColumn width], newWidth = ratio ? GROUP_RATIO_IMAGE_COLUMN_WIDTH : GROUP_SPEED_IMAGE_COLUMN_WIDTH;
+    CGFloat oldWidth = [ulImageTableColumn width], newWidth = ratio ? GROUP_RATIO_IMAGE_COLUMN_WIDTH : GROUP_SPEED_IMAGE_COLUMN_WIDTH;
     if (oldWidth != newWidth)
     {
         [ulImageTableColumn setWidth: newWidth];
@@ -867,16 +910,14 @@
 
 - (void) createFileMenu: (NSMenu *) menu forFiles: (NSArray *) files
 {
-    NSEnumerator * enumerator = [files objectEnumerator];
-    NSDictionary * dict;
-    while ((dict = [enumerator nextObject]))
+    for (FileListNode * node in files)
     {
-        NSString * name = [dict objectForKey: @"Name"];
+        NSString * name = [node name];
         
         NSMenuItem * item = [[NSMenuItem alloc] initWithTitle: name action: @selector(checkFile:) keyEquivalent: @""];
         
         NSImage * icon;
-        if (![[dict objectForKey: @"IsFolder"] boolValue])
+        if (![node isFolder])
             icon = [[NSWorkspace sharedWorkspace] iconForFileType: [name pathExtension]];
         else
         {
@@ -889,13 +930,13 @@
             icon = [[NSWorkspace sharedWorkspace] iconForFileType: NSFileTypeForHFSTypeCode('fldr')];
         }
         
-        [item setRepresentedObject: dict];
+        [item setRepresentedObject: node];
         
         [icon setScalesWhenResized: YES];
         [icon setSize: NSMakeSize(16.0, 16.0)];
         [item setImage: icon];
         
-        NSIndexSet * indexSet = [dict objectForKey: @"Indexes"];
+        NSIndexSet * indexSet = [node indexes];
         [item setState: [fMenuTorrent checkForFiles: indexSet]];
         [item setEnabled: [fMenuTorrent canChangeDownloadCheckForFiles: indexSet]];
         
@@ -904,27 +945,4 @@
     }
 }
 
-- (void) resizePiecesBarIncrement
-{
-    BOOL done;
-    if ([fDefaults boolForKey: @"PiecesBar"])
-    {
-        fPiecesBarPercent = MIN(fPiecesBarPercent + PIECE_CHANGE, 1.0);
-        done = fPiecesBarPercent == 1.0;
-    }
-    else
-    {
-        fPiecesBarPercent = MAX(fPiecesBarPercent - PIECE_CHANGE, 0.0);
-        done = fPiecesBarPercent == 0.0;
-    }
-    
-    if (done)
-    {
-        [fPiecesBarTimer invalidate];
-        fPiecesBarTimer = nil;
-    }
-    
-    [self reloadData];
-}
-
 @end
