@@ -33,20 +33,21 @@ function testSafari3()
 };
 
 $(document).ready( function() {
-	// Initialise a torrent controller to handle events
-	
 	// Initialise the dialog controller
 	dialog = new Dialog();
 	
 	// Initialise the main Transmission controller
 	transmission = new Transmission();
 
+	// IE specific fixes here
+	if ($.browser.msie) {
+		try {
+		  document.execCommand("BackgroundImageCache", false, true);
+		} catch(err) {}
+		$('.dialog_container').css('height',$(window).height()+'px');
+	}
+
 	if ($.browser.safari) {
-		
-		// Fix div height problem - causes scrollbar flash in
-		// firefox so have to be safari-specific
-		$('#torrent_inspector').css('height', '100%');
-		
 		// Move search field's margin down for the styled input
 		$('#torrent_search').css('margin-top', 3);		
 	}
@@ -69,11 +70,15 @@ $(document).ready( function() {
 		// different padding.
 		$('div.dialog_container div.dialog_window a').css('padding', '2px 10px 3px');
 	}
-    if (iPhone)
-        if(window.navigator.standalone)
-            // Fix min height for iPhone when run in full screen mode from home screen
-            // so the footer appears in the right place
-            $('body div#torrent_container').css('min-height', '338px');
+	if (iPhone){
+		window.onload = function(){ setTimeout(function() { window.scrollTo(0,1); },500); };
+		window.onorientationchange = function(){ setTimeout( function() { window.scrollTo(0,1); },100); };
+		if(window.navigator.standalone)
+			// Fix min height for iPhone when run in full screen mode from home screen
+			// so the footer appears in the right place
+			$('body div#torrent_container').css('min-height', '338px');
+		$("label[for=torrent_upload_url]").text("URL: ");
+	}
 });
 
 /*
@@ -88,12 +93,18 @@ Array.prototype.clone = function () {
 /**
  * "innerHTML = html" is pretty slow in FF.  Happily a lot of our innerHTML
  * changes are triggered by periodic refreshes on torrents whose state hasn't
- * changed sine the last update, so even this simple test helps a lot.
+ * changed since the last update, so even this simple test helps a lot.
  */
 function setInnerHTML( e, html )
 {
-	if( e.innerHTML != html )
+	/* innerHTML is listed as a string, but the browser seems to change it.
+	 * For example, "&infin;" gets changed to "∞" somewhere down the line.
+	 * So, let's use an arbitrary  different field to test our state... */
+	if( e.currentHTML != html )
+	{
+		e.currentHTML = html;
 		e.innerHTML = html;
+	}
 };
 
 /*
@@ -211,7 +222,7 @@ Math.roundWithPrecision = function(floatnum, precision) {
  */
 Math.ratio = function( numerator, denominator )
 {
-	var result = Math.roundWithPrecision((numerator / denominator), 2);
+	var result = Math.floor(100 * numerator / denominator) / 100;
 
 	// check for special cases
 	if (isNaN(result)) result = 0;
